 /*
 * Copyright 1998-2011 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2011 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "via_chrome9_drv.h"
#include "via_chrome9_external_card.h"
#include "drm_crtc.h"

extern struct via_cea_timing_info_rec via_cea_timing_table[];
extern struct display_timing_table via_lcd_modes[];

/* Support Mode, the order must follow the rule "small to big". */
struct via_chrome9_support_mode via_chrome9_spport_mode[] = {
	/*  Index,         HActive, VActive*/
	{"480x640",     M480x640,     480,     640},
	{"640x480",     M640x480,     640,     480},
	{"720x480",     M720x480,     720,     480},
	{"720x576",     M720x576,     720,     576},
	{"800x480",     M800x480,     800,     480},
	{"800x600",     M800x600,     800,     600},
	{"848x480",     M848x480,     848,     480},
	{"852x480",     M852x480,     856,     480},
	{"960x600",     M960x600,     960,     600},
	{"1000x600",    M1000x600,    1000,    600},
	{"1024x512",    M1024x512,    1024,    512},
	{"1024x600",    M1024x600,    1024,    600},
	{"1024x576",    M1024x576,    1024,    576},
	{"1024x768",    M1024x768,    1024,    768},
	{"1088x612",    M1088x612,    1088,    612},
	{"1152x720",    M1152x720,    1152,    720},
	{"1152x864",    M1152x864,    1152,    864},
	{"1200x720",    M1200x720,    1200,    720},
	{"1280x600",    M1280x600,    1280,    600},
	{"1280x720",    M1280x720,    1280,    720},
	{"1280x768",    M1280x768,    1280,    768},
	{"1280x800",    M1280x800,    1280,    800},
	{"1280x960",    M1280x960,    1280,    960},
	{"1280x1024",   M1280x1024,   1280,    1024},
	{"1360x768",    M1360x768,    1360,    768},
	{"1366x768",    M1366x768,    1366,    768},
	{"1368x768",	M1368x768,	  1368,    768},
	{"1400x1050",   M1400x1050,   1400,    1050},
	{"1440x900",    M1440x900,    1440,    900},
	{"1440x1050",   M1440x1050,   1440,    1050},
	{"1600x900",    M1600x900,    1600,    900},
	{"1600x1024",   M1600x1024,   1600,    1024},
	{"1600x1200",   M1600x1200,   1600,    1200},
	{"1680x1050",   M1680x1050,   1680,    1050},
	{"1792x1344",   M1792x1344,   1792,    1344},
	{"1856x1392",   M1856x1392,   1856,    1392},
	{"1920x1080",   M1920x1080,   1920,    1080},
	{"1920x1200",   M1920x1200,   1920,    1200},
	{"1920x1440",   M1920x1440,   1920,    1440},
	{"2048x1536",   M2048x1536,   2048,    1536},
	{"720x400",     M720x400,     720,     400},
	{"832x624",     M832x624,     832,     624},
	{"",            -1,    0,       0}
};
static struct drm_display_mode via_lcd_video_modes[] = {
	{ DRM_MODE("480x640", DRM_MODE_TYPE_DRIVER,
		DCLK_22_000M, 480, 504,
		512, 520, 0, 640, 644, 646, 648, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_PVSYNC)},
	{ DRM_MODE("640x480", DRM_MODE_TYPE_DRIVER,
		DCLK_25_175M, 640, 656,
		752, 800, 0, 480, 490, 492, 525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC)},
	{ DRM_MODE("800x480", DRM_MODE_TYPE_DRIVER,
		DCLK_29_581M, 800, 824,
		896, 992, 0, 480, 483, 490, 500, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_PVSYNC)},
	{ DRM_MODE("800x600", DRM_MODE_TYPE_DRIVER,
		DCLK_40_000M, 800, 840,
		968, 1056, 0, 600, 601, 605, 628, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC)},
	{ DRM_MODE("1024x600", DRM_MODE_TYPE_DRIVER,
		DCLK_48_875M, 1024, 1064,
		1168, 1312, 0, 600, 601, 604, 622, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_PVSYNC)},
	{ DRM_MODE("1024x768", DRM_MODE_TYPE_DRIVER,
		DCLK_56_250M, 1024, 1072,
		1104, 1184, 0, 768, 771, 775, 790, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_NVSYNC)},
	{ DRM_MODE("1280x768", DRM_MODE_TYPE_DRIVER,
		DCLK_80_136M, 1280, 1344,
		1480, 1680, 0, 768, 769, 772, 795, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_PVSYNC)},
	{ DRM_MODE("1280x800", DRM_MODE_TYPE_DRIVER,
		DCLK_72_000M, 1280, 1328,
		1360, 1440, 0, 800, 803, 809, 823, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_NVSYNC)},
	{ DRM_MODE("1280x1024", DRM_MODE_TYPE_DRIVER,
		DCLK_108_000M, 1280, 1328,
		1440, 1688, 0, 1024, 1025, 1028, 1066, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC)},
	{ DRM_MODE("1366x768", DRM_MODE_TYPE_DRIVER,
		DCLK_85_860M, 1368, 1440,
		1584, 1800, 0, 768, 769, 772, 795, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_PVSYNC)},
#if 0
	/**
	 * 1366x768 timing supported by DMTv1r12.pdf .
	 * Prepare for 1366x768 IGA2 LCD complete support .
	 */
	{ DRM_MODE("1366x768", DRM_MODE_TYPE_DRIVER, DCLK_85_860M, 1366, 1436,
		1579, 1792, 0, 768, 771, 774, 798, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC)},
#endif
	{ DRM_MODE("1360x768", DRM_MODE_TYPE_DRIVER,
		DCLK_84_750M, 1360, 1432,
		1568, 1776, 0, 768, 771, 776, 798, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC)},
	{ DRM_MODE("1400x1050", DRM_MODE_TYPE_DRIVER,
		DCLK_101_000M, 1400, 1448,
		1480, 1560, 0, 1050, 1053, 1057, 1080, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_NVSYNC)},
	{ DRM_MODE("1440x900", DRM_MODE_TYPE_DRIVER,
		DCLK_88_750M, 1440, 1495,
		1600, 1760, 0, 900, 901, 903, 912, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_NVSYNC)},
	{ DRM_MODE("1600x1200", DRM_MODE_TYPE_DRIVER,
		DCLK_130_250M, 1600, 1648,
		1680, 1760, 0, 1200, 1203, 1207, 1235, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_NVSYNC)},
	{ DRM_MODE("1920x1080", DRM_MODE_TYPE_DRIVER,
		DCLK_150_340M, 1920, 1951,
		1999, 2120, 0, 1080, 1082, 1088, 1120, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC)}
};

#define VIA_LCD_VIDEO_MODES_NUM	15

static struct drm_display_mode via_cea_video_modes[] = {
	{ DRM_MODE("640x480", DRM_MODE_TYPE_DRIVER,
		DCLK_25_200M, 640, 656,
		752, 800, 1, 480,  490,  492,  525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("720x480P_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_27_000M, 720, 736,
		798, 858, 2, 480,  489,  495,  525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("720x480P_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_27_000M, 720, 736,
		798, 858, 3, 480,  489,  495,  525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("1280x720P", DRM_MODE_TYPE_DRIVER,
		DCLK_74_250M, 1280, 1390,
		1430, 1650, 4, 720,  725,  730,  750, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC) },
	{ DRM_MODE("1920x1080I", DRM_MODE_TYPE_DRIVER,
		DCLK_74_250M, 1920, 2008,
		2052, 2200, 5, 1080, 1084, 1094, 1125, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("720x480I_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_27_000M, 1440, 1478,
		1602, 1716, 6, 480, 488, 494, 525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("720x480I_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_27_000M, 1440, 1478,
		1602, 1716, 7, 480, 488, 494, 525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("720x240P_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_27_000M, 1440, 1478,
		1602, 1716, 8, 240, 244, 247, 262, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("720x240P_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_27_000M, 1440, 1478,
		1602, 1716, 9, 240, 244, 247, 262, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("2880x480I_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 2880, 2956,
		3204, 3432, 10, 480, 488, 494, 525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("2880x480I_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 2880, 2956,
		3204, 3432, 11, 480, 488, 494, 525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("2880x240P_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 2880, 2956,
		3204, 3432, 12, 240, 244, 247, 262, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("2880x240P_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 2880, 2956,
		3204, 3432, 13, 240, 244, 247, 262, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("1440x480P_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 1440, 1472,
		1596, 1716, 14, 480, 489, 495, 525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("1440x480P_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 1440, 1472,
		1596, 1716, 15, 480, 489, 495, 525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("1920x1080P", DRM_MODE_TYPE_DRIVER,
		DCLK_148_500M, 1920, 2008,
		2052, 2200, 16, 1080, 1084, 1089, 1125, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC) },
	{ DRM_MODE("720x576P_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_27_000M, 720, 732,
		796, 864, 17, 576,  581,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("720x576P_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_27_000M, 720, 732,
		796, 864, 18, 576,  581,  586,  625, 0,
		   DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("1280x720P", DRM_MODE_TYPE_DRIVER,
		DCLK_74_250M, 1280, 1720,
		1760, 1980, 19, 720,  725,  730,  750, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC) },
	{ DRM_MODE("1920x1080I", DRM_MODE_TYPE_DRIVER,
		DCLK_74_250M, 1920, 2448,
		2492, 2640, 20, 1080,  1084,  1094,  1125, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("720x576I_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_27_000M, 1440, 1464,
		1590, 1728, 21, 576,  580,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("720x576I_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_27_000M, 1440, 1464,
		1590, 1728, 22, 576,  580,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("720x288P_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_27_000M, 1440, 1464,
		1590, 1728, 23, 288,  290,  293,  312, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("720x288P_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_27_000M, 1440, 1464,
		1590, 1728, 24, 288,  290,  293,  312, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("2880x576I_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 2880, 2928,
		3180, 3456, 25, 576,  580,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("2880x576I_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 2880, 2928,
		3180, 3456, 26, 576,  580,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("2880x288P_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 2880, 2928,
		3180, 3456, 27, 288,  290,  293,  312, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("2880x288P_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 2880, 2928,
		3180, 3456, 28, 288,  290,  293,  312, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("1440x576P_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 1440, 1464,
		1592, 1728, 29, 576,  581,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("1440x576P_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 1440, 1464,
		1592, 1728, 30, 576,  581,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("1920x1080P", DRM_MODE_TYPE_DRIVER,
		DCLK_148_500M, 1920, 2448,
		2492, 2640, 31, 1080,  1084,  1089,  1125, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC) },
	{ DRM_MODE("1920x1080P", DRM_MODE_TYPE_DRIVER,
		DCLK_74_250M, 1920, 2558,
		2602, 2750, 32, 1080,  1084,  1089,  1125, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC) },
	{ DRM_MODE("1920x1080P", DRM_MODE_TYPE_DRIVER,
		DCLK_74_250M, 1920, 2448,
		2492, 2640, 33, 1080,  1084,  1089,  1125, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC) },
	{ DRM_MODE("1920x1080P", DRM_MODE_TYPE_DRIVER,
		DCLK_74_250M, 1920, 2008,
		2052, 2200, 34, 1080,  1084,  1089,  1125, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC) },
	{ DRM_MODE("2880x480P_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_108_000M, 2880, 2944,
		3192, 3432, 35, 480,  489,  495,  525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("2880x480P_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_108_000M, 2880, 2944,
		3192, 3432, 36, 480,  489,  495,  525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("2880x576P_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_108_000M, 2880, 2928,
		3184, 3456, 37, 576,  581,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("2880x576P_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_108_000M, 2880, 2928,
		3184, 3456, 38, 576,  581,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("1920x1080I", DRM_MODE_TYPE_DRIVER,
		DCLK_72_000M, 1920, 1952,
		2120, 2304, 39, 1080,  1126,  1136,  1250, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("1920x1080I", DRM_MODE_TYPE_DRIVER,
		DCLK_148_500M, 1920, 2448,
		2492, 2640, 40, 1080,  1084,  1094,  1125, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("1280x720P", DRM_MODE_TYPE_DRIVER,
		DCLK_148_500M, 1280, 1720,
		1760, 1980, 41, 720,  725,  730,  750, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC) },
	{ DRM_MODE("720x576P_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 720, 732,
		796, 864, 42, 576,  581,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("720x576P_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 720, 732,
		796, 864, 43, 576,  581,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("720x576I_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 1440, 1464,
		1590, 1728, 44, 576,  580,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("720x576I_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 1440, 1464,
		1590, 1728, 45, 576,  580,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("1920x1080I", DRM_MODE_TYPE_DRIVER,
		DCLK_148_500M, 1920, 2008,
		2052, 2200, 46, 1080,  1084,  1094,  1125, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("1280x720P", DRM_MODE_TYPE_DRIVER,
		DCLK_148_500M, 1280, 1390,
		1430, 1650, 47, 720,  725,  730,  750, 0,
		DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_PVSYNC) },
	{ DRM_MODE("720x480P_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 720, 736,
		798, 858, 48, 480,  489,  495,  525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("720x480P_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 720, 736,
		798, 858, 49, 480,  489,  495,  525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("720x480I_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 1440, 1478,
		1602, 1716, 50, 480,  488,  494,  525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("720x480I_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_54_000M, 1440, 1478,
		1602, 1716, 51, 480,  488,  494,  525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("720x576P_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_108_000M, 720, 732,
		796, 864, 52, 576,  581,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("720x576P_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_108_000M, 720, 732,
		796, 864, 53, 576,  581,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("720x576I_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_108_000M, 1440, 1464,
		1590, 1728, 54, 576,  580,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("720x576I_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_108_000M, 1440, 1464,
		1590, 1728, 55, 576,  580,  586,  625, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("720x480P_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_108_000M, 720, 736,
		798, 858, 56, 480,  489,  495,  525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("720x480P_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_108_000M, 720, 736,
		798, 858, 57, 480,  489,  495,  525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC) },
	{ DRM_MODE("720x480I_4_3", DRM_MODE_TYPE_DRIVER,
		DCLK_108_000M, 1440, 1478,
		1602, 1716, 58, 480,  488,  494,  525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) },
	{ DRM_MODE("720x480I_16_9", DRM_MODE_TYPE_DRIVER,
		DCLK_108_000M, 1440, 1478,
		1602, 1716, 59, 480,  488,  494,  525, 0,
		DRM_MODE_FLAG_NHSYNC | DRM_MODE_FLAG_NVSYNC |
		DRM_MODE_FLAG_INTERLACE) }
};

#define VIA_CEA_VIDEO_MODES_NUM	59

const int via_chrome9_connector_convert[] = {
	DRM_MODE_CONNECTOR_Unknown,
	/* VIA_CHROME9_CONNECTOR_CRT_INDEX */
	DRM_MODE_CONNECTOR_VGA,
	/* VIA_CHROME9_CONNECTOR_LCD_INDEX */
	DRM_MODE_CONNECTOR_LVDS,
	/* VIA_CHROME9_CONNECTOR_DVI_D_INDEX */
	DRM_MODE_CONNECTOR_DVID,
	/* VIA_CHROME9_CONNECTOR_DP_1_INDEX */
	DRM_MODE_CONNECTOR_DisplayPort,
	/* VIA_CHROME9_CONNECTOR_DP_2_INDEX */
	DRM_MODE_CONNECTOR_DisplayPort,
	/* VIA_CHROME9_CONNECTOR_TV_INDEX */
	DRM_MODE_CONNECTOR_SVIDEO,
	/* VIA_CHROME9_CONNECTOR_HDMI_INDEX */
	DRM_MODE_CONNECTOR_HDMIA,
	DRM_MODE_CONNECTOR_Unknown,
};

static void via_chrome9_helper_set_mode(struct drm_encoder  *encoder)
{
	struct drm_crtc *crtc = encoder->crtc;

	KMS_DEBUG("\n");
	if (crtc && crtc->enabled) {
		drm_crtc_helper_set_mode(crtc, &crtc->mode,
			crtc->x, crtc->y, crtc->fb);
	}
}

static void via_chrome9_property_change_mode(struct drm_encoder  *encoder)
{
	KMS_DEBUG("\n");
	via_chrome9_helper_set_mode(encoder);
}

bool via_chrome9_detect_device_by_read_edid(struct drm_connector *connector,
		u8 device_type)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	bool ret = false;
	u8 *raw_edid;

	if (via_chrome9_connector->ddc_bus) {
		/* for hotplug to get newest edid of the connector */
		kfree(via_chrome9_connector->edid);
		via_chrome9_connector->edid = NULL;
		via_chrome9_connector->edid =
			drm_get_edid(&via_chrome9_connector->base,
				&via_chrome9_connector->ddc_bus->adapter);
		if (!via_chrome9_connector->edid) {
			KMS_DEBUG("%s: probed a monitor but no|invalid EDID\n",
				drm_get_connector_name(connector));
			ret = false;
		} else {
			raw_edid = (u8 *)via_chrome9_connector->edid;

			if ((0x00 == raw_edid[0]) && (0xff == raw_edid[1]) &&
				(device_type == ((raw_edid[20] & 0x80) >> 7)))
				ret = true;
		}
	} else {
		KMS_DEBUG("there is no ddc bus for this connector %s:\n",
			drm_get_connector_name(connector));
		ret = false;
	}
	return ret;
}

void via_chrome9_get_monitor_physical_size(struct drm_connector *connector,
		u32 *width, u32 *height)
{
	struct list_head *mode_list =  &connector->probed_modes;
	struct drm_display_mode *mode, *t;

	*width = 0;
	*height = 0;
	list_for_each_entry_safe(mode, t, mode_list, head) {
		if (mode->hdisplay > *width)
			*width = mode->hdisplay;

		if (mode->vdisplay > *height)
			*height = mode->vdisplay;
	}

	if (0 == *width || 0 == *height) {
		*width = 1024;
		*height = 768;
	}
}

/* Search Valid mode */
bool via_chrome9_is_valid_mode(u16 hdisplay, u16 vdisplay)
{
	u16 i = 0;

	while (via_chrome9_spport_mode[i].modeindex > -1) {
		if ((hdisplay == via_chrome9_spport_mode[i].hactive) &&
			(vdisplay == via_chrome9_spport_mode[i].vactive))
			return true;

		i++;
	}
	return false;
}

static void via_chrome9_connector_destory(struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);

	kfree(via_chrome9_connector->edid);
	via_chrome9_connector->edid = NULL;
	kfree(via_chrome9_connector->connector_private);
	via_chrome9_connector->connector_private = NULL;
	kfree(via_chrome9_connector->options);
	via_chrome9_connector->options = NULL;
	kfree(via_chrome9_connector->connector_info);
	via_chrome9_connector->connector_info = NULL;

	drm_sysfs_connector_remove(connector);
	drm_connector_cleanup(connector);
	kfree(connector);
	KMS_DEBUG("via_chrome9_connector_destory %s\n",
		drm_get_connector_name(connector));
}

struct drm_encoder *via_chrome9_best_single_encoder(
		struct drm_connector *connector)
{
	int enc_id = connector->encoder_ids[0];
	struct drm_mode_object *obj;
	struct drm_encoder *encoder;
	/* pick the encoder ids */
	if (enc_id) {
		obj = drm_mode_object_find(connector->dev, enc_id,
			DRM_MODE_OBJECT_ENCODER);
		if (!obj) {
			KMS_DEBUG("can not find the encoder\n");
			return NULL;
		}
		encoder = obj_to_encoder(obj);
			return encoder;
	}
	KMS_DEBUG("there is no encoder for this connector\n");
	return NULL;
}

int via_chrome9_ddc_get_modes(
		struct via_chrome9_connector *via_chrome9_connector)
{
	int ret;

	if (!via_chrome9_connector->ddc_bus) {
		KMS_DEBUG("there is no ddc bus for this connector!\n");
		return -1;
	}

	/* this connector may force status */
	if (!via_chrome9_connector->edid) {
		via_chrome9_connector->edid =
			drm_get_edid(&via_chrome9_connector->base,
				&via_chrome9_connector->ddc_bus->adapter);
	}

	if (via_chrome9_connector->edid) {
		drm_mode_connector_update_edid_property(
			&via_chrome9_connector->base,
			via_chrome9_connector->edid);
		ret = drm_add_edid_modes(&via_chrome9_connector->base,
			via_chrome9_connector->edid);
		return ret;
	}

	return 0;
}

void via_chrome9_set_vga_xorg_options(void *options, int size,
		struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_vga_options *via_vga_options =
		(struct via_vga_options *)via_chrome9_connector->options;
	struct drm_connector_helper_funcs *connector_func =
					connector->helper_private;
	struct via_chrome9_external_card *card = NULL;
	struct via_gfx_dpa via_gfx_dpa;
	struct drm_encoder *encoder = connector_func->best_encoder(connector);

	copy_from_user(via_vga_options, options, size);

	if (encoder) {
		card = to_via_chrome9_external_card(encoder);
		if (card) {
			via_gfx_dpa.clk_adjust = via_vga_options->clk_adjust;
			via_gfx_dpa.clk_driving_sel =
				via_vga_options->clk_driving_sel;
			via_gfx_dpa.clk_polarity =
				via_vga_options->clk_polarity;
			via_gfx_dpa.data_driving_sel =
				via_vga_options->data_driving_sel;
			via_gfx_dpa.is_clk_adjust_used =
				via_vga_options->is_clk_adjust_used;
			via_gfx_dpa.is_clk_driving_sel_used =
				via_vga_options->is_clk_driving_sel_used;
			via_gfx_dpa.is_clk_polarity_used =
				via_vga_options->is_clk_polarity_used;
			via_gfx_dpa.is_data_driving_sel_used =
				via_vga_options->is_data_driving_sel_used;
			via_chrome9_load_user_gfx_dpa_setting(
				card->di_port, &via_gfx_dpa);
		}
	}

	KMS_DEBUG("struct via_dvi_options size is %d", size);
	KMS_DEBUG("via_vga_options->clk_polarity=%d\n",
		via_vga_options->clk_polarity);
	KMS_DEBUG("via_vga_options->clk_adjust=%d\n",
		via_vga_options->clk_adjust);
	KMS_DEBUG("via_vga_options->clk_driving_sel=%d\n",
		via_vga_options->clk_driving_sel);
	KMS_DEBUG("via_vga_options->data_driving_sel=%d\n",
		via_vga_options->data_driving_sel);
	KMS_DEBUG("via_vga_options->no_ddc_value=%d\n",
		via_vga_options->no_ddc_value);
}

int via_vga_get_modes(struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	int ret = 0;
	struct via_vga_options *via_vga_options =
		(struct via_vga_options *)via_chrome9_connector->options;
	KMS_DEBUG("\n");
	if (!via_vga_options->no_ddc_value) {
		ret = via_chrome9_ddc_get_modes(via_chrome9_connector);
	} else {
		kfree(via_chrome9_connector->edid);
		via_chrome9_connector->edid = NULL;
		connector->display_info.raw_edid = NULL;
	}
	return ret;
}

int via_vga_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode)
{
	/* Check Clock Range */
	if (mode->clock > 400000) {
		KMS_DEBUG("mode clock is high\n");
		return MODE_CLOCK_HIGH;
	}
	if (mode->clock < 25000) {
		KMS_DEBUG("mode clock is low\n");
		return MODE_CLOCK_LOW;
	}

	/* Check Resolution Range */
	if (!via_chrome9_is_valid_mode(mode->hdisplay, mode->vdisplay)) {
		KMS_DEBUG("mode is not valid\n");
		return MODE_BAD;
	}

	return MODE_OK;
}


/* detect this connector connect status */
static enum drm_connector_status via_chrome9_vga_detect(
		struct drm_connector *connector,  bool force)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct drm_encoder *encoder;
	struct drm_encoder_helper_funcs *encoder_funcs;
	struct via_vga_options *via_vga_options =
		(struct via_vga_options *)via_chrome9_connector->options;
	enum drm_connector_status ret = connector_status_disconnected;
	bool result = false;
	KMS_DEBUG("\n");
	encoder = via_chrome9_best_single_encoder(connector);
	if (!encoder) {
		ret = connector_status_disconnected;
		KMS_DEBUG("there is no best encoder for this connector\n");
		return ret;
	}

	if (!via_vga_options->no_ddc_value) {
		result = via_chrome9_detect_device_by_read_edid(
			connector, ANALOG_DEVICE);
	} else {
		kfree(via_chrome9_connector->edid);
		via_chrome9_connector->edid = NULL;
		connector->display_info.raw_edid = NULL;
	}

	if (result) {
		ret = connector_status_connected;
	} else {
		encoder_funcs = encoder->helper_private;
		if (encoder_funcs->detect)
			ret = encoder_funcs->detect(encoder, connector);
	}

	return ret;
}

static int via_chrome9_vga_set_property(struct drm_connector *connector,
		struct drm_property *property,
		uint64_t value)
{
	/* implement later if necessary */
	return 0;
}

static int via_chrome9_vga_get_modes(struct drm_connector *connector)
{

	return via_vga_get_modes(connector);
}

static int via_chrome9_vga_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode)
{
	return via_vga_mode_valid(connector, mode);
}

static struct drm_connector_funcs via_chrome9_vga_connector_funcs = {
	.dpms = drm_helper_connector_dpms,
	.detect = via_chrome9_vga_detect,
	.fill_modes = drm_helper_probe_single_connector_modes,
	.destroy = via_chrome9_connector_destory,
	.set_property = via_chrome9_vga_set_property,
};

static struct drm_connector_helper_funcs
	via_chrome9_vga_connector_helper_funcs = {
	.get_modes = via_chrome9_vga_get_modes,
	.mode_valid = via_chrome9_vga_mode_valid,
	.best_encoder = via_chrome9_best_single_encoder,
};


/* Get a build-in mode record according to panel id. */
int via_chrome9_get_panel_mode_record(struct drm_connector *connector,
		u32 panel_index)
{
	struct drm_device *dev = connector->dev;
	struct drm_display_mode *new_mode = NULL;
	int ret = 0;
	int i = 0;

	for (i = 0; i < VIA_LCD_VIDEO_MODES_NUM; i++) {
		if (panel_index == via_lcd_modes[i].mode_index) {
			new_mode = drm_mode_create(dev);
			if (new_mode) {
				memcpy(new_mode->name,
					via_lcd_video_modes[i].name,
					DRM_DISPLAY_MODE_LEN);
				/* When we using panel 1366x768,
				want to light mode 1366x768
				but in our mode list we just have 1368x768,
				so change the name to 1366x768 */

				if (1368 == new_mode->hdisplay &&
					768 == new_mode->vdisplay)
					memcpy(new_mode->name, "1366x768",
						DRM_DISPLAY_MODE_LEN);

				new_mode->type = via_lcd_video_modes[i].type;
				new_mode->clock = via_lcd_video_modes[i].clock;
				new_mode->hdisplay =
					via_lcd_video_modes[i].hdisplay;
				new_mode->hsync_start =
					via_lcd_video_modes[i].hsync_start;
				new_mode->hsync_end =
					via_lcd_video_modes[i].hsync_end;
				new_mode->htotal =
					via_lcd_video_modes[i].htotal;
				new_mode->vdisplay =
					via_lcd_video_modes[i].vdisplay;
				new_mode->vsync_start =
					via_lcd_video_modes[i].vsync_start;
				new_mode->vsync_end =
					via_lcd_video_modes[i].vsync_end;
				new_mode->vtotal =
					via_lcd_video_modes[i].vtotal;
				new_mode->vscan = via_lcd_video_modes[i].vscan;
				new_mode->flags = via_lcd_video_modes[i].flags;
				drm_mode_probed_add(connector, new_mode);
				ret++;
			}
		}
	}
	return ret;
}

void via_chrome9_set_lcd_xorg_options(void *options, int size,
		struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_lcd_options *via_lcd_options =
		(struct via_lcd_options *)via_chrome9_connector->options;
	struct drm_connector_helper_funcs *connector_func =
		connector->helper_private;
	struct via_chrome9_external_card *card = NULL;
	struct via_gfx_dpa via_gfx_dpa;
	struct via_lvds_dpa via_lvds_dpa;
	struct drm_encoder *encoder = connector_func->best_encoder(connector);
	bool is_dual_channel_bak = false;
	u32 physical_width_bak = 0, physical_height_bak = 0;

	KMS_DEBUG("\n");

	is_dual_channel_bak = via_lcd_options->dual_channel;
	physical_width_bak = via_lcd_options->physical_width;
	physical_height_bak = via_lcd_options->physical_height;

	copy_from_user(via_lcd_options, options, size);

	KMS_DEBUG("physical_old = %dx%d , physical_new = %dx%d,\n",
		physical_width_bak, physical_height_bak,
		via_lcd_options->physical_width,
		via_lcd_options->physical_height);

	/**
	 *  when we use EDID to light LCD, we won't set PanelSize option.
	 *  so here, via_lcd_options->physical_width will be 0,
	 *  it will recover the real panel physical size
	 *  which analized from EDID.
	 *  so we should bak and restore the real size.
	 */
	if ((via_lcd_options->physical_width == 0) ||
		(via_lcd_options->physical_height == 0)) {
		via_lcd_options->physical_width = physical_width_bak ;
		via_lcd_options->physical_height = physical_height_bak;
	}

	if (encoder) {

		if (via_lcd_options->fix_on_iga1)
			encoder->possible_crtcs = 0x1;
		else
			encoder->possible_crtcs = 0x2;

		card = to_via_chrome9_external_card(encoder);
		if (card) {
			via_gfx_dpa.clk_adjust =
				via_lcd_options->clk_adjust;
			via_gfx_dpa.clk_driving_sel =
				via_lcd_options->clk_driving_sel;
			via_gfx_dpa.clk_polarity =
				via_lcd_options->clk_polarity;
			via_gfx_dpa.data_driving_sel =
				via_lcd_options->data_driving_sel;
			via_gfx_dpa.is_clk_adjust_used =
				via_lcd_options->is_clk_adjust_used;
			via_gfx_dpa.is_clk_driving_sel_used =
				via_lcd_options->is_clk_driving_sel_used;
			via_gfx_dpa.is_clk_polarity_used =
				via_lcd_options->is_clk_polarity_used;
			via_gfx_dpa.is_data_driving_sel_used =
				via_lcd_options->is_data_driving_sel_used;

			via_lvds_dpa.is_vt_1636_clk_sel_st1_used =
				via_lcd_options->is_vt_1636_clk_sel_st1_used;
			via_lvds_dpa.is_vt_1636_clk_sel_st2_used =
				via_lcd_options->is_vt_1636_clk_sel_st2_used;
			via_lvds_dpa.vt1636_clk_sel_st1 =
				via_lcd_options->vt1636_clk_sel_st1;
			via_lvds_dpa.vt1636_clk_sel_st2 =
				via_lcd_options->vt1636_clk_sel_st2;
			via_chrome9_load_user_gfx_dpa_setting(card->di_port,
				&via_gfx_dpa);
			via_chrome9_load_user_lvds_dpa_setting(card,
				&via_lvds_dpa);
		}

		via_chrome9_lvds_no_dithering_set(
			via_lcd_options->no_dithering);
		via_chrome9_lvds_msb_set(encoder, via_lcd_options->msb);
		if (is_dual_channel_bak != via_lcd_options->dual_channel) {
			KMS_DEBUG(
				"Dual/Single Channel changed! old = %d, now = %d\n",
				is_dual_channel_bak,
				via_lcd_options->dual_channel);
			via_chrome9_helper_set_mode(encoder);
		}
	}

	KMS_DEBUG("via_lcd_options->msb=%d\n",
		via_lcd_options->msb);
	KMS_DEBUG("via_lcd_options->center=%d\n",
		via_lcd_options->center);
	KMS_DEBUG("via_lcd_options->fix_on_iga1=%d\n",
		via_lcd_options->fix_on_iga1);
	KMS_DEBUG("via_lcd_options->hysical_width=%d\n",
		via_lcd_options->physical_width);
	KMS_DEBUG("via_lcd_options->hysical_height=%d\n",
		via_lcd_options->physical_height);
	KMS_DEBUG("via_lcd_options->panel_index=%d\n",
		via_lcd_options->panel_index);
	KMS_DEBUG("via_lcd_options->dual_channel=%d\n",
		via_lcd_options->dual_channel);
	KMS_DEBUG("via_lcd_options->no_dithering=%d\n",
		via_lcd_options->no_dithering);
}

int via_lcd_get_modes(struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_lcd_options *via_lcd_options =
		(struct via_lcd_options *)via_chrome9_connector->options;

	int ret = 0;

	if (via_lcd_options->panel_index) {
		ret = via_chrome9_get_panel_mode_record(connector,
			via_lcd_options->panel_index);
	} else {
		ret = via_chrome9_ddc_get_modes(via_chrome9_connector);
		via_chrome9_get_monitor_physical_size(connector,
			&via_lcd_options->physical_width,
			&via_lcd_options->physical_height);
	}
	return ret;
}

int via_lcd_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_lcd_options *via_lcd_options =
		(struct via_lcd_options *)via_chrome9_connector->options;
	struct drm_device *dev = connector->dev;
	int chipset = dev->pci_device;

	KMS_DEBUG("via_chrome9_lcd_mode_valid mode is: %dx%d\n",
		mode->hdisplay, mode->vdisplay);

	if (via_lcd_options) {
		if (via_lcd_options->physical_width &&
			via_lcd_options->physical_height) {
			/* Don't support mode larger than physical size */
			if (VX900_DEVICE != chipset) {
				if (mode->hdisplay >
					via_lcd_options->physical_width)
					return MODE_PANEL;
				if (mode->vdisplay >
					via_lcd_options->physical_height)
					return MODE_PANEL;
			} else {
				/* HW limitation 410 only can
				do <= 1.33 scaling */
				if (mode->hdisplay * 100 >
					via_lcd_options->physical_width * 133)
					return MODE_PANEL;
				if (mode->vdisplay * 100 >
					via_lcd_options->physical_height * 133)
					return MODE_PANEL;
				/* Now we can not support H V different scale */
				if ((mode->hdisplay >
					via_lcd_options->physical_width) &&
					(mode->vdisplay <
					via_lcd_options->physical_height))
					return MODE_PANEL;
				if ((mode->hdisplay <
					via_lcd_options->physical_width) &&
					(mode->vdisplay >
					via_lcd_options->physical_height))
					return MODE_PANEL;
			}
		}
	}
	KMS_DEBUG("via_chrome9_lcd_mode_valid mode is OK\n");
	return MODE_OK;

}

/* detect this connector connect status */
static enum drm_connector_status via_chrome9_lcd_detect(
		struct drm_connector *connector,  bool force)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_lcd_options *via_lcd_options =
		(struct via_lcd_options *)via_chrome9_connector->options;
	KMS_DEBUG("\n");
	if (via_lcd_options) {
		if (via_lcd_options->panel_index) {
			KMS_DEBUG("lcd connected panel_index is %d\n",
				via_lcd_options->panel_index);
			return connector_status_connected;
		} else {
			KMS_DEBUG("detect lcd by read edid\n");
			if (true == via_chrome9_detect_device_by_read_edid(
				connector, DIGITAL_DEVICE)) {
				KMS_DEBUG("lcd is connected\n");
				return connector_status_connected;
			}
		}
	}

	return connector_status_disconnected;
}

int via_chrome9_lcd_set_property(struct drm_connector *connector,
		struct drm_property *property,
		uint64_t value)
{
	/* implement later if necessary */
	return 0;
}

int via_chrome9_lcd_get_modes(struct drm_connector *connector)
{
	KMS_DEBUG("\n");
	return via_lcd_get_modes(connector);
}

int via_chrome9_lcd_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode)
{
	KMS_DEBUG("\n");
	return via_lcd_mode_valid(connector, mode);
}

static struct drm_connector_funcs via_chrome9_lcd_connector_funcs = {
	.dpms = drm_helper_connector_dpms,
	.detect = via_chrome9_lcd_detect,
	.fill_modes = drm_helper_probe_single_connector_modes,
	.destroy = via_chrome9_connector_destory,
	.set_property = via_chrome9_lcd_set_property,
};

static struct drm_connector_helper_funcs
	via_chrome9_lcd_connector_helper_funcs = {
	.get_modes = via_chrome9_lcd_get_modes,
	.mode_valid = via_chrome9_lcd_mode_valid,
	.best_encoder = via_chrome9_best_single_encoder,
};
void via_chrome9_set_hdmi_xorg_options(void *options, int size,
		struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_hdmi_options *via_hdmi_options =
		(struct via_hdmi_options *)via_chrome9_connector->options;
	struct drm_connector_helper_funcs *connector_func =
		connector->helper_private;
	struct i2c_client *client = NULL;
	struct via_chrome9_external_card *card = NULL;
	struct via_gfx_dpa via_gfx_dpa;
	struct drm_encoder *encoder = connector_func->best_encoder(connector);
	struct drm_device *dev = connector->dev;
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	/* bak old hdmi borders options. */
	u32 left_border_bak = via_hdmi_options->left_border;
	u32 right_border_bak = via_hdmi_options->right_border;
	u32 top_border_bak = via_hdmi_options->top_border;
	u32 bottom_border_bak = via_hdmi_options->bottom_border;

	copy_from_user(via_hdmi_options, options, size);
	KMS_DEBUG("\n");
	if (encoder) {
		card = to_via_chrome9_external_card(encoder);
		if (card) {
			client = via_chrome9_external_card_to_i2c_client(card);
			via_gfx_dpa.clk_adjust = via_hdmi_options->clk_adjust;
			via_gfx_dpa.clk_driving_sel =
				via_hdmi_options->clk_driving_sel;
			via_gfx_dpa.clk_polarity =
				via_hdmi_options->clk_polarity;
			via_gfx_dpa.data_driving_sel =
				via_hdmi_options->data_driving_sel;
			via_gfx_dpa.is_clk_adjust_used =
				via_hdmi_options->is_clk_adjust_used;
			via_gfx_dpa.is_clk_driving_sel_used =
				via_hdmi_options->is_clk_driving_sel_used;
			via_gfx_dpa.is_clk_polarity_used =
				via_hdmi_options->is_clk_polarity_used;
			via_gfx_dpa.is_data_driving_sel_used =
				via_hdmi_options->is_data_driving_sel_used;
			via_chrome9_load_user_gfx_dpa_setting(card->di_port,
				&via_gfx_dpa);
		}

		/**
		 * set hdmi overscan adjust borders property init values.
		 * only VX900 support this option... need iga downscaling
		 */
		if ((p_priv->via_chrome9_chip_caps & CAPS_IGA2_DOWNSCALING) ||
			(p_priv->via_chrome9_chip_caps &
			CAPS_IGA1_DOWNSCALING)) {

			KMS_DEBUG(
				"via_hdmi_options->borders = %d, %d, %d, %d\n",
				via_hdmi_options->left_border,
				via_hdmi_options->right_border,
				via_hdmi_options->top_border,
				via_hdmi_options->bottom_border);

			/* step1: re-set mode. */
			if( ( left_border_bak != via_hdmi_options->left_border ) || 
			    ( right_border_bak != via_hdmi_options->right_border ) ||
			    ( top_border_bak != via_hdmi_options->top_border ) ||
			    ( bottom_border_bak != via_hdmi_options->bottom_border ) ) {
				KMS_DEBUG("borders option changed! need set mode!\n");
				via_chrome9_helper_set_mode(encoder);
			}

			/* step2: refresh properties value. */
			drm_connector_property_set_value(
				&via_chrome9_connector->base,
				p_priv->mode_info.left_border_property,
				via_hdmi_options->left_border);
			drm_connector_property_set_value(
				&via_chrome9_connector->base,
				p_priv->mode_info.right_border_property,
				via_hdmi_options->right_border);
			drm_connector_property_set_value(
				&via_chrome9_connector->base,
				p_priv->mode_info.top_border_property,
				via_hdmi_options->top_border);
			drm_connector_property_set_value(
				&via_chrome9_connector->base,
				p_priv->mode_info.bottom_border_property,
				via_hdmi_options->bottom_border);

		}
	}
}

static unsigned int via_chrome9_check_hdmi_i2c_status(
		unsigned int check_bits, unsigned int condition)
{
	unsigned int    status = true;
	unsigned int    loop = 0, max = 50;

	if (condition) {
		while ((MMIO_RD(0xC0B8) & check_bits) && loop < max) {
			/* delay 20 us */
			udelay(20);

			if (++loop == max)
				status = false;
		}
	} else {
		while (!(MMIO_RD(0xC0B8) & check_bits) && loop < max) {
			/* delay 20 us */
			udelay(20);
			if (++loop == max)
				status = false;
		}
	}
	return status;
}

unsigned  int via_chrome9_ddc_read_byte_by_hdmi(unsigned short slave_addr,
		unsigned char index,
		unsigned char *p_byte)
{
	unsigned int    temp = 0;
	unsigned int    status = true;

	/* Enable DDC */
	MMIO_WR_MASK(0xC000, 0x00000001, 0x00000001);
	MMIO_WR(0xC0C4, (MMIO_RD(0xC0C4) & 0xFC7FFFFF) | 0x00800000);
	MMIO_WR(0xC0B8, 0x00000001);

	/* START */
	MMIO_WR(0xC0B8, 0x0011);
	MMIO_WR(0xC0B8, 0x0019);
	if (status)
		status = via_chrome9_check_hdmi_i2c_status(0x0018, true);

	/* Slave Address */
	temp = slave_addr;
	temp <<= 16;
	temp |= MMIO_RD(0xC0B4) & 0xFF00FFFF;
	MMIO_WR(0xC0B4, temp);
	MMIO_WR(0xC0B8, 0x0009);
	if (status)
		status = via_chrome9_check_hdmi_i2c_status(0x0008, true);

	/* Offset */
	temp = index;
	temp <<= 16;
	temp |= MMIO_RD(0xC0B4) & 0xFF00FFFF;
	MMIO_WR(0xC0B4, temp);
	MMIO_WR(0xC0B8, 0x0009);
	if (status)
		status = via_chrome9_check_hdmi_i2c_status(0x0008, true);

	/* START */
	MMIO_WR(0xC0B8, 0x0011);
	MMIO_WR(0xC0B8, 0x0019);
	status = via_chrome9_check_hdmi_i2c_status(0x0018, true);

	/* Slave Address + 1 */
	temp = slave_addr + 1;
	temp <<= 16;
	temp |= MMIO_RD(0xC0B4) & 0xFF00FFFF;
	MMIO_WR(0xC0B4, temp);
	MMIO_WR(0xC0B8, 0x0009);
	if (status)
		status = via_chrome9_check_hdmi_i2c_status(0x0008, true);

	/* Read Data */
	MMIO_WR(0xC0B8, 0x0009);
	via_chrome9_check_hdmi_i2c_status(0x0008, true);
	if (status)
		status = via_chrome9_check_hdmi_i2c_status(0x0080, false);

	MMIO_WR(0xC0B8, (MMIO_RD(0xC0B8) & ~0x80));

	/* STOP */
	MMIO_WR(0xC0B8, 0x0021);
	MMIO_WR(0xC0B8, 0x0029);
	if (status)
		status = via_chrome9_check_hdmi_i2c_status(0x0828, true);

	if (status) {
		*p_byte = (unsigned char)((MMIO_RD(0xC0B4) & 0x0000FF00) >> 8);
	} else {
		/* Reset */
		MMIO_WR_MASK(0xC0C4, 0x00000080, 0x00000080);
		MMIO_WR_MASK(0xC0C4, 0x00000000, 0x00000080);
	}

	return status;
}

unsigned  int via_chrome9_ddc_write_by_hdmi(
		unsigned short slave_addr, unsigned char index)
{
	unsigned int temp;
	unsigned int status = true;

	/* Reset */
	/* MMIO_WR_MASK(0xC0C4, 0x00000080, 0x00000080); */
	MMIO_WR_MASK(0xC0C4, 0x00000000, 0x00000080);

	/* Enable DDC */
	MMIO_WR_MASK(0xC000, 0x00000001, 0x00000001);
	MMIO_WR(0xC0C4, (MMIO_RD(0xC0C4) & 0xFC7FFFFF) | 0x00800000);
	MMIO_WR(0xC0B8, 0x00000001);

	/* START */
	MMIO_WR(0xC0B8, 0x0011);
	MMIO_WR(0xC0B8, 0x0019);
	if (status)
		status = via_chrome9_check_hdmi_i2c_status(0x0018, true);

	/* Slave Address */
	temp = slave_addr;
	temp <<= 16;
	temp |= MMIO_RD(0xC0B4) & 0xFF00FFFF;
	MMIO_WR(0xC0B4, temp);
	MMIO_WR(0xC0B8, 0x0009);
	if (status)
		status = via_chrome9_check_hdmi_i2c_status(0x0008, true);

	/* Offset */
	temp = index;
	temp <<= 16;
	temp |= MMIO_RD(0xC0B4) & 0xFF00FFFF;
	MMIO_WR(0xC0B4, temp);
	MMIO_WR(0xC0B8, 0x0009);
	if (status)
		status = via_chrome9_check_hdmi_i2c_status(0x0008, true);
	return status;
}

unsigned int via_chrome9_ddc_read_bytes_by_hdmi(unsigned short slave_addr,
		unsigned char index, unsigned char *p_byte, int n_read)
{
	unsigned int temp = 0;
	unsigned int status = true;
	unsigned int loop;
	unsigned char ckm = 0;

	/* Enable DDC*/
	MMIO_WR_MASK(0xC000, 0x00000001, 0x00000001);
	MMIO_WR(0xC0C4, (MMIO_RD(0xC0C4) & 0xFC7FFFFF) | 0x00800000);
	MMIO_WR(0xC0B8, 0x00000001);

	/* START */
	MMIO_WR(0xC0B8, 0x0011);
	MMIO_WR(0xC0B8, 0x0019);
	if (status)
		status = via_chrome9_check_hdmi_i2c_status(0x0018, true);

	/* Slave Address */
	temp = slave_addr;
	temp <<= 16;
	temp |= MMIO_RD(0xC0B4) & 0xFF00FFFF;
	MMIO_WR(0xC0B4, temp);
	MMIO_WR(0xC0B8, 0x0009);
	if (status)
		status = via_chrome9_check_hdmi_i2c_status(0x0008, true);

	/* Offset */
	temp = index;
	temp <<= 16;
	temp |= MMIO_RD(0xC0B4) & 0xFF00FFFF;
	MMIO_WR(0xC0B4, temp);
	MMIO_WR(0xC0B8, 0x0009);
	if (status)
		status = via_chrome9_check_hdmi_i2c_status(0x0008, true);

	/* START */
	MMIO_WR(0xC0B8, 0x0011);
	MMIO_WR(0xC0B8, 0x0019);
	if (status)
		status = via_chrome9_check_hdmi_i2c_status(0x0018, true);

	/* Slave Address + 1 */
	temp = slave_addr + 1;
	temp <<= 16;
	temp |= MMIO_RD(0xC0B4) & 0xFF00FFFF;
	MMIO_WR(0xC0B4, temp);
	MMIO_WR(0xC0B8, 0x0009);
	if (status)
		status = via_chrome9_check_hdmi_i2c_status(0x0008, true);

	for (loop = 0; loop < n_read; loop++) {
		MMIO_WR(0xC0B8, 0x0009);
		via_chrome9_check_hdmi_i2c_status(0x0008, true);
		via_chrome9_check_hdmi_i2c_status(0x0080, false);
		/* Remove the comment when upstream!! */
		/* correct EDID check sum */
		if (127 == loop % 128) {
			*p_byte++ = (0x100 - ckm);
			ckm = 0;
		} else {
			ckm +=
				(unsigned char)
				((MMIO_RD(0xC0B4) & 0x0000FF00) >> 8);
			*p_byte++ =
				(unsigned char)
				((MMIO_RD(0xC0B4) & 0x0000FF00) >> 8);
		}
		MMIO_WR(0xC0B8, (MMIO_RD(0xC0B8) & ~0x80));
	}

	/* STOP */
	MMIO_WR(0xC0B8, 0x0021);
	MMIO_WR(0xC0B8, 0x0029);

	status = via_chrome9_check_hdmi_i2c_status(0x0828, true);
	if (!status) {
		/* Reset */
		MMIO_WR_MASK(0xC0C4, 0x00000080, 0x00000080);
		MMIO_WR_MASK(0xC0C4, 0x00000000, 0x00000080);
	}
	return status;
}

bool via_chrome9_check_std_edid_block(u8 *edid_data)
{
	if ((0 == edid_data[0]) && (0xFF == edid_data[1])
		&& (0xFF == edid_data[2])) {
		return true;
	} else {
		return false;
	}
}

bool via_chrome9_check_ext_edid_block(u8 *edid_data)
{
	bool ret = false;
	/* Remove the comment when upstream!! */
	/* only check CEA Block currently! */
	/* 0x02: CEA flag. 0x03: only support version 3 */
	if (edid_data[0] == 0x02 && edid_data[1] == 0x03) {
		KMS_DEBUG("Found CEA Block\n");
		ret = true;
	}
	return ret;
}

void via_chrome9_prune_un_preferred_modes(struct drm_device *dev,
		struct list_head *mode_list)
{
	struct drm_display_mode *mode, *t;

	list_for_each_entry_safe(mode, t, mode_list, head) {
		if (!(mode->type & DRM_MODE_TYPE_PREFERRED)) {
			list_del(&mode->head);
			drm_mode_destroy(dev, mode);
		}
	}
}

/**
 * via_chrome9_mode_equal - test modes for equality
 * @mode1: first mode
 * @mode2: second mode
 *
 * proting from drm_mode_equal
 * do not care hskew, fix me if any trouble
 *
 * Check to see if @mode1 and @mode2 are equivalent.
 *
 * RETURNS:
 * True if the modes are equal, false otherwise.
 */
bool via_chrome9_mode_equal(struct drm_display_mode *mode1,
		struct drm_display_mode *mode2)
{
	if (mode1->clock && mode2->clock &&
	     mode1->hdisplay == mode2->hdisplay &&
	     mode1->hsync_start == mode2->hsync_start &&
	     mode1->hsync_end == mode2->hsync_end &&
	     mode1->htotal == mode2->htotal &&
	     mode1->vdisplay == mode2->vdisplay &&
	     mode1->vsync_start == mode2->vsync_start &&
	     mode1->vsync_end == mode2->vsync_end &&
	     mode1->vtotal == mode2->vtotal &&
	     mode1->vscan == mode2->vscan &&
	     mode1->flags == mode2->flags)
		return true;

	return false;
}
void via_chrome9_prune_un_cea_modes(struct drm_device *dev,
		struct list_head *mode_list)
{
	int i;
	struct drm_display_mode *mode, *t;

	list_for_each_entry_safe(mode, t, mode_list, head) {
		for (i = 0; i < VIA_CEA_VIDEO_MODES_NUM; i++) {
			if (via_chrome9_mode_equal(mode,
				&via_cea_video_modes[i])) {
				mode->hskew = via_cea_video_modes[i].hskew;
				mode->type |= DRM_MODE_TYPE_DRIVER;
				break;
			}
		}
		if (VIA_CEA_VIDEO_MODES_NUM == i) {
			list_del(&mode->head);
			drm_mode_destroy(dev, mode);
		}
	}
}

int via_chrome9_add_cea_mode_line(struct drm_connector *connector,
		u8 *ext_edid)
{
	int i = 0;
	int mode_count = 0;
	bool first_native_mode = true;
	u8 tmp = 0;
	u8 video_block_index = 0;
	u32 descriptor_num = 0;
	u8 hdmi_mode_index;
	struct via_chrome9_connector *via_con =
		to_via_chrome9_connector(connector);
	struct via_hdmi_options *via_hdmi_options =
		(struct via_hdmi_options *)via_con->options;
	struct drm_device *dev = connector->dev;
	struct drm_display_mode *new_mode = NULL;

	if (!via_hdmi_options->attach_all_modes) {
		/* Remove the comment when upstream!! */
		/* Byte 04 is where the Data Block Collection begins */
		video_block_index = 4;
		tmp = ext_edid[4];
		/* Remove the comment when upstream!! */
		/* Locate Video Block */
		/* Bit[7:5]: Block Type Tag: value 2 is video specific */
		while (((tmp & 0xE0) != 0x40)
			&& (video_block_index < (ext_edid[2]))) {
			/* Remove the comment when upstream!! */
			/* Bit[4:0]:
			* Total number of bytes in this block
			* following this byte
			*/
			tmp &= 0x1F;
			/* The next byte is assumed to be the beginning of the
			*   next data block.
			*/
			video_block_index += (tmp + 1);
			/* Locate the next data block */
			tmp = ext_edid[video_block_index];
		}
		if ((tmp & 0xE0) == 0x40) {
			descriptor_num = tmp & 0x1F;
			for (i = 0; i < descriptor_num; i++) {
				tmp = ext_edid[video_block_index+1+i];
				hdmi_mode_index = tmp & 0x7F;

				KMS_DEBUG(
					"Monitor support CEA mode ID:%d %dx%d@%d I:%d\n",
					hdmi_mode_index,
					via_cea_timing_table[
						hdmi_mode_index-1].hor_size,
					via_cea_timing_table[
						hdmi_mode_index-1].ver_size,
					via_cea_timing_table[
						hdmi_mode_index-1].refresh_rate,
					via_cea_timing_table[
						hdmi_mode_index-1].scan_type);

				if (via_cea_timing_table[
					hdmi_mode_index-1].is_built &&
					via_cea_video_modes[
					hdmi_mode_index-1].name != NULL) {
					new_mode = drm_mode_create(dev);
					if (new_mode) {
						memcpy(new_mode,
							&via_cea_video_modes[hdmi_mode_index-1],
							sizeof(struct drm_display_mode));
						if ((tmp & 0x80) &&
							first_native_mode) {
							new_mode->type |=
								DRM_MODE_TYPE_PREFERRED;
							first_native_mode =
								false;
						}
						drm_mode_probed_add(connector,
							new_mode);
						mode_count++;
						KMS_DEBUG(
							"Driver add CEA modeID:\
							%d %dx%d@%d I:%d\n",
							hdmi_mode_index,
							via_cea_timing_table[hdmi_mode_index-1].hor_size,
							via_cea_timing_table[hdmi_mode_index-1].ver_size,
							via_cea_timing_table[hdmi_mode_index-1].refresh_rate,
							via_cea_timing_table[hdmi_mode_index-1].scan_type);
					}
				}
			}
		}
	}

	if (0 == mode_count) {
		i = 0;
		while (via_cea_video_modes[i].name &&
			via_cea_timing_table[i].is_built) {
			new_mode = drm_mode_create(dev);
			if (new_mode) {
				memcpy(new_mode,
					&via_cea_video_modes[i],
					sizeof(struct drm_display_mode));
				drm_mode_probed_add(connector, new_mode);
				mode_count++;
				KMS_DEBUG(
					"Driver add CEA mode ID:%d %dx%d@%d I:%d\n",
					i,
					via_cea_timing_table[i].hor_size,
					via_cea_timing_table[i].ver_size,
					via_cea_timing_table[i].refresh_rate,
					via_cea_timing_table[i].scan_type);
			}
			i++;
		}
	}
	return mode_count;
}

u32 via_chrome9_detect_connect_type(u8 *ext_edid)
{
	u8  vendor_block_index = 0;
	u8  tmp = 0;
	u32 ret = VIA_CHROME9_DEVICE_CONNECTOR_DVI_D;
	/* Remove the comment when upstream!! */
	/* Check Data Block amount. If no data block exists, return DVI connect.
	Byte 2: byte number offset d where detail timing data begin. */
	/* if 0, No Data Block, No Detailed Timing Descriptor.*/
	/* if 4, No Data Block.*/
	if (!(ext_edid[2] == 0 || ext_edid[2] == 4)) {
		/* Remove the comment when upstream!! */
		/* Byte 04 is where the Data Block Collection begins */
		vendor_block_index = 4;
		tmp = ext_edid[vendor_block_index];
		/* Remove the comment when upstream!! */
		/* Locate Vendor Description Block
		*  to identify if connect to HDMI.
		*/
		/* Bit[7:5]: Block Type Tag: value 3 is vendor specific*/
		while (((tmp & 0xE0) != 0x60) &&
			(vendor_block_index < (ext_edid[2]))) {
			/* Remove the comment when upstream!! */
			/* Bit[4:0]: Total number of bytes in this block
			*   following this byte
			*/
			tmp &= 0x1F;
			/* The next byte is assumed to be the beginning
			of the next data block. */
			vendor_block_index += (tmp+1);
			/* Locate the next data block */
			tmp = ext_edid[vendor_block_index];
		}

		if (0x60 == (tmp & 0xE0)) {
			/* Remove the comment when upstream!! */
			/* Read 24-bit IEEE registration.*/
			/* HDMI identifier = 0x000C03.*/
			if ((ext_edid[vendor_block_index+1] == 0x03)
			&& (ext_edid[vendor_block_index+2] == 0x0C)
			&& (ext_edid[vendor_block_index+3] == 0x00)) {
				ret =  VIA_CHROME9_DEVICE_CONNECTOR_HDMI;
			} else {
				ret = VIA_CHROME9_DEVICE_CONNECTOR_DVI_D;
			}
		}
	}
	return ret;
}

void via_chrome9_get_internal_hdmi_dvi_edid(u8 *std_edid, u8 *ext_edid)
{
	u8 edid_blk0_blk1[256];
	u8 edid_blk2_blk3[256];
	u8 *std_edid_ptr = NULL;
	u8 *ext_edid_ptr = NULL;
	u8 edid_flag;
	bool is_std_edid_read_ok = false;
	bool has_ext_edid = false;
	int status;

	via_chrome9_ddc_read_bytes_by_hdmi(0xA0, 0x00, edid_blk0_blk1, 256);
	if (via_chrome9_check_std_edid_block(edid_blk0_blk1)) {
		KMS_DEBUG("hdmi read std edid success\n");
		is_std_edid_read_ok = true;
		std_edid_ptr = &edid_blk0_blk1[0x0];
		edid_flag = edid_blk0_blk1[0x7e];
		KMS_DEBUG("hdmi std edid flag is %d\n", edid_flag);
		switch (edid_flag) {
		case 0x00:
			KMS_DEBUG("hdmi has no extension block\n");
			has_ext_edid = false;
			break;

		case 0x01:
			if (via_chrome9_check_ext_edid_block(
				&edid_blk0_blk1[0x80])) {
				KMS_DEBUG("Read ext edid success at block 1\n");
				ext_edid_ptr = &edid_blk0_blk1[0x80];
				has_ext_edid = true;
			} else {
				KMS_DEBUG("Read ext edid fail\n");
				has_ext_edid = false;
			}
			break;

		case 0x02:
		case 0x03:
		default:
			/* Remove the comment when upstream!! */
			/* For HDMI 4-block EDID */
			/* Use segment pointer 0x60 to access
			the next two blocks of E-EDID */
			KMS_DEBUG("HDMI 4-block EDID\n");
			status = via_chrome9_ddc_write_by_hdmi(0x60, 0x01);
			KMS_DEBUG("ret of set segment pointer is %d\n", status);
			status = via_chrome9_ddc_read_bytes_by_hdmi(
				0xA0, 0x00, edid_blk2_blk3, 256);
			KMS_DEBUG("ret of read block 2,3 is %d", status);
			if (via_chrome9_check_ext_edid_block(
				&edid_blk2_blk3[0x00])) {
				KMS_DEBUG("Read ext edid success ar block 2\n");
				ext_edid_ptr = &edid_blk2_blk3[0x00];
				has_ext_edid = true;
			} else {
				if (via_chrome9_check_ext_edid_block(
					&edid_blk2_blk3[0x80])) {
					KMS_DEBUG(
						"Read ext edid success at block 3\n");
					ext_edid_ptr = &edid_blk2_blk3[0x80];
					has_ext_edid = true;
				} else {
					has_ext_edid = false;
					KMS_DEBUG("Read ext edid fail\n");
				}
			}
			break;
		}
	} else {
		KMS_DEBUG("Read hdmi std edid fail\n");
	}

	if (is_std_edid_read_ok) {
		memcpy(std_edid, std_edid_ptr, 128);
		if (has_ext_edid)
			memcpy(ext_edid, ext_edid_ptr, 128);
	}
}

static enum drm_connector_status via_chrome9_hdmi_detect(
		struct drm_connector *connector,  bool force)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_hdmi_connector_private_info *via_hdmi_con_info =
		(struct via_hdmi_connector_private_info *)
		via_chrome9_connector->connector_info;
	enum drm_connector_status ret = connector_status_disconnected;
	struct drm_connector_helper_funcs *via_hdmi_helper_funcs =
		(struct drm_connector_helper_funcs *)connector->helper_private;
	struct drm_encoder  *encoder =
		via_hdmi_helper_funcs->best_encoder(connector);
	struct via_chrome9_encoder *via_chrome9_encoder = NULL;
	struct via_hdmi_encoder_private_info *via_hdmi_enc_info = NULL;
	struct via_hdmi_options *via_hdmi_options =
		(struct via_hdmi_options *)via_chrome9_connector->options;
	u8 *block;
	u32 mm_c730;

	if (encoder) {
		via_chrome9_encoder = to_via_chrome9_encoder(encoder);
		via_hdmi_enc_info = (struct via_hdmi_encoder_private_info *)
			via_chrome9_encoder->encoder_private;
	} else {
		KMS_DEBUG("hdmi connector does not have encoder\n");
		return ret;
	}
	mm_c730 = MMIO_RD(0xc730) & 0xC0000000;
	if (VIA_IRQ_DP_HOT_UNPLUG == mm_c730) {
		memset(via_hdmi_con_info->std_edid, 0,
			sizeof(via_hdmi_con_info->std_edid));
		memset(via_hdmi_con_info->ext_edid, 0,
			sizeof(via_hdmi_con_info->ext_edid));
		kfree((u8 *)via_chrome9_connector->edid);
		connector->display_info.raw_edid = NULL;
		via_chrome9_connector->edid = NULL;
		via_hdmi_con_info->read_edid = true;
		return ret;
	}

	if (via_hdmi_con_info->read_edid) {
		block = kzalloc(EDID_LENGTH * 2, GFP_KERNEL);
		if (block == NULL) {
			KMS_DEBUG("edid block alloc fail!\n");
			return -1;
		}
		kfree((u8 *)via_chrome9_connector->edid);
		connector->display_info.raw_edid = NULL;
		via_chrome9_connector->edid = NULL;
		via_chrome9_get_internal_hdmi_dvi_edid(
			via_hdmi_con_info->std_edid,
			via_hdmi_con_info->ext_edid);
		memcpy(block, via_hdmi_con_info->std_edid, 128);
		memcpy(block + 128, via_hdmi_con_info->ext_edid, 128);
		via_chrome9_connector->edid = (struct edid *)block;
		connector->display_info.raw_edid = block;
		via_hdmi_con_info->read_edid = false;
	}

	if (via_chrome9_check_std_edid_block(via_hdmi_con_info->std_edid)) {
		if (via_chrome9_check_ext_edid_block(
			via_hdmi_con_info->ext_edid)) {
			via_hdmi_con_info->connector_type =
				via_chrome9_detect_connect_type(
				via_hdmi_con_info->ext_edid);
		} else {
			if (DIGITAL_DEVICE ==
				((via_hdmi_con_info->std_edid[20] &
				0x80) >> 7)) {
				via_hdmi_con_info->connector_type =
					VIA_CHROME9_DEVICE_CONNECTOR_DVI_D;
				KMS_DEBUG(
					"No ext edid hdmi connector type is DVI\n");

			} else {
				via_hdmi_con_info->connector_type =
					VIA_CHROME9_DEVICE_CONNECTOR_UNKNOW;
				KMS_DEBUG(
					"No ext edid hdmi connector type is unknow\n");
			}
		}
	} else {
		via_hdmi_con_info->connector_type =
			VIA_CHROME9_DEVICE_CONNECTOR_UNKNOW;
		KMS_DEBUG("No edid hdmi connector type is unknow\n");
	}

	if ((via_hdmi_options->attach_all_modes) &&
		(VIA_CHROME9_DEVICE_CONNECTOR_DVI_D !=
		via_hdmi_con_info->connector_type)) {
		ret = connector_status_connected;
	} else {
		if (VIA_CHROME9_DEVICE_CONNECTOR_HDMI ==
			via_hdmi_con_info->connector_type)
			ret = connector_status_connected;
	}

	if (connector_status_connected == ret) {
		via_hdmi_enc_info->dp_1_ephy_mode = via_hdmi_mode;
		via_hdmi_enc_info->connector_type =
			via_hdmi_con_info->connector_type;
		KMS_DEBUG("DP1 EPHY set as HDMI mode\n");
	} else {
		KMS_DEBUG("internal hdmi is not connected\n");
	}
	return ret;
}

int via_chrome9_hdmi_set_property(struct drm_connector *connector,
		struct drm_property *property,
		uint64_t value)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_hdmi_options *via_hdmi_option =
		(struct via_hdmi_options *)via_chrome9_connector->options;
	struct drm_connector_helper_funcs *via_hdmi_helper_funcs =
		(struct drm_connector_helper_funcs *)connector->helper_private;
	struct drm_encoder  *encoder =
		via_hdmi_helper_funcs->best_encoder(connector);
	struct drm_device *dev = connector->dev;
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;

	if (property == p_priv->mode_info.left_border_property) {
		if (!encoder) {
			KMS_DEBUG("encoder is NULL\n");
			return 0;
		}

		if (value < 0 || value > MAX_SUPPORTED_HOR_BORDER)
			return 0;

		if (via_hdmi_option->left_border != value) {
			via_hdmi_option->left_border = value;
			via_chrome9_property_change_mode(encoder);
		}
	}


	if (property == p_priv->mode_info.right_border_property) {
		if (!encoder) {
			KMS_DEBUG("encoder is NULL\n");
			return 0;
		}

		if (value < 0 || value > MAX_SUPPORTED_HOR_BORDER)
			return 0;

		if (via_hdmi_option->right_border != value) {
			via_hdmi_option->right_border = value;
			via_chrome9_property_change_mode(encoder);
		}
	}


	if (property == p_priv->mode_info.top_border_property) {
		if (!encoder) {
			KMS_DEBUG("encoder is NULL\n");
			return 0;
		}

		if (value < 0 || value > MAX_SUPPORTED_VER_BORDER)
			return 0;

		if (via_hdmi_option->top_border != value) {
			via_hdmi_option->top_border = value;
			via_chrome9_property_change_mode(encoder);
		}
	}


	if (property == p_priv->mode_info.bottom_border_property) {
		if (!encoder) {
			KMS_DEBUG("encoder is NULL\n");
			return 0;
		}

		if (value < 0 || value > MAX_SUPPORTED_VER_BORDER)
			return 0;

		if (via_hdmi_option->bottom_border != value) {
			via_hdmi_option->bottom_border = value;
			via_chrome9_property_change_mode(encoder);
		}
	}

	return 0;
}

int via_chrome9_hdmi_get_modes(struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_hdmi_connector_private_info *via_hdmi_con_info =
		(struct via_hdmi_connector_private_info *)
		via_chrome9_connector->connector_info;

	struct drm_device *dev = connector->dev;
	int ret;

	if (via_chrome9_connector->edid) {
		drm_mode_connector_update_edid_property(
			&via_chrome9_connector->base,
			via_chrome9_connector->edid);
		drm_add_edid_modes(
			&via_chrome9_connector->base,
			via_chrome9_connector->edid);
	}

	via_chrome9_prune_un_preferred_modes(
		dev, &(connector->probed_modes));
	via_chrome9_prune_un_cea_modes(
		dev, &(connector->probed_modes));
	ret = via_chrome9_add_cea_mode_line(
		connector, via_hdmi_con_info->ext_edid);
	KMS_DEBUG(
		"via_chrome9_hdmi_get_modes %s\n",
		drm_get_connector_name(connector));
	return ret;
}

int via_chrome9_hdmi_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode)
{
	struct drm_device *dev = connector->dev;
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	KMS_DEBUG("\n");
		/* Currently, only VX855 can support CEA interlace mode. */
		if ((p_priv->pseudo_chip_revision < REVISION_VX855_A0) &&
			(mode->flags & DRM_MODE_FLAG_INTERLACE))
			return MODE_NOMODE;

		if (mode->type & DRM_MODE_TYPE_DRIVER)
			return MODE_OK;
		else
			return MODE_NOMODE;
	return MODE_OK;
}

static struct drm_connector_funcs
	via_chrome9_hdmi_connector_funcs = {
	.dpms = drm_helper_connector_dpms,
	.detect = via_chrome9_hdmi_detect,
	.fill_modes = drm_helper_probe_single_connector_modes,
	.destroy = via_chrome9_connector_destory,
	.set_property = via_chrome9_hdmi_set_property,
};

static struct drm_connector_helper_funcs
	via_chrome9_hdmi_connector_helper_funcs = {
	.get_modes = via_chrome9_hdmi_get_modes,
	.mode_valid = via_chrome9_hdmi_mode_valid,
	.best_encoder = via_chrome9_best_single_encoder,
};
static enum drm_connector_status via_chrome9_hdmi_dvi_detect(
		struct drm_connector *connector,  bool force)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_dvi_connector_private_info *via_dvi_con_info =
		(struct via_dvi_connector_private_info *)
		via_chrome9_connector->connector_info;
	enum drm_connector_status ret = connector_status_disconnected;
	struct drm_connector_helper_funcs *via_hdmi_dvi_helper_funcs =
		(struct drm_connector_helper_funcs *)connector->helper_private;
	struct drm_encoder  *encoder =
		via_hdmi_dvi_helper_funcs->best_encoder(connector);
	struct via_chrome9_encoder *via_chrome9_encoder = NULL;
	struct via_hdmi_encoder_private_info *via_hdmi_enc_info = NULL;
	struct via_dvi_options *via_dvi_options =
		(struct via_dvi_options *)via_chrome9_connector->options;
	u8 *block;
	u32 mm_c730;
	/* if there is a connector , to decide the connector type */
	if (encoder) {
		via_chrome9_encoder = to_via_chrome9_encoder(encoder);
		via_hdmi_enc_info = (struct via_hdmi_encoder_private_info *)
			via_chrome9_encoder->encoder_private;
	} else {
		KMS_DEBUG("hdmi-dvi connector does not have encoder\n");
		return ret;
	}

	mm_c730 = MMIO_RD(0xc730) & 0xC0000000;
	if (VIA_IRQ_DP_HOT_UNPLUG == mm_c730) {
		memset(via_dvi_con_info->std_edid, 0,
			sizeof(via_dvi_con_info->std_edid));
		memset(via_dvi_con_info->ext_edid, 0,
			sizeof(via_dvi_con_info->ext_edid));
		kfree((u8 *)via_chrome9_connector->edid);
		connector->display_info.raw_edid = NULL;
		via_chrome9_connector->edid = NULL;
		via_dvi_con_info->read_edid = true;
		return ret;
	}

	if (via_dvi_con_info->read_edid) {
		block = kzalloc(EDID_LENGTH * 2, GFP_KERNEL);
		if (block == NULL) {
			KMS_DEBUG("edid block alloc fail!\n");
			return -1;
		}
		kfree((u8 *)via_chrome9_connector->edid);
		connector->display_info.raw_edid = NULL;
		via_chrome9_connector->edid = NULL;

		via_chrome9_get_internal_hdmi_dvi_edid(
			via_dvi_con_info->std_edid,
			via_dvi_con_info->ext_edid);

		memcpy(block, via_dvi_con_info->std_edid, 128);
		memcpy(block + 128, via_dvi_con_info->ext_edid, 128);
		connector->display_info.raw_edid = block;
		via_chrome9_connector->edid = (struct edid *)block;
		via_dvi_con_info->read_edid = false;
	}

	if (via_chrome9_check_std_edid_block(
		via_dvi_con_info->std_edid)) {
		if (via_chrome9_check_ext_edid_block(
			via_dvi_con_info->ext_edid)) {
			via_dvi_con_info->connector_type =
				via_chrome9_detect_connect_type(
				via_dvi_con_info->ext_edid);
		} else {
			if (DIGITAL_DEVICE ==
				((via_dvi_con_info->std_edid[20] & 0x80)
				>> 7)) {
				via_dvi_con_info->connector_type =
					VIA_CHROME9_DEVICE_CONNECTOR_DVI_D;
				KMS_DEBUG(
					"No ext edid hdmi connector type is DVI\n");
			} else {
				via_dvi_con_info->connector_type =
					VIA_CHROME9_DEVICE_CONNECTOR_UNKNOW;
				KMS_DEBUG(
					"No ext edid hdmi connector type is unknow\n");
			}
		}
	} else {
		via_dvi_con_info->connector_type =
			VIA_CHROME9_DEVICE_CONNECTOR_UNKNOW;
		KMS_DEBUG("No edid hdmi connector type is unknow\n");
	}

	if ((via_dvi_options->no_ddc_value) &&
		(VIA_CHROME9_DEVICE_CONNECTOR_HDMI !=
		via_dvi_con_info->connector_type)) {
		via_dvi_con_info->connector_type =
			VIA_CHROME9_DEVICE_CONNECTOR_DVI_D;
		ret = connector_status_connected;
	} else {
		if (VIA_CHROME9_DEVICE_CONNECTOR_DVI_D ==
			via_dvi_con_info->connector_type)
			ret = connector_status_connected;
	}

	if (connector_status_connected == ret) {
		via_hdmi_enc_info->dp_1_ephy_mode = via_hdmi_mode;
		via_hdmi_enc_info->connector_type =
			via_dvi_con_info->connector_type;
		KMS_DEBUG("DP1 EPHY set as HDMI mode\n");
	} else {
		KMS_DEBUG("hdmi-dvi is not connected\n");
	}
	return ret;
}

int via_chrome9_hdmi_dvi_set_property(struct drm_connector *connector,
		struct drm_property *property,
		uint64_t value)
{
	/* implement later if necessary */
	return 0;
}

int via_chrome9_hdmi_dvi_get_modes(struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_dvi_connector_private_info *via_dvi_con_info =
		(struct via_dvi_connector_private_info *)
		via_chrome9_connector->connector_info;
	struct via_dvi_options *via_dvi_options =
		(struct via_dvi_options *)via_chrome9_connector->options;

	int ret = 0;
	if (via_dvi_options->no_ddc_value) {
		via_dvi_con_info->physical_width = 4096;
		via_dvi_con_info->physical_height = 2048;
	} else {

		if (via_chrome9_connector->edid) {
			drm_mode_connector_update_edid_property(
				&via_chrome9_connector->base,
				via_chrome9_connector->edid);
			ret = drm_add_edid_modes(&via_chrome9_connector->base,
				via_chrome9_connector->edid);
		}

		via_chrome9_get_monitor_physical_size(connector,
			&via_dvi_con_info->physical_width,
			&via_dvi_con_info->physical_height);
	}
	KMS_DEBUG("%s\n", drm_get_connector_name(connector));
	return ret;
}

int via_chrome9_hdmi_dvi_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_dvi_connector_private_info *via_dvi_con_info =
		(struct via_dvi_connector_private_info *)
		via_chrome9_connector->connector_info;
	KMS_DEBUG("\n");
	/* Check Clock Range */
	if (mode->clock > 165000)
		return MODE_CLOCK_HIGH;
	if (mode->clock < 25000)
		return MODE_CLOCK_LOW;
	/* Check monitor physical size */
	if (mode->hdisplay > via_dvi_con_info->physical_width)
		return MODE_BAD_HVALUE;
	if (mode->vdisplay > via_dvi_con_info->physical_height)
		return MODE_BAD_VVALUE;
	/* Check Resolution Range */
	if (via_chrome9_is_valid_mode(mode->hdisplay, mode->vdisplay) == 0)
		return MODE_BAD;
	return MODE_OK;
}

static struct drm_connector_funcs via_chrome9_hdmi_dvi_connector_funcs = {
	.dpms = drm_helper_connector_dpms,
	.detect = via_chrome9_hdmi_dvi_detect,
	.fill_modes = drm_helper_probe_single_connector_modes,
	.destroy = via_chrome9_connector_destory,
	.set_property = via_chrome9_hdmi_dvi_set_property,
};

static struct drm_connector_helper_funcs
	via_chrome9_hdmi_dvi_connector_helper_funcs = {
	.get_modes = via_chrome9_hdmi_dvi_get_modes,
	.mode_valid = via_chrome9_hdmi_dvi_mode_valid,
	.best_encoder = via_chrome9_best_single_encoder,
};


void via_chrome9_set_dp_xorg_options(
		void *options, int size, struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_dp_options *via_dp_options =
		(struct via_dp_options *)via_chrome9_connector->options;
	copy_from_user(via_dp_options, options, size);
}

void via_chrome9_dp_1_clear_aux_read_regs(void)
{
	MMIO_WR(AUX_R_DATA0_REG, 0x0);
	MMIO_WR(AUX_R_DATA1_REG, 0x0);
	MMIO_WR(AUX_R_DATA2_REG, 0x0);
	MMIO_WR(AUX_R_DATA3_REG, 0x0);
}

void via_chrome9_dp_1_clear_aux_write_regs(void)
{
	MMIO_WR(AUX_W_DATA0_REG, 0x0);
	MMIO_WR(AUX_W_DATA1_REG, 0x0);
	MMIO_WR(AUX_W_DATA2_REG, 0x0);
	MMIO_WR(AUX_W_DATA3_REG, 0x0);
}


bool via_chrome9_dp_1_reply_cmd_status(struct drm_connector *connector)
{
	struct drm_device *dev = connector->dev;
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	u32 aux_cmd_status;
	u32 data;
	int i, j;
	for (i = 0; i < NUM_RE_ISSUE_COMMAND; i++) {
		for (j = 0; j < NUM_RECHECK; j++) {
			via_chrome9_delay_usec(dev, 1200);
			aux_cmd_status = MMIO_RD(AUX_TIMER_REG) & 0x3C000007;

			if (p_priv->pseudo_chip_revision > REVISION_VX900_A1 &&
				ACK_OK == aux_cmd_status) {
				return true;
			} else if (aux_cmd_status == 0x00000001) {
				/* hardware issue */
				return true;
			} else if (RE_ISSUE_CMD == (aux_cmd_status & 0xFFFF)) {
				/* re-issue Cmd */
				data = MMIO_RD(AUX_TIMER_REG) & 0xFFFFFFF0;
				data |= 0x03;
				MMIO_WR(AUX_TIMER_REG, data);
				break;
			}
		}
	}

	return false;
}

bool via_chrome9_dp_1_write_dpcd(struct drm_connector *connector,
		u32 offset,
		int byte_count)
{
	struct drm_device *dev = connector->dev;
	u32 cmd;
	u32 data;
	/* AUX read offset */
	cmd = offset << 12;
	 /* AUX read length */
	cmd |= byte_count << 4;
	 /* AUX write CMD */
	cmd |= AUX_CMD_WRITE;
	MMIO_WR(AUX_CMD_REG, cmd);
	/* issue request based on above */
	data = MMIO_RD(AUX_TIMER_REG) & 0xFFFFFFF0;
	data |= 0x03;
	 /* trigger */
	MMIO_WR(AUX_TIMER_REG, data);
	via_chrome9_delay_usec(dev, 10000);
	return via_chrome9_dp_1_reply_cmd_status(connector);
}


bool via_chrome9_dp_1_read_dpcd(struct drm_connector *connector, u32 offset)
{
	u32 cmd;
	u32 data;

	via_chrome9_dp_1_clear_aux_read_regs();
	/* assign command */
	/* AUX read offset */
	cmd = offset << 12;
	/* AUX read length */
	cmd |= 1 << 4;
	/* AUX read CMD */
	cmd |= AUX_CMD_READ;
	MMIO_WR(AUX_CMD_REG, cmd);
	/* issue request based on above */
	data = MMIO_RD(AUX_TIMER_REG) & 0xFFFFFFF0;
	data |= 0x03;
	MMIO_WR(AUX_TIMER_REG, data);
	return via_chrome9_dp_1_reply_cmd_status(connector);
}

void via_chrome9_dp_1_read_dpcd_version(struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_dp_connector_private_info *via_dp_connector_private_info =
		(struct via_dp_connector_private_info *)
		via_chrome9_connector->connector_private;
	u32 dpcd_version;

	via_dp_connector_private_info->dpcd_version = NONEVERSION;

	/* read DPCD Version */
	if (via_chrome9_dp_1_read_dpcd(connector, 0x00)) {
		KMS_DEBUG("via chrome9 dp1 read dpcd sucess\n");
		dpcd_version = MMIO_RD(AUX_R_DATA0_REG);
		KMS_DEBUG(
			"via chrome9 dp1 version is %d\n", dpcd_version);
		dpcd_version &= 0xFF;
		switch (dpcd_version) {
		case 0x10:
			/* Version 1.0 */
			via_dp_connector_private_info->dpcd_version |=
				DPCD_VER_1 | DPCD_REV_0;
			break;
		case 0x11:
			/* Version 1.1 */
			via_dp_connector_private_info->dpcd_version |=
				DPCD_VER_1 | DPCD_REV_1;
			break;
		default:
			via_dp_connector_private_info->dpcd_version =
				NONEVERSION;
			break;
		}
	} else {
		KMS_DEBUG("via chrome9 dp1 read dpcd fail\n");
	}
	KMS_DEBUG("dp1 dpcd version is:%d\n",
		via_dp_connector_private_info->dpcd_version);
	/* Let HW use AUX channel */
	MMIO_WR(AUX_TIMER_REG, 0x0);
}

u8 via_chrome9_dp_1_read_dpcd_link_status(
		struct drm_connector *connector)
{
	u32  dwData;
	u32  dwLaneAlign;
	u32   dwLaneStatus[4];
	u8 linkStatus = DP_LINKSTATUS_DONE;
	int i;

	/* read DPCD Lane Status */
	via_chrome9_dp_1_read_dpcd(connector, 0x202);
	dwData = MMIO_RD(AUX_R_DATA0_REG) & 0xFF;
	dwLaneStatus[0] = dwData & 0xF;
	dwLaneStatus[1] = (dwData >> 4) & 0xF;

	via_chrome9_dp_1_read_dpcd(connector, 0x203);
	dwData = MMIO_RD(AUX_R_DATA0_REG) & 0xFF;
	dwLaneStatus[2] = dwData & 0xF;
	dwLaneStatus[3] = (dwData >> 4) & 0xF;

	via_chrome9_dp_1_read_dpcd(connector, 0x204);
	dwData = MMIO_RD(AUX_R_DATA0_REG) & 0xFF;
	dwLaneAlign = dwData & BIT0;

	if (!dwLaneAlign) {
		linkStatus = DP_LINKSTATUS_LOST;
	} else {
		for (i = 0; i < LANE; i++) {
			if (dwLaneStatus[i] != 0x7) {
				linkStatus = DP_LINKSTATUS_LOST;
				break;
			}
		}
	}
	KMS_DEBUG("DP_LINKSTATUS: %s\n",
		((linkStatus == DP_LINKSTATUS_LOST) ? "LOST" : "LINK"));
	/* Let HW use AUX channel */
	MMIO_WR(AUX_TIMER_REG, 0x0);
	return linkStatus;
}

bool via_chrome9_dp_1_link_training(struct drm_connector *connector)
{
	struct drm_device *dev = connector->dev;
	struct drm_connector_helper_funcs *via_dp_1_helper_funcs =
		(struct drm_connector_helper_funcs *)connector->helper_private;
	struct drm_encoder  *encoder =
		via_dp_1_helper_funcs->best_encoder(connector);
	struct via_chrome9_encoder *via_chrome9_encoder = NULL;
	struct via_dp_encoder_private_info *via_dp_encoder_private_info = NULL;
	u32 data;

	if (encoder) {
		via_chrome9_encoder = to_via_chrome9_encoder(encoder);
		via_dp_encoder_private_info =
			(struct via_dp_encoder_private_info *)
			via_chrome9_encoder->encoder_private;
	} else {
		KMS_DEBUG("DP1 connector does not have encoder\n");
		return 0;
	}

	via_chrome9_dp_1_clear_aux_read_regs();
	via_chrome9_dp_1_clear_aux_write_regs();
	via_dp_encoder_private_info->link_training_done = false;
	/* reset link training */
	MMIO_WR_MASK(DP_DATA_PASS_ENABLE_REG, 0x01, 0x01);
	MMIO_WR_MASK(DP_NAUD_MUTE_REG, 0x80000000, 0x80000000);
	via_chrome9_delay_usec(dev, 1000);
	MMIO_WR_MASK(DP_NAUD_MUTE_REG, 0x0, 0x80000000);
	via_chrome9_delay_usec(dev, 1000);
	/* [9:7]=100b, 4 lanes, [10]=0, disable SW link training */
	/* [12:11]=01b, set clock recovery state */
	data = MMIO_RD(DP_LINK_TRAINING_REG) & 0xFFFF0000;
	data |= 0x00000200;
	switch (via_dp_encoder_private_info->link_speed) {
	case SPEED_162MHz:
		data &= 0xFFFFFFFD;
		break;
	case SPEED_270MHz:
		data |= 0x00000002;
		break;
	default:
		data |= 0x00000002;
		break;
	}
	MMIO_WR(DP_LINK_TRAINING_REG, data);

	/* Reset EPHY */
	MMIO_WR_MASK(DP_EPHY_TX_PWR_REG, 0x55000000, 0xFF000000);
	via_chrome9_delay_usec(dev, 2000);
	MMIO_WR_MASK(DP_EPHY_TX_PWR_REG, 0x00000000, 0xFF000000);
	/* enable scramble */
	MMIO_WR_MASK(DP_VIDEO_CTRL_REG, 0x00000001, 0x00000001);
	/* enable DP, set enhanced mode */
	data = MMIO_RD(DP_ENABLE_IF_REG) & 0xC3000000;
	 /* if enable standard mode , set 0x0000000d */
	data |= 0x0000000d;
	MMIO_WR(DP_ENABLE_IF_REG, data);

	switch (via_dp_encoder_private_info->link_speed) {
	case SPEED_162MHz:
		/* if enable standard mode , set 0x00018406 */
		MMIO_WR(AUX_W_DATA0_REG, 0x00018406);
		break;
	case SPEED_270MHz:
		/* if enable standard mode , set 0x0001840A */
		MMIO_WR(AUX_W_DATA0_REG, 0x0001840A);
		break;
	default:
		MMIO_WR(AUX_W_DATA0_REG, 0x0001840A);
		break;
	}
	MMIO_WR(AUX_W_DATA1_REG, 0x0);
	if (!via_chrome9_dp_1_write_dpcd(connector, 0x100, 2)) {
		/* Let HW use AUX channel */
		MMIO_WR(AUX_TIMER_REG, 0x0);
		return false;
	}
	/* Let HW use AUX channel */
	MMIO_WR(AUX_TIMER_REG, 0x0);
	via_chrome9_delay_usec(dev, 20000);
	data = MMIO_RD(DP_LINK_TRAINING_REG) & 0x7FFFFFFF;
	data |= 0x0000003C;
	if (via_dp_encoder_private_info->link_speed == SPEED_270MHz)
		data |= 0x00000002;

	MMIO_WR(DP_LINK_TRAINING_REG, data);
	/* enable HW link training */
	MMIO_WR_MASK(DP_LINK_TRAINING_REG, 0x00000001, 0x00000001);
	via_chrome9_delay_usec(dev, 20000);
	if ((MMIO_RD(DP_LINK_TRAINING_REG) & 0x80000000) != 0x80000000) {
		/* Let HW use AUX channel */
		MMIO_WR(AUX_TIMER_REG, 0x0);
		return false;
	}
	via_chrome9_delay_usec(dev, 4000);
	if ((MMIO_RD(DP_ENABLE_IF_REG) & 0x30000000) != 0x30000000) {
		/* Let HW use AUX channel */
		MMIO_WR(AUX_TIMER_REG, 0x0);
		return false;
	}
	via_dp_encoder_private_info->link_training_done = true;
	via_dp_encoder_private_info->dp_1_ephy_mode = via_display_port_mode;
	/* clear [31], and [0] */
	MMIO_WR_MASK(DP_LINK_TRAINING_REG, 0x0, 0x80000001);
	/* Let HW use AUX channel */
	MMIO_WR(AUX_TIMER_REG, 0x0);
	return true;
}

bool via_chrome9_dp_1_init_ephy(struct drm_connector *connector)
{
	struct drm_device *dev = connector->dev;
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_dp_connector_private_info *via_dp_connector_private_info =
		(struct via_dp_connector_private_info *)
		via_chrome9_connector->connector_private;
	u32 data = 0;

	/* (1) enable Bandgap and set EPHY symbol rate */
	data = MMIO_RD(DP_EPHY_PLL_REG) & 0xFFF800FF;
	/* real EPHY symbol rate decided by HW link training */
	switch (via_dp_connector_private_info->link_speed) {
	case SPEED_162MHz:
		data |= 0x0004BE00;
		break;
	case SPEED_270MHz:
		data |= 0x00029E00;
		break;
	default:
		data |= 0x00029E00;
		break;
	}
	data |= 0x00000001;
	MMIO_WR(DP_EPHY_PLL_REG, data);
	via_chrome9_delay_usec(dev, 1000);
	/* (2) start AUX channel, CMOP is on, Tx and Rx are off */
	data = MMIO_RD(DP_EPHY_MISC_PWR_REG) & 0xFFFF3FFF;
	data |= 0x00004000;
	MMIO_WR(DP_EPHY_MISC_PWR_REG, data);
	via_chrome9_delay_usec(dev, 4000);
	data = MMIO_RD(DP_EPHY_MISC_PWR_REG) & 0xFFFF3FFF;
	data |= 0x00008000;
	MMIO_WR(DP_EPHY_MISC_PWR_REG, data);
	via_chrome9_delay_usec(dev, 4000);
	/* (3) enable Mpll and Mpll reg power */
	data = MMIO_RD(DP_EPHY_PLL_REG) & 0xF9FFFFF9;
	data |= 0x00000006;
	MMIO_WR(DP_EPHY_PLL_REG, data);
	via_chrome9_delay_usec(dev, 1000);
	/* enable DP */
	MMIO_WR(DP_ENABLE_IF_REG,
		(MMIO_RD(DP_ENABLE_IF_REG) | 0x00000001));
	via_chrome9_delay_usec(dev, 1000);
	/* (4) enable Tpll and Tpll reg power */
	MMIO_WR(DP_EPHY_PLL_REG, (MMIO_RD(DP_EPHY_PLL_REG) | 0x06000000));
	via_chrome9_delay_usec(dev, 1000);
	/* (5) Resister Tuning (RT), RT reset */
	MMIO_WR(DP_EPHY_TX_PWR_REG, (MMIO_RD(DP_EPHY_TX_PWR_REG) & 0xFFFFFFFD));
	/* power up RT */
	MMIO_WR(DP_EPHY_TX_PWR_REG, (MMIO_RD(DP_EPHY_TX_PWR_REG) | 0x00000001));
	/* disable RT reset */
	MMIO_WR(DP_EPHY_TX_PWR_REG, (MMIO_RD(DP_EPHY_TX_PWR_REG) | 0x00000002));
	via_chrome9_delay_usec(dev, 1000);
	/* enable RT auto calibration mode */
	MMIO_WR(DP_EPHY_TX_PWR_REG, (MMIO_RD(DP_EPHY_TX_PWR_REG) | 0x00000004));
	via_chrome9_delay_usec(dev, 1000);
	/* disable RT mode */
	MMIO_WR(DP_EPHY_TX_PWR_REG, (MMIO_RD(DP_EPHY_TX_PWR_REG) & 0xFFFFFFFB));
	via_chrome9_delay_usec(dev, 1000);
	/* power down RT */
	MMIO_WR(DP_EPHY_TX_PWR_REG, (MMIO_RD(DP_EPHY_TX_PWR_REG) & 0xFFFFFFFE));
	/* (6)
	* [19:8] Tx power status default settings for all 4 lanes
	* [7:0] Tx power down mode for all 4 lanes,
	* [15:8] HDMI TX output voltage swing 1200 mV for all 4 lanes
	*/
	data = MMIO_RD(DP_EPHY_TX_PWR_REG) & 0x0000F0FF;
	data |= 0xAAFF0F00;
	MMIO_WR(DP_EPHY_TX_PWR_REG, data);
	via_chrome9_delay_usec(dev, 4000);
	/* [7:0] Tx power control function CMOP on / PISO off for all 4 lanes */
	/* Tx power control function on for all 4 lanes */
	MMIO_WR(DP_EPHY_TX_PWR_REG, (MMIO_RD(DP_EPHY_TX_PWR_REG) & 0x00FFFFFF));
	via_chrome9_delay_usec(dev, 1000);
	/* Driver awaken for all 4 lanes */
	MMIO_WR(DP_EPHY_TX_PWR_REG, (MMIO_RD(DP_EPHY_TX_PWR_REG) | 0x0000F000));
	MMIO_WR(DP_EPHY_MISC_PWR_REG,
		(MMIO_RD(DP_EPHY_MISC_PWR_REG) | 0x00000001));
	via_chrome9_delay_usec(dev, 2000);

	return true;
}


void via_chrome9_dp_1_resume_pre_handle(struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct drm_connector_helper_funcs *via_dp_helper_funcs =
		(struct drm_connector_helper_funcs *)connector->helper_private;
	struct drm_encoder  *encoder;
	struct via_chrome9_encoder *via_chrome9_encoder = NULL;
	struct via_dp_encoder_private_info *via_dp_encoder_private_info = NULL;

	/* when resume from suspend,
	DP1 should to do init ephy and link training first. */
	if (VIA_CHROME9_DEVICE_CONNECTOR_DP_1 ==
		via_chrome9_connector->connector_id) {
		encoder = via_dp_helper_funcs->best_encoder(connector);
		if (encoder) {
			via_chrome9_encoder = to_via_chrome9_encoder(encoder);
			via_dp_encoder_private_info =
				(struct via_dp_encoder_private_info *)
				via_chrome9_encoder->encoder_private;

			via_dp_encoder_private_info->link_training_done = false;
			via_dp_encoder_private_info->link_speed = SPEED_NONE;

			via_chrome9_dp_1_init_ephy(connector);

			/* mainly do link training, and consider DP1 NoDDCValue
			option in detect after resume. */
			connector->funcs->detect(connector, true);

		} else {
			KMS_DEBUG("DP1 connector does not have encoder.\n");
		}
	}

}

int via_chrome9_dp_1_get_link_speed(struct drm_connector *connector)
{
	struct drm_connector_helper_funcs *via_dp_1_helper_funcs =
		(struct drm_connector_helper_funcs *)connector->helper_private;
	struct drm_encoder  *encoder =
		via_dp_1_helper_funcs->best_encoder(connector);
	struct via_chrome9_encoder *via_chrome9_encoder = NULL;
	struct via_dp_encoder_private_info *via_dp_encoder_private_info = NULL;
	u32 link_speed;

	if (encoder) {
		via_chrome9_encoder = to_via_chrome9_encoder(encoder);
		via_dp_encoder_private_info =
			(struct via_dp_encoder_private_info *)
			via_chrome9_encoder->encoder_private;
	} else {
		KMS_DEBUG("DP1 connector does not have encoder\n");
		return 0;
	}
	if (via_dp_encoder_private_info->link_training_done) {
		KMS_DEBUG("link training had done already, return directly!\n");
		return via_dp_encoder_private_info->link_speed;
	}

	via_chrome9_dp_1_read_dpcd(connector, 0x01);
	link_speed = MMIO_RD(AUX_R_DATA0_REG) & 0xFF;
	/* Let HW use AUX channel */
	MMIO_WR(AUX_TIMER_REG, 0x0);
	/* Use default link speed of DP monitor */
	switch (link_speed) {
	case 0x06:
		via_dp_encoder_private_info->link_speed = SPEED_162MHz;
		break;
	case 0x0A:
		via_dp_encoder_private_info->link_speed = SPEED_270MHz;
		break;
	default:
		via_dp_encoder_private_info->link_speed = SPEED_270MHz;
		break;
	}
	/* Start link training to confirm link speed */
	if ((!via_chrome9_dp_1_link_training(connector)) &&
		(via_dp_encoder_private_info->link_speed == SPEED_270MHz)) {
		KMS_DEBUG(
			"DP1 link training failed by 270MHz, now test again by 162MHz!\n");
		/* Reduce link speed to 162
		MHz and retry link training if 270 MHz link training fails */
		MMIO_WR(DP_LINK_TRAINING_REG,
			(MMIO_RD(DP_LINK_TRAINING_REG) & 0x7FFFFFFC));
		via_chrome9_dp_1_clear_aux_write_regs();
		/* Clear DPCD training pattern */
		via_chrome9_dp_1_write_dpcd(connector, 0x102, 1);
		via_dp_encoder_private_info->link_speed = SPEED_162MHz;
		/* Reset EPHY for changed link speed */
		via_chrome9_dp_1_init_ephy(connector);
		if (!via_chrome9_dp_1_link_training(connector)) {
			via_dp_encoder_private_info->link_speed = SPEED_NONE;
			via_dp_encoder_private_info->link_training_done = false;
			via_dp_encoder_private_info->dp_1_ephy_mode =
				via_unknow_mode;
			KMS_DEBUG("DP1 link training fails\n");
		}
	}

	return via_dp_encoder_private_info->link_speed;
}

bool via_chrome9_dp_1_read_edid(struct drm_connector *connector,
		u32 slave_addr, u32 dwOffset, u8 *data)
{
	u32 cmd;

	via_chrome9_dp_1_clear_aux_read_regs();
	via_chrome9_dp_1_clear_aux_write_regs();
	/* 1. write slave addr offset for start */
	cmd = dwOffset & 0x000000FF;
	MMIO_WR(AUX_W_DATA0_REG, cmd);
	cmd = slave_addr << 11 | 1 << 4 | AUX_CMD_I2C_WRITE;
	MMIO_WR(AUX_CMD_REG, cmd);
	/* issue request based on above */
	cmd = MMIO_RD(AUX_TIMER_REG) & 0xFFFFFFF0;
	cmd |= 0x03;
	MMIO_WR(AUX_TIMER_REG, cmd);
	if (!via_chrome9_dp_1_reply_cmd_status(connector))
		return false;

	/* 2. write command with read length */
	cmd = slave_addr << 11 | 1 << 4  | AUX_CMD_I2C_READ;
	MMIO_WR(AUX_CMD_REG, cmd);
	/* issue request based on above */
	cmd = MMIO_RD(AUX_TIMER_REG) & 0xFFFFFFF0;
	cmd |= 0x03;
	MMIO_WR(AUX_TIMER_REG, cmd);
	if (!via_chrome9_dp_1_reply_cmd_status(connector))
		return false;

	/* 3. read */
	*data = (MMIO_RD(AUX_R_DATA0_REG) & 0xFF);
	return true;
}

void via_chrome9_get_dp_1_edid_block_data(struct drm_connector *connector,
		u8 *edid_data, int edid_size)
{
	int i = 0;
	/* Raw Data */
	for (i = 0; i < edid_size; i++)
		via_chrome9_dp_1_read_edid(connector, 0xA0, i, edid_data+i);
}


static enum drm_connector_status via_chrome9_dp_1_detect(
		struct drm_connector *connector,  bool force)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_dp_connector_private_info *via_dp_connector_private_info =
		(struct via_dp_connector_private_info *)
		via_chrome9_connector->connector_private;
	struct via_dp_options *via_dp_options =
		(struct via_dp_options *)via_chrome9_connector->options;
	enum drm_connector_status ret = connector_status_disconnected;
	if (via_dp_options->no_ddc_value) {
		ret = connector_status_connected;
		kfree(via_chrome9_connector->edid);
		via_chrome9_connector->edid = NULL;
		connector->display_info.raw_edid = NULL;
	} else {
		via_chrome9_dp_1_read_dpcd_version(connector);
		if (via_dp_connector_private_info->dpcd_version) {
			ret = connector_status_connected;
			KMS_DEBUG("dp1 is connected\n");
		} else {
			kfree(via_chrome9_connector->edid);
			via_chrome9_connector->edid = NULL;
			connector->display_info.raw_edid = NULL;
			KMS_DEBUG("dp1 is not connected\n");
		}
	}

	if (connector_status_connected == ret)
		via_dp_connector_private_info->link_speed =
			via_chrome9_dp_1_get_link_speed(connector);

	KMS_DEBUG("dp1 detect: %sconnected!\n",
		((ret == connector_status_connected) ? "" : "dis"));
	return ret;
}

int via_chrome9_dp_1_set_property(struct drm_connector *connector,
		struct drm_property *property,
		uint64_t value)
{
	/* implement later if necessary */
	return 0;
}

int via_chrome9_dp_1_get_modes(struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct drm_device *dev = connector->dev;
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	struct via_dp_connector_private_info *dp_private_info =
		(struct via_dp_connector_private_info *)
		via_chrome9_connector->connector_private;
	struct via_dp_options *via_dp_options =
		(struct via_dp_options *)via_chrome9_connector->options;
	int ret = 0, i = 0, edid_size = 0;
	u8 *block = NULL;
	u8 edid_ext_num = 0;
	bool is_valid = false;
	if (via_dp_options->no_ddc_value) {
		dp_private_info->physical_width = 4096;
		dp_private_info->physical_height = 4096;
	} else {
		if (!via_chrome9_connector->edid) {
			if (!via_chrome9_dp_1_read_edid(connector,
				0xA0, 0x7E, &edid_ext_num)) {
				KMS_DEBUG(
					"dp1 edid extention num read fail!\n");
				/* set default value to 0 */
				edid_ext_num = 0;
			} else {
				KMS_DEBUG(
					"dp1 edid ext_num is %d", edid_ext_num);
				if (edid_ext_num > 254) {
					KMS_DEBUG(
						"dp1 edid extention num large than 254, not correct\n");
					/* set default value to 0 */
					edid_ext_num = 0;
				}
			}
			edid_size = EDID_LENGTH * (1 + edid_ext_num);
			block = kzalloc(edid_size, GFP_KERNEL);
			if (block == NULL) {
				KMS_DEBUG("edid block alloc fail!\n");
				return -1;
			} else {
				via_chrome9_connector->edid =
					(struct edid *)block;
			}
			if (p_priv->pseudo_chip_revision > REVISION_VX900_A1) {
				KMS_DEBUG("chip revision is %x",
					p_priv->pseudo_chip_revision);

				/* Parse the EDID block
				*  read edid 5 times most to
				*   avoid edid read fail
				*/
				for (i = 0; i < 5; i++) {
					via_chrome9_get_dp_1_edid_block_data(
						connector, block, edid_size);
					is_valid = drm_edid_is_valid(
						via_chrome9_connector->edid);
					if (!is_valid) {
						memset(block, 0, edid_size);
					} else {
						/* edid read success,
						break here */
						break;
					}
				}
				if (5 == i)
					KMS_DEBUG(
						"The dp1 monitor edid read fail for five times !!!\n");
			}
			connector->display_info.raw_edid = block;
		}

		if (via_chrome9_connector->edid) {
			drm_mode_connector_update_edid_property(
				&via_chrome9_connector->base,
				via_chrome9_connector->edid);
			ret = drm_add_edid_modes(&via_chrome9_connector->base,
				via_chrome9_connector->edid);
		}

		via_chrome9_get_monitor_physical_size(
			connector,
			&dp_private_info->physical_width,
			&dp_private_info->physical_height);
	}
	return ret;
}

int via_chrome9_dp_1_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_dp_connector_private_info *dp_private_info =
		(struct via_dp_connector_private_info *)
		via_chrome9_connector->connector_private;
	int link_speed = dp_private_info->link_speed;

	/* Check Clock Range */
	if (mode->clock > 400000)
		return MODE_CLOCK_HIGH;
	if (mode->clock < 25000)
		return MODE_CLOCK_LOW;
	/* Check monitor physical size */
	if (mode->hdisplay > dp_private_info->physical_width)
		return MODE_BAD_HVALUE;
	if (mode->vdisplay > dp_private_info->physical_height)
		return MODE_BAD_VVALUE;
	/* Check Resolution Range */
	if (via_chrome9_is_valid_mode(mode->hdisplay,
		mode->vdisplay) == 0)
		return MODE_BAD;
	/* Hardware limitation: TU ratio only 15 bits. */
	dp_private_info->tu_ratio =
		via_chrome9_calculate_tu(link_speed, mode->clock, 24);

	if (dp_private_info->tu_ratio > 0x7FFF)
		return MODE_BAD;

	return MODE_OK;
}

static struct drm_connector_funcs
	via_chrome9_dp_1_connector_funcs = {
	.dpms = drm_helper_connector_dpms,
	.detect = via_chrome9_dp_1_detect,
	.fill_modes = drm_helper_probe_single_connector_modes,
	.destroy = via_chrome9_connector_destory,
	.set_property = via_chrome9_dp_1_set_property,
};

static struct drm_connector_helper_funcs
	via_chrome9_dp_1_connector_helper_funcs = {
	.get_modes = via_chrome9_dp_1_get_modes,
	.mode_valid = via_chrome9_dp_1_mode_valid,
	.best_encoder = via_chrome9_best_single_encoder,
};

void via_chrome9_dp_2_clear_aux_write_regs(void)
{
	MMIO_WR(DP2_AUX_W_DATA0_REG, 0x0);
	MMIO_WR(DP2_AUX_W_DATA1_REG, 0x0);
	MMIO_WR(DP2_AUX_W_DATA2_REG, 0x0);
	MMIO_WR(DP2_AUX_W_DATA3_REG, 0x0);
}

void via_chrome9_dp_2_clear_aux_read_regs(void)
{
	MMIO_WR(DP2_AUX_R_DATA0_REG, 0x0);
	MMIO_WR(DP2_AUX_R_DATA1_REG, 0x0);
	MMIO_WR(DP2_AUX_R_DATA2_REG, 0x0);
	MMIO_WR(DP2_AUX_R_DATA3_REG, 0x0);
}

bool via_chrome9_dp_2_reply_cmd_status(struct drm_connector *connector)
{
	struct drm_device *dev = connector->dev;
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	u32 aux_cmd_status;
	u32 data;
	int i, j;

	for (i = 0; i < NUM_RE_ISSUE_COMMAND; i++) {
		for (j = 0; j < NUM_RECHECK; j++) {
			via_chrome9_delay_usec(dev, 1200);
			aux_cmd_status = MMIO_RD(DP2_AUX_TIMER_REG) &
				0x3C000007;
			if (p_priv->pseudo_chip_revision > REVISION_VX900_A1 &&
				ACK_OK == aux_cmd_status) {
				return true;
			} else if (0x00000001 == aux_cmd_status) {
				/* hardware issue */
				return true;
			} else if (RE_ISSUE_CMD == (aux_cmd_status & 0xFFFF)) {
				/* re-issue Cmd */
				data = MMIO_RD(DP2_AUX_TIMER_REG) & 0xFFFFFFF0;
				data |= 0x03;
				MMIO_WR(DP2_AUX_TIMER_REG, data);
				break;
			}
		}
	}
	return false;
}

bool via_chrome9_dp2_read_dpcd(struct drm_connector *connector, u32 offset)
{
	u32 cmd;
	u32 data;

	via_chrome9_dp_2_clear_aux_read_regs();
	/* assign command */
	/* AUX read offset */
	cmd = offset << 12;
	/* AUX read length */
	cmd |= 1 << 4;
	/* AUX read CMD */
	cmd |= AUX_CMD_READ;
	MMIO_WR(DP2_AUX_CMD_REG, cmd);
	/* issue request based on above */
	data = MMIO_RD(DP2_AUX_TIMER_REG) & 0xFFFFFFF0;
	data |= 0x03;
	MMIO_WR(DP2_AUX_TIMER_REG, data);
	return via_chrome9_dp_2_reply_cmd_status(connector);
}

void via_chrome9_dp_2_read_dpcd_version(struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_dp_connector_private_info *via_dp_connector_private_info =
		(struct via_dp_connector_private_info *)
		via_chrome9_connector->connector_private;
	u32 dpcd_version;

	via_dp_connector_private_info->dpcd_version = NONEVERSION;

	/* read DPCD Version */
	if (via_chrome9_dp2_read_dpcd(connector, 0x00)) {
		KMS_DEBUG("via chrome9 dp2 read dpcd is sucess\n");
		dpcd_version = MMIO_RD(DP2_AUX_R_DATA0_REG) & 0xFF;
		KMS_DEBUG("via chrome9 dp2 version is %d\n", dpcd_version);
		switch (dpcd_version) {
		case 0x10:
			/* Version 1.0 */
			via_dp_connector_private_info->dpcd_version |=
				DPCD_VER_1 | DPCD_REV_0;
			break;
		case 0x11:
			/* Version 1.1 */
			via_dp_connector_private_info->dpcd_version |=
				DPCD_VER_1 | DPCD_REV_1;
			break;
		default:
			via_dp_connector_private_info->dpcd_version =
				NONEVERSION;
			break;
		}
	} else {
		KMS_DEBUG("via chrome9 dp2 read dpcd is fail\n");
	}
	KMS_DEBUG("dp2 dpcd version is:%d\n",
		via_dp_connector_private_info->dpcd_version);
	/* Let HW use AUX channel */
	MMIO_WR(DP2_AUX_TIMER_REG, 0x0);
}

void via_chrome9_dp_2_turn_on_aux_power_sequence(
		struct drm_connector *connector)
{
	struct drm_device *dev = connector->dev;
	/* Delay TD0 time to wait HPD signal from sink */
	via_chrome9_delay_usec(dev, 10);
	/* Delay TD5 time */
	via_chrome9_delay_usec(dev, 10);
}

void via_chrome9_dp_2_turn_off_aux_power_sequence(
		struct drm_connector *connector)
{
	struct drm_device *dev = connector->dev;
	/* Delay TD5 time */
	via_chrome9_delay_usec(dev, 10);
}

bool via_chrome9_dp_2_write_dpcd(struct drm_connector *connector,
		u32 offset,
		int byte_count)
{
	struct drm_device *dev = connector->dev;
	u32 cmd;
	u32 data;
	/* if EPHY disable, turn on power sequence */
	if (!(MMIO_RD(DP2_ENABLE_IF_REG) & BIT0))
		via_chrome9_dp_2_turn_on_aux_power_sequence(connector);

	/* AUX read offset */
	cmd = offset << 12;
	/* AUX read length */
	cmd |= byte_count << 4;
	/* AUX write CMD */
	cmd |= AUX_CMD_WRITE;
	MMIO_WR(DP2_AUX_CMD_REG, cmd);

	/* issue request based on above */
	data = MMIO_RD(DP2_AUX_TIMER_REG) & 0xFFFFFFF0;
	data |= 0x03;
	/* trigger */
	MMIO_WR(DP2_AUX_TIMER_REG, data);
	via_chrome9_delay_usec(dev, 10000);
	/* when leave, turn off power sequence */
	if (!(MMIO_RD(DP2_ENABLE_IF_REG) & BIT0))
		via_chrome9_dp_2_turn_off_aux_power_sequence(connector);

	return via_chrome9_dp_2_reply_cmd_status(connector);
}

bool via_chrome9_dp_2_read_dpcd(struct drm_connector *connector, u32 offset)
{
	u32 cmd;
	u32 data;

	via_chrome9_dp_2_clear_aux_read_regs();
	/* assign command */
	/* AUX read offset */
	cmd = offset << 12;
	/* AUX read length */
	cmd |= 1 << 4;
	/* AUX read CMD */
	cmd |= AUX_CMD_READ;
	MMIO_WR(DP2_AUX_CMD_REG, cmd);
	/* issue request based on above */
	data = MMIO_RD(DP2_AUX_TIMER_REG) & 0xFFFFFFF0;
	data |= 0x03;
	MMIO_WR(DP2_AUX_TIMER_REG, data);
	return via_chrome9_dp_2_reply_cmd_status(connector);
}

u8 via_chrome9_dp_2_read_dpcd_link_status(
		struct drm_connector *connector)
{
	u32  dwData;
	u32  dwLaneAlign;
	u32   dwLaneStatus[4];
	u8 linkStatus = DP_LINKSTATUS_DONE;
	int i;

	/* read DPCD Lane Status */
	via_chrome9_dp_2_read_dpcd(connector, 0x202);
	dwData = MMIO_RD(DP2_AUX_R_DATA0_REG) & 0xFF;
	dwLaneStatus[0] = dwData & 0xF;
	dwLaneStatus[1] = (dwData >> 4) & 0xF;

	via_chrome9_dp_2_read_dpcd(connector, 0x203);
	dwData = MMIO_RD(DP2_AUX_R_DATA0_REG) & 0xFF;
	dwLaneStatus[2] = dwData & 0xF;
	dwLaneStatus[3] = (dwData >> 4) & 0xF;

	via_chrome9_dp_2_read_dpcd(connector, 0x204);
	dwData = MMIO_RD(DP2_AUX_R_DATA0_REG) & 0xFF;
	dwLaneAlign = dwData & BIT0;

	if (!dwLaneAlign) {
		linkStatus = DP_LINKSTATUS_LOST;
	} else {
		for (i = 0; i < LANE; i++) {
			if (dwLaneStatus[i] != 0x7) {
				linkStatus = DP_LINKSTATUS_LOST;
				break;
			}
		}
	}
	KMS_DEBUG("DP_LINKSTATUS: %s\n",
		((linkStatus == DP_LINKSTATUS_LOST) ? "LOST" : "LINK"));
	/* Let HW use AUX channel */
	MMIO_WR(DP2_AUX_TIMER_REG, 0x0);
	return linkStatus;
}


bool via_chrome9_dp_2_link_training(struct drm_connector *connector)
{
	struct drm_device *dev = connector->dev;
	struct drm_connector_helper_funcs *via_dp_2_helper_funcs =
		(struct drm_connector_helper_funcs *)connector->helper_private;
	struct drm_encoder  *encoder =
		via_dp_2_helper_funcs->best_encoder(connector);
	struct via_chrome9_encoder *via_chrome9_encoder = NULL;
	struct via_dp_encoder_private_info *via_dp_encoder_private_info = NULL;
	u32 data;

	if (encoder) {
		via_chrome9_encoder = to_via_chrome9_encoder(encoder);
		via_dp_encoder_private_info =
			(struct via_dp_encoder_private_info *)
			via_chrome9_encoder->encoder_private;
	} else {
		KMS_DEBUG(
			"DP1 connector does not have encoder\n");
		return 0;
	}
	via_chrome9_dp_2_clear_aux_read_regs();
	via_chrome9_dp_2_clear_aux_write_regs();
	via_dp_encoder_private_info->link_training_done = false;
	/* reset link training */
	MMIO_WR_MASK(DP_DATA_PASS_ENABLE_REG, 0x01, 0x01);
	MMIO_WR_MASK(DP2_NAUD_MUTE_REG, 0x80000000, 0x80000000);
	via_chrome9_delay_usec(dev, 1000);
	MMIO_WR_MASK(DP2_NAUD_MUTE_REG, 0x0, 0x80000000);
	via_chrome9_delay_usec(dev, 1000);

	/* [9:7]=100b, 4 lanes, [10]=0, disable SW link training ,
	* [12:11]=01b, set clock recovery state
	*/
	data = MMIO_RD(DP2_LINK_TRAINING_REG) & 0xFFFF0000;
	data |= 0x0000023C;

	switch (via_dp_encoder_private_info->link_speed) {
	case SPEED_162MHz:
		data &= 0xFFFFFFFD;
		break;
	case SPEED_270MHz:
		data |= 0x00000002;
		break;
	default:
		data |= 0x00000002;
		break;
	}
	MMIO_WR(DP2_LINK_TRAINING_REG, data);
	/* Reset EPHY */
	MMIO_WR_MASK(DP2_EPHY_TX_PWR_REG, 0x00000055, 0x000000FF);
	via_chrome9_delay_usec(dev, 2000);
	MMIO_WR_MASK(DP2_EPHY_TX_PWR_REG, 0x00000000, 0x000000FF);
	/* enable scramble */
	MMIO_WR_MASK(DP2_VIDEO_CTRL_REG, 0x00000001, 0x00000001);
	/* enable DP, set enhanced mode */
	data = MMIO_RD(DP2_ENABLE_IF_REG) & 0xC3000000;
	/* if enable standard mode , set 0x0000000d */
	data |= 0x0000000d;
	MMIO_WR(DP2_ENABLE_IF_REG, data);

	switch (via_dp_encoder_private_info->link_speed) {
	case SPEED_162MHz:
		/* if enable standard mode , set 0x00018406 */
		MMIO_WR(DP2_AUX_W_DATA0_REG, 0x00018406);
		break;
	case SPEED_270MHz:
		/* if enable standard mode , set 0x0001840A */
		MMIO_WR(DP2_AUX_W_DATA0_REG, 0x0001840A);
		break;
	default:
		MMIO_WR(DP2_AUX_W_DATA0_REG, 0x0001840A);
		break;
	}
	MMIO_WR(DP2_AUX_W_DATA1_REG, 0x0);
	if (!via_chrome9_dp_2_write_dpcd(connector, 0x100, 2)) {
		/* Let HW use AUX channel */
		MMIO_WR(DP2_AUX_TIMER_REG, 0x0);
		return false;
	}
	/* Let HW use AUX channel */
	MMIO_WR(DP2_AUX_TIMER_REG, 0x0);
	via_chrome9_delay_usec(dev, 20000);
	data = MMIO_RD(DP2_LINK_TRAINING_REG) & 0x7FFFFFFF;
	data |= 0x0000003C;
	if (SPEED_270MHz == via_dp_encoder_private_info->link_speed)
		data |= 0x00000002;

	MMIO_WR(DP2_LINK_TRAINING_REG, data);
	/* enable HW link training */
	MMIO_WR_MASK(DP2_LINK_TRAINING_REG, 0x00000001, 0x00000001);
	via_chrome9_delay_usec(dev, 20000);
	if ((MMIO_RD(DP2_LINK_TRAINING_REG) & 0x80000000) != 0x80000000) {
		/* Let HW use AUX channel */
		MMIO_WR(DP2_AUX_TIMER_REG, 0x0);
		return false;
	}
	via_chrome9_delay_usec(dev, 4000);
	if ((MMIO_RD(DP2_ENABLE_IF_REG) & 0x30000000) != 0x30000000) {
		/* Let HW use AUX channel */
		MMIO_WR(DP2_AUX_TIMER_REG, 0x0);
		return false;
	}
	via_dp_encoder_private_info->link_training_done = true;
	/* clear [31], and [0] */
	MMIO_WR_MASK(DP2_LINK_TRAINING_REG, 0x0, 0x80000001);
	/* Let HW use AUX channel */
	MMIO_WR(DP2_AUX_TIMER_REG, 0x0);

	return true;
}

bool via_chrome9_dp_2_init_ephy(struct drm_connector *connector)
{
	struct drm_device *dev = connector->dev;
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_dp_connector_private_info *via_dp_connector_private_info =
		(struct via_dp_connector_private_info *)
		via_chrome9_connector->connector_private;

	u32 data;

	/* (1) enable Bandgap and set EPHY symbol rate */
	data = MMIO_RD(DP2_EPHY_PLL_REG) & 0x0C7000FF;
	/* real EPHY symbol rate decided by HW link training */
	switch (via_dp_connector_private_info->link_speed) {
	case SPEED_162MHz:
		data |= 0x0002BE00;
		break;
	case SPEED_270MHz:
		data |= 0x00019E00;
		break;
	default:
		data |= 0x00019E00;
		break;
	}
	data |= 0xF1080001;
	MMIO_WR(DP2_EPHY_PLL_REG, data);
	via_chrome9_delay_usec(dev, 1000);
	/* (2) start AUX channel, CMOP is on, Tx and Rx are off */
	data = MMIO_RD(DP2_EPHY_TX_PWR_REG) & 0xCFFFFFFF;
	data |= 0x10000000;
	MMIO_WR(DP2_EPHY_TX_PWR_REG, data);
	via_chrome9_delay_usec(dev, 4000);
	data = MMIO_RD(DP2_EPHY_TX_PWR_REG) & 0xCFFFFFFF;
	data |= 0x20000000;
	MMIO_WR(DP2_EPHY_TX_PWR_REG, data);
	via_chrome9_delay_usec(dev, 4000);
	/* (3) enable Mpll and Mpll reg power
	(4) enable Tpll and Tpll reg power*/
	MMIO_WR(DP2_EPHY_PLL_REG, (MMIO_RD(DP2_EPHY_PLL_REG) & (~0x02000000)));
	via_chrome9_delay_usec(dev, 1000);
	/* (5) Resister Tuning (RT), RT reset
	* (6) [19:8] Tx power status default settings for all 4 lanes*/
	data = MMIO_RD(DP2_EPHY_TX_PWR_REG2) & 0xFFF000FF;
	data |= 0x0000AA00;
	MMIO_WR(DP2_EPHY_TX_PWR_REG2, data);
	/* [7:0] Tx power down mode for all 4 lanes,
	[15:8] HDMI TX output voltage swing 1200 mV for all 4 lanes */
	data = MMIO_RD(DP2_EPHY_TX_PWR_REG) & 0xFFFF0000;
	data |= 0x0000FFFF;
	MMIO_WR(DP2_EPHY_TX_PWR_REG, data);
	via_chrome9_delay_usec(dev, 4000);
	/* [7:0] Tx power control function CMOP on / PISO off for all 4 lanes */
	/* Tx power control function on for all 4 lanes */
	MMIO_WR(DP2_EPHY_TX_PWR_REG,
		(MMIO_RD(DP2_EPHY_TX_PWR_REG) & 0xFFFFFFAA));
	via_chrome9_delay_usec(dev, 4000);
	/* [7:0] Tx power control function on for all 4 lanes */
	/* Tx power control function on for all 4 lanes */
	MMIO_WR(DP2_EPHY_TX_PWR_REG,
		(MMIO_RD(DP2_EPHY_TX_PWR_REG) & 0xFFFFFF00));
	via_chrome9_delay_usec(dev, 1000);
	/* [19:16] Driver awaken for all 4 lanes */
	MMIO_WR(DP2_EPHY_TX_IDLE_REG,
		(MMIO_RD(DP2_EPHY_TX_IDLE_REG) | 0x000F0000));
	/* (7) DP2 enable */
	MMIO_WR(DP2_ENABLE_IF_REG,
		(MMIO_RD(DP2_ENABLE_IF_REG) | 0x00000001));
	via_chrome9_delay_usec(dev, 2000);

	return true;
}


void via_chrome9_dp_2_resume_pre_handle(struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct drm_connector_helper_funcs *via_dp_helper_funcs =
		(struct drm_connector_helper_funcs *)connector->helper_private;
	struct drm_encoder  *encoder;
	struct via_chrome9_encoder *via_chrome9_encoder = NULL;
	struct via_dp_encoder_private_info *via_dp_encoder_private_info = NULL;

	/* when resume from suspend,
	DP2 should to do init ephy and link training first. */
	if (VIA_CHROME9_DEVICE_CONNECTOR_DP_2 ==
		via_chrome9_connector->connector_id) {
		encoder = via_dp_helper_funcs->best_encoder(connector);
		if (encoder) {
			via_chrome9_encoder = to_via_chrome9_encoder(encoder);
			via_dp_encoder_private_info =
				(struct via_dp_encoder_private_info *)
				via_chrome9_encoder->encoder_private;

			via_dp_encoder_private_info->link_training_done = false;
			via_dp_encoder_private_info->link_speed = SPEED_NONE;

			via_chrome9_dp_2_init_ephy(connector);

			/* mainly do link training, and consider DP2
			NoDDCValue option in detect after resume. */
			connector->funcs->detect(connector, true);
		} else {
			KMS_DEBUG("DP2 connector does not have encoder.\n");
		}
	}
}

int via_chrome9_dp_2_get_link_speed(struct drm_connector *connector)
{
	struct drm_connector_helper_funcs *via_dp_2_helper_funcs =
		(struct drm_connector_helper_funcs *)connector->helper_private;
	struct drm_encoder  *encoder =
		via_dp_2_helper_funcs->best_encoder(connector);
	struct via_chrome9_encoder *via_chrome9_encoder = NULL;
	struct via_dp_encoder_private_info *via_dp_encoder_private_info = NULL;
	u32 link_speed;

	if (encoder) {
		via_chrome9_encoder = to_via_chrome9_encoder(encoder);
		via_dp_encoder_private_info =
			(struct via_dp_encoder_private_info *)
			via_chrome9_encoder->encoder_private;
	} else {
		KMS_DEBUG("DP2 connector does not have encoder\n");
		return 0;
	}

	if (via_dp_encoder_private_info->link_training_done) {
		KMS_DEBUG("link training had done already, return directly!\n");
		return via_dp_encoder_private_info->link_speed;
	}

	via_chrome9_dp_2_read_dpcd(connector, 0x01);
	link_speed = MMIO_RD(DP2_AUX_R_DATA0_REG) & 0xFF;
	/* Let HW use AUX channel */
	MMIO_WR(DP2_AUX_TIMER_REG, 0x0);
	/* Use default link speed of DP monitor */
	switch (link_speed) {
	case 0x06:
		via_dp_encoder_private_info->link_speed = SPEED_162MHz;
		break;
	case 0x0A:
		via_dp_encoder_private_info->link_speed = SPEED_270MHz;
		break;
	default:
		via_dp_encoder_private_info->link_speed = SPEED_270MHz;
		break;
	}
	/* Start link training to confirm link speed */
	if ((!via_chrome9_dp_2_link_training(connector)) &&
	(SPEED_270MHz == via_dp_encoder_private_info->link_speed)) {
		KMS_DEBUG(
			"DP2 link training failed by 270MHz, now test again by 162MHz!\n");
		/* Reduce link speed to 162 MHz and
		*   retry link training if 270 MHz link training fails
		*/
		MMIO_WR(DP2_LINK_TRAINING_REG,
			(MMIO_RD(DP2_LINK_TRAINING_REG) & 0x7FFFFFFC));
		via_chrome9_dp_2_clear_aux_write_regs();
		/* Clear DPCD training pattern */
		via_chrome9_dp_2_write_dpcd(connector, 0x102, 1);
		via_dp_encoder_private_info->link_speed = SPEED_162MHz;
		/* Reset EPHY for changed link speed */
		via_chrome9_dp_2_init_ephy(connector);
		if (!via_chrome9_dp_2_link_training(connector)) {
			via_dp_encoder_private_info->link_speed = SPEED_NONE;
			via_dp_encoder_private_info->link_training_done = false;
			KMS_DEBUG("DP2 link training fails\n");
		}
	}
	return via_dp_encoder_private_info->link_speed;
}

bool via_chrome9_dp_2_read_edid(struct drm_connector *connector,
		u32 slave_addr, u32 dwOffset, u8 *data)
{
	u32 cmd;

	via_chrome9_dp_2_clear_aux_read_regs();
	via_chrome9_dp_2_clear_aux_write_regs();
	/* 1. write slave addr offset for start */
	cmd = dwOffset & 0x000000FF;
	MMIO_WR(DP2_AUX_W_DATA0_REG, cmd);
	cmd = slave_addr << 11 | 1 << 4 | AUX_CMD_I2C_WRITE;
	MMIO_WR(DP2_AUX_CMD_REG, cmd);
	/* AUX trigger */
	MMIO_WR(DP2_AUX_TIMER_REG, (MMIO_RD(DP2_AUX_TIMER_REG)|0x3));
	if (!via_chrome9_dp_2_reply_cmd_status(connector))
		return false;

	/* 2. write command with read length */
	cmd = slave_addr << 11 | 1 << 4 | AUX_CMD_I2C_READ;
	MMIO_WR(DP2_AUX_CMD_REG, cmd);
	 /* AUX trigger */
	MMIO_WR(DP2_AUX_TIMER_REG, (MMIO_RD(DP2_AUX_TIMER_REG)|0x3));
	if (!via_chrome9_dp_2_reply_cmd_status(connector))
		return false;

	/* 3. read */
	*data = (MMIO_RD(DP2_AUX_R_DATA0_REG) & 0xFF);
	return true;
}

void via_chrome9_get_dp_2_edid_block_data(struct drm_connector *connector,
		u8 *edid_data, int edid_size)
{
	int i = 0;
	KMS_DEBUG("edid_size is %d", edid_size);
	/* Read Block 0 Raw Data */
	for (i = 0; i < edid_size; i++)
		via_chrome9_dp_2_read_edid(connector, 0xA0, i, edid_data+i);
}

static enum drm_connector_status via_chrome9_dp_2_detect(
		struct drm_connector *connector, bool force)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_dp_connector_private_info *via_dp_connector_private_info =
		(struct via_dp_connector_private_info *)
		via_chrome9_connector->connector_private;
	struct via_dp_options *via_dp_options =
		(struct via_dp_options *)via_chrome9_connector->options;
	enum drm_connector_status ret = connector_status_disconnected;

	if (via_dp_options->no_ddc_value) {
		ret = connector_status_connected;
		kfree(via_chrome9_connector->edid);
		via_chrome9_connector->edid = NULL;
		connector->display_info.raw_edid = NULL;
	} else {
		via_chrome9_dp_2_read_dpcd_version(connector);
		if (via_dp_connector_private_info->dpcd_version) {
			ret = connector_status_connected;
			KMS_DEBUG("dp2 is connected\n");
		} else {
			kfree(via_chrome9_connector->edid);
			via_chrome9_connector->edid = NULL;
			connector->display_info.raw_edid = NULL;
			KMS_DEBUG("dp2 is not connected\n");
		}
	}
	if (connector_status_connected == ret)
		via_dp_connector_private_info->link_speed =
			via_chrome9_dp_2_get_link_speed(connector);

	KMS_DEBUG("dp2 detect: %sconnected!\n",
		((ret == connector_status_connected) ? "" : "dis"));
	return ret;
}

int via_chrome9_dp_2_set_property(struct drm_connector *connector,
		struct drm_property *property,
		uint64_t value)
{
	/* implement later if necessary */
	return 0;
}

int via_chrome9_dp_2_get_modes(struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct drm_device *dev = connector->dev;
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	struct via_dp_connector_private_info *dp_private_info =
		(struct via_dp_connector_private_info *)
		via_chrome9_connector->connector_private;
	struct via_dp_options *via_dp_options =
		(struct via_dp_options *)via_chrome9_connector->options;
	int ret = 0, i = 0;
	int edid_size = 0;
	u8 *block = NULL;
	u8 edid_ext_num = 0;
	bool is_valid = false;

	if (via_dp_options->no_ddc_value) {
		dp_private_info->physical_width = 4096;
		dp_private_info->physical_height = 4096;

	} else {
		if (!via_chrome9_connector->edid) {
			if (!via_chrome9_dp_2_read_edid(connector, 0xA0, 0x7E,
				&edid_ext_num)) {
				KMS_DEBUG(
					"dp2 edid extention num read fail!\n");
				/* set default value to 0 */
				edid_ext_num = 0;
			} else {
				KMS_DEBUG("dp2 edid ext_num is %d",
					edid_ext_num);

				if (edid_ext_num > 254) {
					KMS_DEBUG(
						"dp2 edid extention num large than 254, not correct\n");
					/* set default value to 0 */
					edid_ext_num = 0;
				}
			}
			edid_size = EDID_LENGTH * (1 + edid_ext_num);
			block = kzalloc(edid_size, GFP_KERNEL);

			if (block == NULL) {
				KMS_DEBUG("edid block alloc fail!\n");
				return -1;
			} else {
				via_chrome9_connector->edid =
					(struct edid *)block;
			}

			if (p_priv->pseudo_chip_revision > REVISION_VX900_A1) {
				/* Remove the comment when upstream!! */
				/* Parse the EDID block */
				/* read edid 5 times most to avoid
				edid read fail */
				for (i = 0; i < 5; i++) {
					via_chrome9_get_dp_2_edid_block_data(
						connector, block, edid_size);
					is_valid = drm_edid_is_valid(
						via_chrome9_connector->edid);

					if (!is_valid) {
						memset(block, 0, edid_size);
					} else {
						/*edid read success,
						*  break here
						*/
						break;
					}
				}

				if (5 == i)
					KMS_DEBUG(
						"The dp2 monitor edid read fail for five times !!!\n");
			}
			connector->display_info.raw_edid = block;
		}

		if (via_chrome9_connector->edid) {
			drm_mode_connector_update_edid_property(
				&via_chrome9_connector->base,
				via_chrome9_connector->edid);
			ret = drm_add_edid_modes(&via_chrome9_connector->base,
				via_chrome9_connector->edid);
		}

		via_chrome9_get_monitor_physical_size(connector,
			&dp_private_info->physical_width,
			&dp_private_info->physical_height);
	}

	return ret;

}

int via_chrome9_dp_2_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_dp_connector_private_info *dp_private_info =
		(struct via_dp_connector_private_info *)
		via_chrome9_connector->connector_private;
	int link_speed = dp_private_info->link_speed;

	/* Check Clock Range */
	if (mode->clock > 400000)
		return MODE_CLOCK_HIGH;
	if (mode->clock < 25000)
		return MODE_CLOCK_LOW;
	/* Check monitor physical size */
	if (mode->hdisplay > dp_private_info->physical_width)
		return MODE_BAD_HVALUE;
	if (mode->vdisplay > dp_private_info->physical_width)
		return MODE_BAD_VVALUE;
	/* Check Resolution Range */
	if (via_chrome9_is_valid_mode(mode->hdisplay, mode->vdisplay) == 0)
		return MODE_BAD;
	/* Hardware limitation: TU ratio only 15 bits. */
	dp_private_info->tu_ratio =
		via_chrome9_calculate_tu(link_speed, mode->clock, 24);
	if (dp_private_info->tu_ratio > 0x7FFF)
		return MODE_BAD;

	return MODE_OK;

}

static struct drm_connector_funcs via_chrome9_dp_2_connector_funcs = {
	.dpms = drm_helper_connector_dpms,
	.detect = via_chrome9_dp_2_detect,
	.fill_modes = drm_helper_probe_single_connector_modes,
	.destroy = via_chrome9_connector_destory,
	.set_property = via_chrome9_dp_2_set_property,
};

static struct drm_connector_helper_funcs
	via_chrome9_dp_2_connector_helper_funcs = {
	.get_modes = via_chrome9_dp_2_get_modes,
	.mode_valid = via_chrome9_dp_2_mode_valid,
	.best_encoder = via_chrome9_best_single_encoder,
};

void via_chrome9_set_dvi_xorg_options(void *options, int size,
		struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_dvi_options *via_dvi_options =
		(struct via_dvi_options *)via_chrome9_connector->options;
	struct drm_connector_helper_funcs *connector_func =
		connector->helper_private;
	struct via_chrome9_external_card *card = NULL;
	struct i2c_client *client = NULL;
	struct via_gfx_dpa via_gfx_dpa;
	struct drm_encoder *encoder = connector_func->best_encoder(connector);
	copy_from_user(via_dvi_options, options, size);
	KMS_DEBUG("\n");
	if (encoder) {
		card = to_via_chrome9_external_card(encoder);
		if (card) {
			client = via_chrome9_external_card_to_i2c_client(card);
			via_gfx_dpa.clk_adjust =
				via_dvi_options->clk_adjust;
			via_gfx_dpa.clk_driving_sel =
				via_dvi_options->clk_driving_sel;
			via_gfx_dpa.clk_polarity =
				via_dvi_options->clk_polarity;
			via_gfx_dpa.data_driving_sel =
				via_dvi_options->data_driving_sel;
			via_gfx_dpa.is_clk_adjust_used =
				via_dvi_options->is_clk_adjust_used;
			via_gfx_dpa.is_clk_driving_sel_used =
				via_dvi_options->is_clk_driving_sel_used;
			via_gfx_dpa.is_clk_polarity_used =
				via_dvi_options->is_clk_polarity_used;
			via_gfx_dpa.is_data_driving_sel_used =
				via_dvi_options->is_data_driving_sel_used;
			via_chrome9_load_user_gfx_dpa_setting(
				card->di_port, &via_gfx_dpa);
		}
	}
	KMS_DEBUG("struct via_dvi_options size is %d", size);
	KMS_DEBUG("via_dvi_options->clk_polarity=%d\n",
		via_dvi_options->clk_polarity);
	KMS_DEBUG("via_dvi_options->clk_adjust=%d\n",
		via_dvi_options->clk_adjust);
	KMS_DEBUG("via_dvi_options->clk_driving_sel=%d\n",
		via_dvi_options->clk_driving_sel);
	KMS_DEBUG("via_dvi_options->data_driving_sel=%d\n",
		via_dvi_options->data_driving_sel);
	KMS_DEBUG("via_dvi_options->no_ddc_value=%d\n",
		via_dvi_options->no_ddc_value);
	KMS_DEBUG("via_dvi_options->ad9389_circuit_state_adjust %d\n",
		via_dvi_options->ad9389_circuit_state_adjust);
}

int via_dvi_get_modes(struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_dvi_connector_private_info *dvi_private_info =
		(struct via_dvi_connector_private_info *)
		via_chrome9_connector->connector_info;
	struct via_dvi_options *via_dvi_options =
		(struct via_dvi_options *)via_chrome9_connector->options;
	int ret = 0;

	if (via_dvi_options->no_ddc_value) {
		dvi_private_info->physical_width = 4096;
		dvi_private_info->physical_height = 2048;
	} else {
		ret = via_chrome9_ddc_get_modes(via_chrome9_connector);
		via_chrome9_get_monitor_physical_size(connector,
			&dvi_private_info->physical_width,
			&dvi_private_info->physical_height);
	}
	return ret;
}

int via_dvi_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_dvi_connector_private_info *dvi_private_info =
		(struct via_dvi_connector_private_info *)
		via_chrome9_connector->connector_info;

	/* Check Clock Range */
	if (mode->clock > 165000)
		return MODE_CLOCK_HIGH;
	if (mode->clock < 25000)
		return MODE_CLOCK_LOW;
	/* Check monitor physical size */
	if (mode->hdisplay > dvi_private_info->physical_width)
		return MODE_BAD_HVALUE;
	if (mode->vdisplay > dvi_private_info->physical_width)
		return MODE_BAD_VVALUE;
	/* Check Resolution Range */
	if (via_chrome9_is_valid_mode(mode->hdisplay, mode->vdisplay) == 0)
		return MODE_BAD;

	return MODE_OK;

}
static enum drm_connector_status via_chrome9_dvi_detect(
		struct drm_connector *connector,  bool force)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	enum drm_connector_status ret = connector_status_disconnected;
	struct via_dvi_options *via_dvi_options =
		(struct via_dvi_options *)via_chrome9_connector->options;
	u8 *raw_edid = NULL ;

	if (BIT5 & via_chrome9_read_vga_io(REG_SR3E)) {
		ret = connector_status_connected;
		KMS_DEBUG("SR3E.5 = 1, internal dvi connected.\n");
	}
	if (via_dvi_options->no_ddc_value) {
		kfree(via_chrome9_connector->edid);
		via_chrome9_connector->edid = NULL;
	} else {
		kfree(via_chrome9_connector->edid);
		via_chrome9_connector->edid = NULL;
		via_chrome9_connector->edid =
			drm_get_edid(&via_chrome9_connector->base,
			&via_chrome9_connector->ddc_bus->adapter);
		if (via_chrome9_connector->edid) {
			raw_edid = (u8 *)via_chrome9_connector->edid;
			if (0x00 == raw_edid[0] && 0xff == raw_edid[1]) {
				if (DIGITAL_DEVICE == (raw_edid[20] >> 7)) {
					ret = connector_status_connected;
					KMS_DEBUG(
						"get edid passed, dvi connected.\n");
					} else {
						ret =
						connector_status_disconnected;
						KMS_DEBUG(
							"the connected device is not a digital device\n");
					}
				}

			} else {
				ret = connector_status_disconnected;
				KMS_DEBUG(
					"Get DVI edid fail in dvi detect.\n");
			}
	}

	return ret;
}

int via_chrome9_dvi_set_property(struct drm_connector *connector,
		struct drm_property *property,
		uint64_t value)
{
	/* implement later if necessary */
	return 0;
}

int via_chrome9_dvi_get_modes(struct drm_connector *connector)
{
	return via_dvi_get_modes(connector);
}

int via_chrome9_dvi_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode)
{
	return via_dvi_mode_valid(connector, mode);
}

static struct drm_connector_funcs
	via_chrome9_dvi_connector_funcs = {
	.dpms = drm_helper_connector_dpms,
	.detect = via_chrome9_dvi_detect,
	.fill_modes = drm_helper_probe_single_connector_modes,
	.destroy = via_chrome9_connector_destory,
	.set_property = via_chrome9_dvi_set_property,
};

static struct drm_connector_helper_funcs
	via_chrome9_dvi_connector_helper_funcs = {
	.get_modes = via_chrome9_dvi_get_modes,
	.mode_valid = via_chrome9_dvi_mode_valid,
	.best_encoder = via_chrome9_best_single_encoder,
};

void via_chrome9_borders_property_attach(
		struct drm_connector *connector,
		struct drm_device *dev)
{
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;

	KMS_DEBUG("\n");
	drm_connector_attach_property( connector,
		p_priv->mode_info.left_border_property, 0);
	drm_connector_attach_property( connector,
		p_priv->mode_info.right_border_property, 0);
	drm_connector_attach_property( connector,
		p_priv->mode_info.top_border_property, 0);
	drm_connector_attach_property( connector,
		p_priv->mode_info.bottom_border_property, 0);
}
/*
 * add the connector
 */
void via_chrome9_add_connector(struct drm_device *dev,
		int connector_id, enum via_chrome9_connector_type type)
{
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	struct drm_connector *connector;
	struct via_chrome9_connector *via_chrome9_connector;
	struct via_vga_options *via_vga_options = NULL;
	struct via_lcd_options *via_lcd_options = NULL;
	struct via_dvi_options *via_dvi_options = NULL;
	struct via_hdmi_options *via_hdmi_options = NULL;
	struct via_dp_options *via_dp_options = NULL;
	struct via_dp_connector_private_info
		*via_dp_1_connector_private_info = NULL;
	struct via_dp_connector_private_info
		*via_dp_2_connector_private_info = NULL;
	struct via_hdmi_connector_private_info
		*via_hdmi_connector_private_info = NULL;
	struct via_dvi_connector_private_info
		*via_dvi_connector_private_info = NULL;
	int connector_index;

	list_for_each_entry(connector, &dev->mode_config.connector_list, head) {
		via_chrome9_connector = to_via_chrome9_connector(connector);
		if ((via_chrome9_connector->connector_id == connector_id) &&
			(via_chrome9_connector->connector_type == type))
			return;
	}
	via_chrome9_connector =
		kzalloc(sizeof(struct via_chrome9_connector),
		GFP_KERNEL);
	if (!via_chrome9_connector) {
		DRM_INFO("allocate the connector error\n");
		return;
	}
	via_chrome9_connector->connector_id = connector_id;
	via_chrome9_connector->connector_type = type;
	connector = &via_chrome9_connector->base;

	switch (connector_id) {
	case VIA_CHROME9_DEVICE_CONNECTOR_CRT:
		connector_index = VIA_CHROME9_CONNECTOR_CRT_INDEX;
		via_vga_options =
			kzalloc(sizeof(struct via_vga_options), GFP_KERNEL);
		if (!via_vga_options) {
			DRM_INFO("allocate the vga option error\n");
			return;
		}
		via_vga_options->no_ddc_value = false;
		via_chrome9_connector->options = (void *)via_vga_options;
		via_chrome9_connector->set_xorg_options =
			via_chrome9_set_vga_xorg_options;
		/* connector->polled = DRM_CONNECTOR_POLL_CONNECT; */
		connector->interlace_allowed = true;
		connector->doublescan_allowed = false;
		drm_connector_init(dev, &via_chrome9_connector->base,
			&via_chrome9_vga_connector_funcs,
			via_chrome9_connector_convert[connector_index]);
		drm_connector_helper_add(&via_chrome9_connector->base,
			&via_chrome9_vga_connector_helper_funcs);
		via_chrome9_connector->ddc_bus =
			via_chrome9_lookup_i2c_chan(0x26);
		break;
	case VIA_CHROME9_DEVICE_CONNECTOR_LCD:
		connector_index = VIA_CHROME9_CONNECTOR_LCD_INDEX;
		connector->interlace_allowed = false;
		connector->doublescan_allowed = false;
		drm_connector_init(dev, &via_chrome9_connector->base,
			&via_chrome9_lcd_connector_funcs,
			via_chrome9_connector_convert[connector_index]);
		drm_connector_helper_add(&via_chrome9_connector->base,
			&via_chrome9_lcd_connector_helper_funcs);
		via_lcd_options =
			kzalloc(sizeof(struct via_lcd_options),
			GFP_KERNEL);

		if (!via_lcd_options) {
			DRM_INFO(
				"allocate the lcd xorg options structure error\n");
			return;
		}

		via_lcd_options->center = false;
		via_lcd_options->dual_channel = false;
		via_lcd_options->msb = false;
		via_lcd_options->no_dithering = false;
		via_lcd_options->fix_on_iga1 = false;
		via_lcd_options->physical_height = 0;
		via_lcd_options->physical_width = 0;
		via_chrome9_connector->options = (void *)via_lcd_options;
		via_chrome9_connector->set_xorg_options =
			via_chrome9_set_lcd_xorg_options;
		via_chrome9_connector->detect = via_chrome9_lcd_detect;
		switch (connector->connector_type_id) {
		case 1:
			via_chrome9_fb_cmd_parse("LCD1", via_lcd_options);
			break;
		case 2:
			via_chrome9_fb_cmd_parse("LCD2", via_lcd_options);
			break;
		default:
			KMS_DEBUG(
				"get lcd%d option fail\n",
				connector->connector_type_id);
			break;

		}
		switch (dev->pci_device) {
		case VX855_DEVICE:
		case VX900_DEVICE:
			via_chrome9_connector->ddc_bus =
				via_chrome9_lookup_i2c_chan(0x2c);
			break;
		default:
			via_chrome9_connector->ddc_bus =
				via_chrome9_lookup_i2c_chan(0x31);
			break;
		}
		break;
	case VIA_CHROME9_DEVICE_CONNECTOR_HDMI:
		via_hdmi_connector_private_info =
			kzalloc(sizeof(struct via_hdmi_connector_private_info),
			GFP_KERNEL);
		if (!via_hdmi_connector_private_info) {
			DRM_INFO(
				"allocate the hdmi connector private info error\n");
			return;
		}
		via_hdmi_connector_private_info->read_edid = true;
		connector->polled = DRM_CONNECTOR_POLL_HPD;
		via_chrome9_connector->connector_info =
			(void *)via_hdmi_connector_private_info;
		via_hdmi_options = kzalloc(sizeof(struct via_hdmi_options),
			GFP_KERNEL);
		if (!via_hdmi_options) {
			DRM_INFO("allocate the hdmi option error\n");
			return;
		}
		via_hdmi_options->attach_all_modes = false;
		via_chrome9_connector->options = (void *)via_hdmi_options;
		via_chrome9_connector->set_xorg_options =
			via_chrome9_set_hdmi_xorg_options;
		connector_index = VIA_CHROME9_CONNECTOR_HDMI_INDEX;
		connector->interlace_allowed = true;
		connector->doublescan_allowed = false;
		drm_connector_init(dev, &via_chrome9_connector->base,
			&via_chrome9_hdmi_connector_funcs,
			via_chrome9_connector_convert[connector_index]);
		drm_connector_helper_add(&via_chrome9_connector->base,
			&via_chrome9_hdmi_connector_helper_funcs);
		via_chrome9_borders_property_attach(connector, dev);
		break;
	case VIA_CHROME9_DEVICE_CONNECTOR_HDMI_DVI_D:
		via_dvi_connector_private_info =
			kzalloc(sizeof(struct via_dvi_connector_private_info),
			GFP_KERNEL);
		if (!via_dvi_connector_private_info) {
			DRM_INFO(
				"allocate the dvi connector private info error\n");
			return;
		}
		via_dvi_connector_private_info->read_edid = true;
		connector->polled = DRM_CONNECTOR_POLL_HPD;
		via_chrome9_connector->connector_info =
			(void *)via_dvi_connector_private_info;

		via_dvi_options =
			kzalloc(sizeof(struct via_dvi_options), GFP_KERNEL);

		if (!via_dvi_options) {
			DRM_INFO("allocate the dvi option error\n");
			return;
		}

		via_chrome9_connector->options = (void *)via_dvi_options;
		via_chrome9_connector->set_xorg_options =
			via_chrome9_set_dvi_xorg_options;
		connector_index = VIA_CHROME9_CONNECTOR_DVI_D_INDEX;
		connector->interlace_allowed = false;
		connector->doublescan_allowed = false;
		drm_connector_init(dev, &via_chrome9_connector->base,
			&via_chrome9_hdmi_dvi_connector_funcs,
			via_chrome9_connector_convert[connector_index]);
		drm_connector_helper_add(&via_chrome9_connector->base,
			&via_chrome9_hdmi_dvi_connector_helper_funcs);
		break;
	case VIA_CHROME9_DEVICE_CONNECTOR_DP_1:
		via_dp_1_connector_private_info =
			kzalloc(sizeof(struct via_dp_connector_private_info),
			GFP_KERNEL);
		if (!via_dp_1_connector_private_info) {
			DRM_INFO("allocate the dp1 private info error\n");
			return;
		}
		connector->polled = DRM_CONNECTOR_POLL_HPD;
		via_chrome9_connector->connector_private =
			(void *)via_dp_1_connector_private_info;
		via_dp_options =
			kzalloc(sizeof(struct via_dp_options), GFP_KERNEL);
		if (!via_dp_options) {
			DRM_INFO("allocate the dp1 option error\n");
			return;
		}
		via_chrome9_connector->options = (void *)via_dp_options;
		via_chrome9_connector->set_xorg_options =
			via_chrome9_set_dp_xorg_options;
		connector_index = VIA_CHROME9_CONNECTOR_DP_1_INDEX;
		connector->interlace_allowed = false;
		connector->doublescan_allowed = false;
		drm_connector_init(dev, &via_chrome9_connector->base,
			&via_chrome9_dp_1_connector_funcs,
			via_chrome9_connector_convert[connector_index]);
		drm_connector_helper_add(&via_chrome9_connector->base,
			&via_chrome9_dp_1_connector_helper_funcs);
		via_chrome9_dp_1_init_ephy(connector);
		break;
	case VIA_CHROME9_DEVICE_CONNECTOR_DP_2:
		via_dp_2_connector_private_info =
			kzalloc(sizeof(struct via_dp_connector_private_info),
			GFP_KERNEL);
		if (!via_dp_2_connector_private_info) {
			DRM_INFO("allocate the dp1 private info error\n");
			return;
		}
		connector->polled = DRM_CONNECTOR_POLL_HPD;
		via_chrome9_connector->connector_private =
			(void *)via_dp_2_connector_private_info;
		via_dp_options =
			kzalloc(sizeof(struct via_dp_options), GFP_KERNEL);
		if (!via_dp_options) {
			DRM_INFO("allocate the dp2 option error\n");
			return;
		}
		via_chrome9_connector->options = (void *)via_dp_options;
		via_chrome9_connector->set_xorg_options =
			via_chrome9_set_dp_xorg_options;
		connector_index = VIA_CHROME9_CONNECTOR_DP_2_INDEX;
		connector->interlace_allowed = false;
		connector->doublescan_allowed = false;
		drm_connector_init(dev, &via_chrome9_connector->base,
			&via_chrome9_dp_2_connector_funcs,
			via_chrome9_connector_convert[connector_index]);
		drm_connector_helper_add(&via_chrome9_connector->base,
			&via_chrome9_dp_2_connector_helper_funcs);
		via_chrome9_dp_2_init_ephy(connector);
		break;
	case VIA_CHROME9_DEVICE_CONNECTOR_DVI_D:
		via_dvi_connector_private_info =
			kzalloc(sizeof(struct via_dvi_connector_private_info),
			GFP_KERNEL);
		if (!via_dvi_connector_private_info) {
			DRM_INFO(
				"allocate the dvi connector private info error\n");
			return;
		}
		connector->polled = DRM_CONNECTOR_POLL_HPD;
		via_chrome9_connector->connector_info =
			(void *)via_dvi_connector_private_info;

		via_dvi_options =
			kzalloc(sizeof(struct via_dvi_options), GFP_KERNEL);
		if (!via_dvi_options) {
			DRM_INFO("allocate the dvi option error\n");
			return;
		}
		via_chrome9_connector->options = (void *)via_dvi_options;
		via_chrome9_connector->set_xorg_options =
			via_chrome9_set_dvi_xorg_options;
		via_chrome9_connector->detect = via_chrome9_dvi_detect;
		connector_index = VIA_CHROME9_CONNECTOR_DVI_D_INDEX;
		connector->interlace_allowed = false;
		connector->doublescan_allowed = false;
		drm_connector_init(dev, &via_chrome9_connector->base,
			&via_chrome9_dvi_connector_funcs,
			via_chrome9_connector_convert[connector_index]);
		drm_connector_helper_add(&via_chrome9_connector->base,
			&via_chrome9_dvi_connector_helper_funcs);
		if (p_priv->pseudo_chip_revision >= REVISION_VX800_B1) {
			via_chrome9_connector->ddc_bus =
				via_chrome9_lookup_i2c_chan(0x31);
		} else {
			via_chrome9_connector->ddc_bus =
				via_chrome9_lookup_i2c_chan(0x3D);
		}
		break;

	case VIA_CHROME9_DEVICE_CONNECTOR_LCD_2:
		connector_index = VIA_CHROME9_CONNECTOR_LCD_INDEX;
		connector->interlace_allowed = false;
		connector->doublescan_allowed = false;
		drm_connector_init(dev, &via_chrome9_connector->base,
			&via_chrome9_lcd_connector_funcs,
			via_chrome9_connector_convert[connector_index]);
		drm_connector_helper_add(&via_chrome9_connector->base,
			&via_chrome9_lcd_connector_helper_funcs);
		via_lcd_options =
			kzalloc(sizeof(struct via_lcd_options), GFP_KERNEL);
		if (!via_lcd_options) {
			DRM_INFO(
				"allocate the lcd xorg options structure error\n");
			return;
		}
		via_lcd_options->center = false;
		via_lcd_options->dual_channel = false;
		via_lcd_options->msb = false;
		via_lcd_options->no_dithering = false;
		via_lcd_options->fix_on_iga1 = false;
		via_lcd_options->physical_height = 0;
		via_lcd_options->physical_width = 0;
		via_chrome9_connector->options = (void *)via_lcd_options;
		via_chrome9_connector->set_xorg_options =
			via_chrome9_set_lcd_xorg_options;
		switch (connector->connector_type_id) {
		case 1:
			via_chrome9_fb_cmd_parse("LCD1", via_lcd_options);
			break;
		case 2:
			via_chrome9_fb_cmd_parse("LCD2", via_lcd_options);
			break;
		default:
			KMS_DEBUG("get lcd%d option fail\n",
				connector->connector_type_id);
			break;

		}
		switch (dev->pci_device) {
		case VX855_DEVICE:
		case VX900_DEVICE:
			via_chrome9_connector->ddc_bus =
				via_chrome9_lookup_i2c_chan(0x2c);
			break;
		default:
			via_chrome9_connector->ddc_bus =
				via_chrome9_lookup_i2c_chan(0x31);
			break;
		}
		break;

	case VIA_CHROME9_DEVICE_CONNECTOR_TV:
		break;
	default:
		DRM_INFO("invalid connector type\n");
		break;
	}

	drm_sysfs_connector_add(connector);
}

/*
 * travel the connector and encoder list, bind them together
 */
void via_chrome9_attach_encoder_connector(struct drm_device *dev)
{
	struct drm_connector *connector;
	struct drm_encoder *encoder;
	struct via_chrome9_connector *via_chrome9_connector;
	struct via_chrome9_encoder *via_chrome9_encoder;

	list_for_each_entry(connector, &dev->mode_config.connector_list, head) {
		via_chrome9_connector = to_via_chrome9_connector(connector);
		if (!via_chrome9_is_internal_connector(
			via_chrome9_connector))
			continue;
		list_for_each_entry(encoder,
			&dev->mode_config.encoder_list, head) {
			via_chrome9_encoder = to_via_chrome9_encoder(encoder);
			if ((via_chrome9_encoder->supported_connect_types &
				via_chrome9_connector->connector_id) &&
				((int)via_chrome9_encoder->encoder_type ==
				(int)via_chrome9_connector->connector_type)) {
				drm_mode_connector_attach_encoder(
					connector, encoder);
				KMS_DEBUG(
					"attach  connector id =%x  encoder id =%x\n",
					via_chrome9_connector->connector_id,
					via_chrome9_encoder->encoder_id);
			}
		}
	}

	via_chrome9_external_card_link_enc_conn(dev);
}

int via_chrome9_detect_enc_conn(struct drm_device *dev)
{
	/* init the i2c, create the ddc port array*/
	/* travel the internal encoder list and init */
	via_chrome9_internal_encoder_init(dev);
	/* use i2c to get the external encoder info and do init */
	via_chrome9_add_external_card_driver(dev);
	/* bind the encoder and connector together */
	via_chrome9_attach_encoder_connector(dev);
	return 0;
}
