 /*
 * Copyright 1998-2011 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2011 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "via_chrome9_drv.h"
#include "via_chrome9_display_common.h"
#include "via_chrome9_external_card.h"
#include "via_chrome9_object.h"

extern void via_chrome9_load_fixed_crtc_regs(struct drm_device *dev);

static const struct drm_mode_config_funcs via_chrome9_mode_funcs = {
	.fb_create = via_chrome9_user_framebuffer_create,
	.output_poll_changed = via_chrome9_fb_output_poll_changed
};

static struct via_chrome9_i2c_hw_rec via_chrome9_i2c_hw_rec_table[] = {
	VIA_CHROME9_IIC_SEARIAL_HW_REC(0x31, REG_SR31, 5, 4, 3, 2, 0, 0),
	VIA_CHROME9_IIC_SEARIAL_HW_REC(0x26, REG_SR26, 5, 4, 3, 2, 0, 0),
	VIA_CHROME9_IIC_GPIO_HW_REC(0x3d, REG_SR3D, 5, 4, 3, 2, 7, 6),
	VIA_CHROME9_IIC_GPIO_HW_REC(0x2c, REG_SR2C, 5, 4, 3, 2, 7, 6),
};

#define VIA_CHROME9_IIC_BUS_NUM ARRAY_SIZE(via_chrome9_i2c_hw_rec_table)

static struct via_chrome9_io_reg initial_reg_table_vx800[] = {
	{ REG_SR10, 0xFF, 0x01 },
	{ REG_SR15, 0x02, 0x02 },
	{ REG_SR16, 0xBF, 0x08 },
	{ REG_SR17, 0xFF, 0x1F },
	{ REG_SR18, 0xFF, 0x4E },
	{ REG_SR1A, 0xFB, 0x08 },
	{ REG_SR1B, 0xFF, 0xF0 },
	{ REG_SR1E, 0x07, 0x01 },
	{ REG_SR2A, 0xF0, 0x00 },
	/* VCK and LCK PLL power on.*/
	{ REG_SR2D, 0xFF, 0xFF },
	/* Initial CR09=0*/
	{ REG_CR09, 0xFF, 0x00 },
	/* IGA1 initial  Vertical end*/
	{ REG_CR11, 0x8F, 0x00 },
	/* IGA1 CRT Mode control init*/
	{ REG_CR17, 0x7F, 0x00 },
	/* Cursor Start*/
	{ REG_CR0A, 0xFF, 0x1E },
	/* Cursor End*/
	{ REG_CR0B, 0xFF, 0x00 },
	/* Cursor Location High*/
	{ REG_CR0E, 0xFF, 0x00 },
	/* Cursor Localtion Low*/
	{ REG_CR0F, 0xFF, 0x00 },
	{ REG_CR32, 0xFF, 0x00 },
	{ REG_CR33, 0x7F, 0x00 },
	{ REG_CR34, 0xFF, 0x00 },
	{ REG_CR35, 0xFF, 0x00 },
	{ REG_CR36, 0x38, 0x30 },
	{ REG_CR69, 0xFF, 0x00 },
	{ REG_CR6A, 0xFD, 0x60 },
	{ REG_CR6B, 0xFF, 0x00 },
	{ REG_CR6C, 0xFF, 0x00 },
	/* LCD Scaling Parameter 1*/
	{ REG_CR7A, 0xFF, 0x01 },
	/* LCD Scaling Parameter 2*/
	{ REG_CR7B, 0xFF, 0x02 },
	/* LCD Scaling Parameter 3 */
	{ REG_CR7C, 0xFF, 0x03 },
	/* LCD Scaling Parameter 4*/
	{ REG_CR7D, 0xFF, 0x04 },
	/* LCD Scaling Parameter 5*/
	{ REG_CR7E, 0xFF, 0x07 },
	/* LCD Scaling Parameter 6 */
	{ REG_CR7F, 0xFF, 0x0A },
	/* LCD Scaling Parameter 7*/
	{ REG_CR80, 0xFF, 0x0D },
	/* LCD Scaling Parameter 8*/
	{ REG_CR81, 0xFF, 0x13 },
	/* LCD Scaling Parameter 9*/
	{ REG_CR82, 0xFF, 0x16 },
	/* LCD Scaling Parameter 10*/
	{ REG_CR83, 0xFF, 0x19 },
	/* LCD Scaling Parameter 11*/
	{ REG_CR84, 0xFF, 0x1C },
	/* LCD Scaling Parameter 12*/
	{ REG_CR85, 0xFF, 0x1D },
	/* LCD Scaling Parameter 13*/
	{ REG_CR86, 0xFF, 0x1E },
	/* LCD Scaling Parameter 14*/
	{ REG_CR87, 0xFF, 0x1F },
	/* LCD Panel Type*/
	{ REG_CR88, 0xFF, 0x40 },
	/* LCD Timing Control 0 */
	{ REG_CR89, 0xFF, 0x00 },
	/* LCD Timing Control 1 */
	{ REG_CR8A, 0xFF, 0x88 },
	/* Second power sequence control*/
	{ REG_CRD4, 0xFF, 0x81 },
	/* 24/12 bit LVDS Data off */
	{ REG_CR91, 0xFF, 0x80 },
	{ REG_CR96, 0xFF, 0x00 },
	{ REG_CR97, 0xFF, 0x00 },
	{ REG_CR99, 0xFF, 0x00 },
	{ REG_CR9B, 0xFF, 0x00 },
	/* TMDS/LVDS control register. */
	{ REG_CRD2, 0xFF, 0xFF }
};

#define NUM_TOTAL_VX800_INIT_REG_TBL CMDISP_ARRAYSIZE(initial_reg_table_vx800)

static struct via_chrome9_io_reg initial_reg_table_vx855[] = {
	{ REG_SR10, 0xFF, 0x01 },
	{ REG_SR15, 0x02, 0x02 },
	{ REG_SR16, 0xBF, 0x08 },
	{ REG_SR17, 0xFF, 0x1F },
	{ REG_SR18, 0xFF, 0x4E },
	{ REG_SR1A, 0xFB, 0x08 },
	{ REG_SR1B, 0xFF, 0xF0 },
	{ REG_SR1E, 0x07, 0x01 },
	{ REG_SR2A, 0xF0, 0x00 },
	{ REG_SR58, 0xFF, 0x00 },
	{ REG_SR59, 0xFF, 0x00 },
	/* VCK and LCK PLL power on. */
	{ REG_SR2D, 0xFF, 0xFF },
	/* Initial CR09=0 */
	{ REG_CR09, 0xFF, 0x00 },
	/* IGA1 initial  Vertical end */
	{ REG_CR11, 0x8F, 0x00 },
	/* IGA1 CRT Mode control init */
	{ REG_CR17, 0x7F, 0x00 },
	/* Cursor Start */
	{ REG_CR0A, 0xFF, 0x1E },
	/* Cursor End */
	{ REG_CR0B, 0xFF, 0x00 },
	/* Cursor Location High */
	{ REG_CR0E, 0xFF, 0x00 },
	/* Cursor Localtion Low */
	{ REG_CR0F, 0xFF, 0x00 },
	{ REG_CR32, 0xFF, 0x00 },
	{ REG_CR33, 0x7F, 0x00 },
	{ REG_CR34, 0xFF, 0x00 },
	{ REG_CR35, 0xFF, 0x00 },
	{ REG_CR36, 0x38, 0x30 },
	{ REG_CR69, 0xFF, 0x00 },
	{ REG_CR6A, 0xFD, 0x60 },
	{ REG_CR6B, 0xFF, 0x00 },
	{ REG_CR6C, 0xFF, 0x00 },
	/* LCD scaling Factor*/
	{ REG_CR77, 0xFF, 0x00 },
	/* LCD scaling Factor */
	{ REG_CR78, 0xFF, 0x00 },
	/* LCD scaling Factor*/
	{ REG_CR79, 0xFF, 0x00 },
	/* LCD scaling Factor */
	{ REG_CR9F, 0x03, 0x00 },
	/* LCD Scaling Parameter 1 */
	{ REG_CR7A, 0xFF, 0x01 },
	/* LCD Scaling Parameter 2 */
	{ REG_CR7B, 0xFF, 0x02 },
	/* LCD Scaling Parameter 3 */
	{ REG_CR7C, 0xFF, 0x03 },
	/* LCD Scaling Parameter 4 */
	{ REG_CR7D, 0xFF, 0x04 },
	/* LCD Scaling Parameter 5 */
	{ REG_CR7E, 0xFF, 0x07 },
	/* LCD Scaling Parameter 6*/
	{ REG_CR7F, 0xFF, 0x0A },
	/* LCD Scaling Parameter 7 */
	{ REG_CR80, 0xFF, 0x0D },
	/* LCD Scaling Parameter 8 */
	{ REG_CR81, 0xFF, 0x13 },
	/* LCD Scaling Parameter 9 */
	{ REG_CR82, 0xFF, 0x16 },
	/* LCD Scaling Parameter 10 */
	{ REG_CR83, 0xFF, 0x19 },
	/* LCD Scaling Parameter 11 */
	{ REG_CR84, 0xFF, 0x1C },
	/* LCD Scaling Parameter 12 */
	{ REG_CR85, 0xFF, 0x1D },
	/* LCD Scaling Parameter 13 */
	{ REG_CR86, 0xFF, 0x1E },
	/* LCD Scaling Parameter 14  */
	{ REG_CR87, 0xFF, 0x1F },
	/* LCD Panel Type */
	{ REG_CR88, 0xFF, 0x40 },
	/* LCD Timing Control 0 */
	{ REG_CR89, 0xFF, 0x00 },
	/* LCD Timing Control 1 */
	{ REG_CR8A, 0xFF, 0x88 },
	/* Second power sequence control */
	{ REG_CRD4, 0xFF, 0x81 },
	/* 24/12 bit LVDS Data off */
	{ REG_CR91, 0xFF, 0x80 },
	{ REG_CR96, 0xFF, 0x00 },
	{ REG_CR97, 0xFF, 0x00 },
	{ REG_CR99, 0xFF, 0x00 },
	{ REG_CR9B, 0xFF, 0x00 },
	/* TMDS/LVDS control register. */
	{ REG_CRD2, 0xFF, 0xFF },
	/* IGA1 Scaling Up and Timing Control */
	{ REG_CRFD, 0xFF, 0x00 }
};
#define NUM_TOTAL_VX855_INIT_REG_TBL CMDISP_ARRAYSIZE(initial_reg_table_vx855)

static struct via_chrome9_io_reg initial_reg_table_vx900[] = {
	{ REG_SR10, 0xFF, 0x01 },
	{ REG_SR15, 0x02, 0x02 },
	{ REG_SR16, 0xBF, 0x08 },
	{ REG_SR17, 0xFF, 0x1F },
	{ REG_SR18, 0xFF, 0x4E },
	{ REG_SR1A, 0xFB, 0x08 },
	{ REG_SR1B, 0xFF, 0xF0 },
	{ REG_SR1E, 0x07, 0x01 },
	{ REG_SR2A, 0xF0, 0x00 },
	{ REG_SR58, 0xFF, 0x00 },
	{ REG_SR59, 0xFF, 0x00 },
	/* VCK and LCK PLL power on. */
	{ REG_SR2D, 0xFF, 0xFF },
	/* Initial CR09=0 */
	{ REG_CR09, 0xFF, 0x00 },
	/* IGA1 initial  Vertical end */
	{ REG_CR11, 0x8F, 0x00 },
	/* IGA1 CRT Mode control init  */
	{ REG_CR17, 0x7F, 0x00 },
	/* Cursor Start */
	{ REG_CR0A, 0xFF, 0x1E },
	/* Cursor End */
	{ REG_CR0B, 0xFF, 0x00 },
	/* Cursor Location High */
	{ REG_CR0E, 0xFF, 0x00 },
	/* Cursor Localtion Low */
	{ REG_CR0F, 0xFF, 0x00 },
	{ REG_CR32, 0xFF, 0x00 },
	{ REG_CR33, 0x7F, 0x00 },
	{ REG_CR34, 0xFF, 0x00 },
	{ REG_CR35, 0xFF, 0x00 },
	{ REG_CR36, 0x38, 0x30 },
	/* Disble IGA1 Scaling*/
	{ REG_CR45, 0x01, 0x00 },
	{ REG_CR69, 0xFF, 0x00 },
	{ REG_CR6A, 0xFD, 0x60 },
	{ REG_CR6B, 0xFF, 0x00 },
	{ REG_CR6C, 0xFF, 0x00 },
	/* LCD Scaling Parameter 1*/
	{REG_CR7A, 0xFF, 0x01},
	/* LCD Scaling Parameter 2 */
	{REG_CR7B, 0xFF, 0x02},
	/* LCD Scaling Parameter 3 */
	{REG_CR7C, 0xFF, 0x03},
	/* LCD Scaling Parameter 4 */
	{ REG_CR7D, 0xFF, 0x04 },
	/* LCD Scaling Parameter 5 */
	{ REG_CR7E, 0xFF, 0x07 },
	/* LCD Scaling Parameter 6 */
	{ REG_CR7F, 0xFF, 0x0A },
	/* LCD Scaling Parameter 7 */
	{ REG_CR80, 0xFF, 0x0D },
	/* LCD Scaling Parameter 8 */
	{ REG_CR81, 0xFF, 0x13 },
	/* LCD Scaling Parameter 9 */
	{ REG_CR82, 0xFF, 0x16 },
	/* LCD Scaling Parameter 10 */
	{ REG_CR83, 0xFF, 0x19 },
	/* LCD Scaling Parameter 11 */
	{ REG_CR84, 0xFF, 0x1C },
	/* LCD Scaling Parameter 12 */
	{ REG_CR85, 0xFF, 0x1D },
	/* LCD Scaling Parameter 13 */
	{ REG_CR86, 0xFF, 0x1E },
	/* LCD Scaling Parameter 14 */
	{ REG_CR87, 0xFF, 0x1F },
	/* LCD Panel Type */
	{ REG_CR88, 0xFF, 0x40 },
	/* LCD Timing Control 0 */
	{ REG_CR89, 0xFF, 0x00 },
	/* LCD Timing Control 1 */
	{ REG_CR8A, 0xFF, 0x88 },
	/* Second power sequence control */
	{ REG_CRD4, 0xFF, 0x81 },
	/* 24/12 bit LVDS Data off */
	{ REG_CR91, 0xFF, 0x80 },
	{ REG_CR96, 0xFF, 0x00 },
	{ REG_CR97, 0xFF, 0x00 },
	{ REG_CR99, 0xFF, 0x00 },
	{ REG_CR9B, 0xFF, 0x00 },
	/* TMDS/LVDS control register. */
	{REG_CRD2, 0xFF, 0xFF}
};
#define NUM_TOTAL_VX900_INIT_REG_TBL CMDISP_ARRAYSIZE(initial_reg_table_vx900)

static struct td_timer_info panel_td_timer = { 500, 50, 0, 510 };

static struct td_timer td_timer_regs[] = {
	/* td_timer0 */
	{ { { REG_CR8B, 0, 7 }, { REG_CR8F, 0, 3 } } },
	/* td_timer1 */
	{ { { REG_CR8C, 0, 7 }, { REG_CR8F, 4, 7 } } },
	/* td_timer2 */
	{ { { REG_CR8D, 0, 7 }, { REG_CR90, 0, 3 } } },
	/* td_timer3 */
	{ { { REG_CR8E, 0, 7 }, { REG_CR90, 4, 7 } } }
};

/*
    Function Name:  via_chrome9_init_td_timing_regs
    Description: Init TD timing register (power sequence)
*/
static void via_chrome9_init_td_timing_regs(struct drm_device *dev)
{
	unsigned int td_timer[PANEL_TD_TIMING_AMOUNT];
	unsigned int i;

	/* Calculate TD Timer, every step is 572.1uSec */
	td_timer[0] = panel_td_timer.td_timer0 * 10000 / 5721;
	td_timer[1] = panel_td_timer.td_timer1 * 10000 / 5721;
	td_timer[2] = panel_td_timer.td_timer2 * 10000 / 5721;
	td_timer[3] = panel_td_timer.td_timer3 * 10000 / 5721;

	/* Fill primary power sequence */
	for (i = 0; i < PANEL_TD_TIMING_AMOUNT; i++)
		via_chrome9_load_regs(td_timer[i], PANEL_TD_TIMING_REG_AMOUNT,
			td_timer_regs[i].tdRegs);

	/*Note: VT3353 have two hardware power sequences
	other chips only have one hardware power sequence*/
	switch (dev->pci_device) {
	case VX800_DEVICE:
		/* set CRD4[0] to "1" to select 2nd LCD power sequence. */
		via_chrome9_write_vga_io_bits(REG_CRD4, 0x01, 0x01);
		/* Fill secondary power sequence */
		for (i = 0; i < PANEL_TD_TIMING_AMOUNT; i++)
			via_chrome9_load_regs(td_timer[i],
				PANEL_TD_TIMING_REG_AMOUNT,
				td_timer_regs[i].tdRegs);

		break;

	default:
		break;
	}
}


static int via_chrome9_mode_set_create_properties(struct drm_device *dev)
{
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;

	/* left border */
	p_priv->mode_info.left_border_property = drm_property_create(
		p_priv->ddev, DRM_MODE_PROP_RANGE, HDMI_LEFT_BORDER_NAME, 2);
	if (!p_priv->mode_info.left_border_property) {
		KMS_DEBUG("create border property failed !\n");
		return -ENOMEM;
	}
	p_priv->mode_info.left_border_property->values[0] = 0;
	p_priv->mode_info.left_border_property->values[1] =
		MAX_SUPPORTED_HOR_BORDER;


	/* right border */
	p_priv->mode_info.right_border_property = drm_property_create(
		p_priv->ddev, DRM_MODE_PROP_RANGE, HDMI_RIGHT_BORDER_NAME, 2);
	if (!p_priv->mode_info.right_border_property) {
		KMS_DEBUG("create border property failed !\n");
		return -ENOMEM;
	}
	p_priv->mode_info.right_border_property->values[0] = 0;
	p_priv->mode_info.right_border_property->values[1] =
		MAX_SUPPORTED_HOR_BORDER;


	/* top border */
	p_priv->mode_info.top_border_property = drm_property_create(
		p_priv->ddev, DRM_MODE_PROP_RANGE, HDMI_TOP_BORDER_NAME, 2);
	if (!p_priv->mode_info.top_border_property) {
		KMS_DEBUG("create border property failed !\n");
		return -ENOMEM;
	}
	p_priv->mode_info.top_border_property->values[0] = 0;
	p_priv->mode_info.top_border_property->values[1] =
		MAX_SUPPORTED_VER_BORDER;


	/* bottom border */
	p_priv->mode_info.bottom_border_property = drm_property_create(
		p_priv->ddev, DRM_MODE_PROP_RANGE, HDMI_BOTTOM_BORDER_NAME, 2);
	if (!p_priv->mode_info.bottom_border_property) {
		KMS_DEBUG("create border property failed !\n");
		return -ENOMEM;
	}
	p_priv->mode_info.bottom_border_property->values[0] = 0;
	p_priv->mode_info.bottom_border_property->values[1] =
		MAX_SUPPORTED_VER_BORDER;

	return 0;
}

static void via_chrome9_init_display_engine(struct drm_device *dev)
{
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	u8 val;
	/*1. Write Common Setting for Video Mode */
	switch (dev->pci_device) {
	case VX800_DEVICE:
		via_chrome9_write_vga_io_multi_bits(initial_reg_table_vx800,
			NUM_TOTAL_VX800_INIT_REG_TBL);
		if (p_priv->bios_is_spread_spectrum) {
			val = via_chrome9_read_vga_io(REG_SR3D);
			via_chrome9_write_vga_io(REG_SR3D, val & 0xFE);
		}
		break;

	case VX855_DEVICE:
		via_chrome9_write_vga_io_multi_bits(initial_reg_table_vx855,
			NUM_TOTAL_VX855_INIT_REG_TBL);
		if (p_priv->bios_is_spread_spectrum) {
			val = via_chrome9_read_vga_io(REG_SR3D);
			via_chrome9_write_vga_io(REG_SR3D, val & 0xFE);
		}
		break;

	case VX900_DEVICE:
		via_chrome9_write_vga_io_multi_bits(initial_reg_table_vx900,
			NUM_TOTAL_VX900_INIT_REG_TBL);
		if (p_priv->bios_is_spread_spectrum) {
			val = via_chrome9_read_vga_io(REG_SR2C);
			via_chrome9_write_vga_io(REG_SR2C, val & 0xFE);
		}
		break;
	}

	if (p_priv->bios_is_spread_spectrum) {
		val = via_chrome9_read_vga_io(REG_SR1E);
		via_chrome9_write_vga_io(REG_SR1E, val & 0xF7);
	}

	/*2. load fixed CRTC timing registers*/
	via_chrome9_load_fixed_crtc_regs(dev);

	/*3. Init TD timing register (power sequence)*/
	via_chrome9_init_td_timing_regs(dev);

	/*4. I/O address bit to be 1 */
	/*Enables access to frame buffer at A0000-BFFFFh*/
	via_chrome9_write_misc_io(via_chrome9_read_misc_io() | 0x01);

}

static void  via_chrome9_get_chipset_revision(struct drm_device *dev)
{
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	u32 chipset = 0;
	u32 chipset_revision = 0;

	switch (dev->pci_device) {
	case VX800_DEVICE:
		chipset = 0x000353;
		p_priv->mode_info.chip_index = VX800_INDEX;
		break;

	case VX855_DEVICE:
		chipset = 0x000409;
		p_priv->mode_info.chip_index = VX855_INDEX;
		break;

	case VX900_DEVICE:
		chipset = 0x000410;
		p_priv->mode_info.chip_index = VX900_INDEX;
		break;

	default:
		KMS_DEBUG("An invalid chipset!!!\n");
		break;
	}

	chipset_revision = via_chrome9_read_vga_io(REG_SR3B);
	p_priv->pseudo_chip_revision = (chipset << 8) | chipset_revision;
	KMS_DEBUG("The chipset revision is %x", p_priv->pseudo_chip_revision);
}

/* Function Name: SetChipCaps.
*  Purpose: Set chip capabilities according to chip's device ID.
*  Parameters: Global variable pBIOSInfo.
*  Return value: None.
*/
static void via_chrome9_set_chip_caps(struct drm_device *dev)
{
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	u8 val;
	switch (dev->pci_device) {
	case VX800_DEVICE:
		p_priv->via_chrome9_pll_type = PLL_TYPE_PLL_3_NEW;
		break;

	case VX855_DEVICE:
		p_priv->via_chrome9_pll_type = PLL_TYPE_PLL_3_ADV;
		break;

	case VX900_DEVICE:
		p_priv->via_chrome9_chip_caps = CAPS_IGA1_DOWNSCALING |
			CAPS_IGA1_EXPAND | CAPS_IGA2_DOWNSCALING;
		p_priv->via_chrome9_pll_type = PLL_TYPE_PLL_3_ADV;
		break;
	default:
		KMS_DEBUG("An invalid chipset!!!\n");

		break;
	}
	/* Check if spread spectrum is enable */
	val = via_chrome9_read_vga_io(REG_SR1E);
	if (val & BIT3)
		p_priv->bios_is_spread_spectrum = true;
	else
		p_priv->bios_is_spread_spectrum = false;

}

void via_chrome9_save_display_engines(struct drm_device *dev)
{
	struct drm_via_chrome9_private *dev_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	int i = 0;

	/* Unlock Extended Regs */
	via_chrome9_write_vga_io_bits(REG_SR10, 0x01, BIT0);
	/* Unlock CRTC register protect */
	via_chrome9_write_vga_io_bits(REG_CR47, 0, BIT4);

	 /* Save default SR/CR */
	while (i != 0xff) {
		dev_priv->pm_backup.sr_regs[i] =
			via_chrome9_read_vga_io(i + REG_SR00);
		dev_priv->pm_backup.cr_regs[i] =
			via_chrome9_read_vga_io(i + REG_CR00);
		i++;
	}
}

static void via_chrome9_i2c_hw_reg_init(void)
{
	via_chrome9_write_vga_io(REG_SR31, 0x01);
	via_chrome9_write_vga_io_bits(REG_SR31, 0x30, 0x30);
	via_chrome9_write_vga_io(REG_SR26, 0x01);
	via_chrome9_write_vga_io_bits(REG_SR26, 0x30, 0x30);
	via_chrome9_write_vga_io(REG_SR2C, 0xc2);
	via_chrome9_write_vga_io(REG_SR3D, 0xc0);
	via_chrome9_write_vga_io_bits(REG_SR2C, 0x30, 0x30);
	via_chrome9_write_vga_io_bits(REG_SR3D, 0x30, 0x30);
}


static int via_chrome9_i2c_init(struct drm_device *dev)
{
	int i;
	char name[7];

	via_chrome9_i2c_hw_reg_init();

	for (i = 0; i < VIA_CHROME9_IIC_BUS_NUM; i++) {
		sprintf(name, "0x%x", via_chrome9_i2c_hw_rec_table[i].name);
		via_chrome9_i2c_create(dev,
			&via_chrome9_i2c_hw_rec_table[i], name);
	}
	return 0;
}


int via_chrome9_mode_set_init(struct drm_device *dev)
{
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	int i, ret;

	drm_mode_config_init(dev);

	dev->mode_config.funcs = (void *)&via_chrome9_mode_funcs;
	dev->mode_config.max_width = 4096;
	dev->mode_config.max_height = 4096;
	dev->mode_config.fb_base = p_priv->vram_start;
	p_priv->num_crtc = 2;

	ret = via_chrome9_mode_set_create_properties(dev);
	if (ret) {
		KMS_DEBUG("create properties failed!!\n");
		return ret;
	}

	via_chrome9_set_chip_caps(dev);
	via_chrome9_get_chipset_revision(dev);
	via_chrome9_save_display_engines(dev);
	via_chrome9_init_display_engine(dev);
	via_chrome9_i2c_init(dev);
	via_chrome9_hw_cursor_init(dev);
	for (i = 0; i < p_priv->num_crtc; i++)
		via_chrome9_crtc_init(dev, i);

	ret = via_chrome9_detect_enc_conn(dev);
	if (ret) {
		KMS_DEBUG("detect the encoder and connector error\n");
		return ret;
	}

	/* initialize hpd */
	via_chrome9_hpd_init(dev);

	via_chrome9_fbdev_init(dev);

	drm_kms_helper_poll_init(dev);

	return 0;
}

int via_chrome9_resume_mode_set(struct drm_device *dev)
{
	struct drm_connector *connector;
	struct drm_crtc *crtc;
	struct drm_framebuffer *fb;
	struct via_chrome9_framebuffer *via_fb;
	struct drm_gem_object *obj;
	struct via_chrome9_object *vobj;
	int i, ret;

	via_chrome9_init_display_engine(dev);
	via_chrome9_i2c_hw_reg_init();
	via_chrome9_hw_cursor_init(dev);
	via_chrome9_hpd_init(dev);

	/* enable vsync interrupt for vblank count */
	for (i = 0; i < dev->num_crtcs; i++) {
		if (dev->vblank_enabled[i])
			ret = dev->driver->enable_vblank(dev, i);
	}

	list_for_each_entry(crtc, &dev->mode_config.crtc_list, head) {
		if (!crtc->enabled)
			continue;
		fb = crtc->fb;
		via_fb = to_via_chrome9_framebuffer(fb);
		obj = via_fb->obj;
		vobj = obj->driver_private;
		/* if this bo is allocated by fb. the ptr is valid */
		if (vobj->ptr != NULL)
			memset(vobj->ptr, '\0', vobj->bo.num_pages * PAGE_SIZE);
	}

	list_for_each_entry(connector, &dev->mode_config.connector_list, head) {
		drm_helper_connector_dpms(connector, DRM_MODE_DPMS_ON);
		via_chrome9_dp_1_resume_pre_handle(connector);
		via_chrome9_dp_2_resume_pre_handle(connector);
	}

	drm_helper_resume_force_mode(dev);

	return 0;
}

void via_chrome9_mode_set_fini(struct drm_device *dev)
{
	drm_kms_helper_poll_fini(dev);
	via_chrome9_free_all_i2c_chan();
	via_chrome9_del_external_card_driver();
	via_chrome9_hpd_fini(dev);
	via_chrome9_fbdev_fini(dev);
	drm_mode_config_cleanup(dev);
}

int via_chrome9_suspend_mode_set(struct drm_device *dev)
{
	struct drm_connector *connector;

	list_for_each_entry(connector, &dev->mode_config.connector_list, head) {
		drm_helper_connector_dpms(connector, DRM_MODE_DPMS_OFF);
	}
	via_chrome9_hpd_fini(dev);

	return 0;
}
