 /*
 * Copyright 1998-2011 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2011 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "via_chrome9_drv.h"
#include "via_chrome9_display_common.h"
#include "via_chrome9_mode.h"

unsigned char *mmio_map_base;
unsigned char *mmio_mb1;

#define MASK_START(x)	\
	(((x) == 0x80000000) ? (0x80000000) : (((x) ^ ((x) - 1) + 1) >> 1))
#define MASK_END(x)	\
	(((x) & 0x80000000) ? (0x80000000) : (((x) + MASK_START(x)) >> 1))

/*
 * Function Name: via_chrome9_get_vga_address
 * Description: Transfer the IO name(CRXX/SRXX to the address in MMIO base)
 * Parameters:
 * IN: IO name
 * OUT: the address of the register in MMIO
 * Status Return: none
 * Note: now it just support 32 bit OS, how to support 64bit OS
 *will be done laterly
 * */
inline unsigned long via_chrome9_get_vga_address(u16 io_name)
{
	unsigned long  io_address;

	if (IS_ZSR & io_name)
		io_address = REG_ZSR;
	else if (IS_ZGR & io_name)
		io_address = REG_ZGR;
	else if (IS_ZAR & io_name)
		io_address = REG_ZAR;
	else
		io_address = REG_ZCR;
	io_address = (unsigned long)(mmio_map_base + io_address);

	return io_address;
}

/*
 * Function Name: via_chrome9_read_misc_io
 * Description: Read the data of MISC register
 * Parameters:
 * IN:
 * OUT: Data of MISC register
 * Status Return:
 * Note:
 *
 * */
inline u8  via_chrome9_read_misc_io(void)
{
	u8 io_data;

	io_data = MMIO_RD8(mmio_map_base + REG_RMISC);

	return io_data;
}

/*
 * Function Name: via_chrome9_write_misc_io
 * Description: Write data to MISC register
 * Parameters:
 * IN: data
 * OUT:
 * Status Return:
 * Note:
 * */
inline u32  via_chrome9_write_misc_io(u8 data)
{
	MMIO_WR8((mmio_map_base + REG_WMISC), data);

	return true;
}


/*
 * Function Name:
 * Description:
 * Parameters:
 * IN:
 * OUT:
 * Status Return:
 * Note:
 * */
u8  via_chrome9_read_vga_io(u16 io_name)
{
	u8  io_port_index;
	u8	count = 0;
	u8  ret_val = 0xFF;
	u16  io_data;
	unsigned long io_address;

	io_port_index = io_name & REG_INDEX_MASK;
	io_address = via_chrome9_get_vga_address(io_name);
	while (count < 20) {
		MMIO_WR8(io_address, io_port_index);
		io_data = MMIO_RD16(io_address);
		if (((u8)io_data) == io_port_index) {
			ret_val = (u8)(io_data >> 8);
			break;
		}
		count++;
	}
	return ret_val;
}

/*
 * Function Name:
 * Description:
 * Parameters:
 * IN:
 * OUT:
 * Status Return:
 * Note:
 * */
u32   via_chrome9_write_vga_io(u16 io_name, u8 data)
{
	u8 io_port_index;
	u16 val;
	unsigned long io_address;

	io_port_index = io_name & REG_INDEX_MASK;
	val = (data << 8) | io_port_index;
	io_address = via_chrome9_get_vga_address(io_name);
	MMIO_WR16(io_address, val);
	return true;
}

/*
 * Function Name:
 * Description:
 * Parameters:
 * IN:
 * OUT:
 * Status Return:
 * Note:
 * */
u32   via_chrome9_write_vga_io_bits(u16 io_name, u8 data, u8 bits_mask)
{
	u8  tmpio_data;

	tmpio_data = via_chrome9_read_vga_io(io_name);
	data = (data & bits_mask) | (tmpio_data & (~bits_mask));
	via_chrome9_write_vga_io(io_name, data);
	return true;
}

/*
 * Function Name:
 * Description:
 * Parameters:
 * IN:
 * OUT:
 * Status Return:
 * Note:
 * */
u32   via_chrome9_write_vga_io_multi_bits(struct via_chrome9_io_reg reg_table[],
	u32 item_num)
{
	u32 i;

	for (i = 0; i < item_num; i++)
		via_chrome9_write_vga_io_bits(reg_table[i].index,
			reg_table[i].value, reg_table[i].mask);

	return true;
}

/*
 * Function Name: via_chrome9_load_regs
 * Description: Write centain value to some bits of multiple register
 * Parameters:
 * IN:
 * reg_value: centain value
 * reg_amount: the amount of multiple register
 * p_regbit: the struct indicated which bits of register should be written
 * OUT:
 * Status Return:
 * Note:
 * */
void via_chrome9_load_regs(u32 reg_value, u32 reg_amount,
	struct via_chrome9_io_bit *p_regbit)
{
	u32 reg_mask;
	u32 bit_num = 0;
	u32 data;
	u32 i, j;
	u32 shift_next_reg;
	u32 start_index, end_index;
	u32 cr_index;
	u32 get_bit;

	for (i = 0; i < reg_amount; i++) {
		reg_mask = 0;
		data = 0;
		start_index = p_regbit[i].start_bit;
		end_index = p_regbit[i].end_bit;
		cr_index = p_regbit[i].io_addr;

		shift_next_reg = bit_num;

		for (j = start_index; j <= end_index; j++) {
			reg_mask = reg_mask | (BIT0 << j);
			get_bit = (reg_value & (BIT0 << bit_num));
			data = data |
				((get_bit >> shift_next_reg) << start_index);
			bit_num++;
		}

		via_chrome9_write_vga_io_bits(cr_index, data, reg_mask);
	}
}

/*
 * Function Name:MemSet
 *
 * Description:This function set the memory region started at "pStart" and
 * length "a_LengthInBytes" to 0.
 *
 * Parameters:
 * IN:pStart------memory start address
 * lengthInBytes------memory length(in bytes).
 * */
void via_chrome9_mem_set(void *p_start,  u32 length_in_bytes)
{
	u8 *pchar = (u8 *)p_start;
	u32 i = 0;

	if (p_start == NULL)
		return ;

	for (i = 0; i < length_in_bytes; i++)
		*pchar++ = (u8)0;
	return;
}

void via_chrome9_delay_usec(struct drm_device *dev, u32 usec)
{
	long i;
	unsigned char tmp;

	if (usec > 0) {
		for (i = usec * 4; i > 0; i--)
			tmp = STANDVGA_R8(0x3c3);
	}
}

bool
via_chrome9_delay_msec(struct drm_device *dev, u32 counter)
{
	struct drm_via_chrome9_private *p_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	u32 i = 0;

	if (p_priv->pseudo_chip_revision >= REVISION_VX800_A) {
		via_chrome9_write_vga_io_bits(REG_SR5D, 0x70, BIT4+BIT5+BIT6);

		while (1) {
			if (via_chrome9_read_vga_io(REG_SR5D) & BIT7) {
				i++;
				via_chrome9_write_vga_io_bits(REG_SR5D, 0x70,
					BIT4+BIT5+BIT6);
			}

			if (i == counter)
				break;
		}
	} else {
			via_chrome9_delay_usec(dev, counter * 1000);
	}

	return true;
}

/******************************************************************
*
*                                          Common Function
*
*******************************************************************/
int via_chrome9_calculate_tu(int link_speed, unsigned long pixel_clock,
		int bpp_byte)
{
	int link_clock;
	int tu_ratio;

	KMS_DEBUG("\n");

	switch (link_speed) {
	case SPEED_162MHz:
		link_clock = 162;
		break;

	case SPEED_270MHz:
		link_clock = 270;
		break;

	default:
		link_clock = 270;
		break;
	}
	/* dp supports 18, 24, 30bpp in RGB mode */
	/* in our driver support 24bpp only now */
	bpp_byte = 24;
	tu_ratio = ((pixel_clock / 1000) * bpp_byte * 4096)/(link_clock * LANE);
	return tu_ratio;
}

