 /*
 * Copyright 1998-2011 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2011 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __VIA_CHROME9_EXTERNAL_CARD_H_
#define __VIA_CHROME9_EXTERNAL_CARD_H_

#include <linux/i2c.h>
#include "via_chrome9_drv.h"

struct via_chrome9_external_card;
struct via_chrome9_external_card_ops;

struct via_chrome9_external_card {
	char name[10];
	u32 di_port;
	struct drm_device *drm_dev;
	struct device *dev;
	struct drm_encoder *encoder;
	struct drm_connector *current_con;
	struct list_head list_connectors;
	struct mutex con_mutex;

	struct list_head list;

	struct via_chrome9_external_card_ops *ops;
	void *private_data;
};

struct via_chrome9_external_card_ops {
	int (*init)(struct i2c_client *client);
	void (*deinit)(struct i2c_client *client);
	void (*dpms)(struct via_chrome9_external_card *card, int mode);
	void (*prepare)(struct via_chrome9_external_card *card);
	void (*commit)(struct via_chrome9_external_card *card);

	void (*force)(struct via_chrome9_external_card *card,
		struct drm_connector *connector);
	enum drm_connector_status (*try_detect)(
		struct via_chrome9_external_card *card,
		struct drm_connector *connector);

	void (*mode_set)(struct via_chrome9_external_card *card,
		struct drm_display_mode *mode);
	bool (*mode_fixup)(struct via_chrome9_external_card *card,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjust_mode);

};
#define via_chrome9_external_card_to_i2c_client(card) \
	to_i2c_client(card->dev)

#define to_via_chrome9_external_card(enc) \
	(to_via_chrome9_encoder(enc)->encoder_private)


#define DPA_CLK_30M       30000000
#define DPA_CLK_50M       50000000
#define DPA_CLK_70M       70000000
#define DPA_CLK_100M      100000000
#define DPA_CLK_150M      150000000

enum {
	DPA_CLK_RANGE_30M,
	DPA_CLK_RANGE_30_50M,
	DPA_CLK_RANGE_50_70M,
	DPA_CLK_RANGE_70_100M,
	DPA_CLK_RANGE_100_150M,
	DPA_CLK_RANGE_150M,
};

struct via_chrome9_gfx_dpa_value {
	/* CR96[3:0] */
	u8 dvp0;
	/* SR2A[5] */
	u8 dvp0_data_dri_s1;
	/* SR1B[1] */
	u8 dvp0_data_dri_s;
	/* SR2A[4] */
	u8 dvp0_clock_dri_s1;
	/* SR1E[2] */
	u8 dvp0_clock_dri_s;
	/* CR9B[3:0] */
	u8 dvp1;
	/* SR65[3:0], Data and Clock driving */
	u8 dvp1_driving;
	/* CR97[3:0] */
	u8 dfp_high;
	/* CR99[3:0] */
	u8 dfp_low;
};

struct via_chrome9_gfx_dpa_tv_size_map_table {
	struct via_chrome9_gfx_dpa_value *normal_scan;
	struct via_chrome9_gfx_dpa_value *fit_scan;
	struct via_chrome9_gfx_dpa_value *over_scan;
};

struct via_chrome9_gfx_dpa_tv_type_map_table {
	struct via_chrome9_gfx_dpa_tv_size_map_table std_ntsc;
	struct via_chrome9_gfx_dpa_tv_size_map_table std_pal;
	struct via_chrome9_gfx_dpa_tv_size_map_table std_480p;
	struct via_chrome9_gfx_dpa_tv_size_map_table std_576p;
	struct via_chrome9_gfx_dpa_tv_size_map_table std_720p;
	struct via_chrome9_gfx_dpa_tv_size_map_table std_1080i;
};

struct via_chrome9_tv_dpa_tv_size_map_table {
	u32 *normal_scan;
	u32 *fit_scan;
	u32 *over_scan;
};

struct via_chrome9_tv_dpa_tv_type_map_table {
	struct via_chrome9_tv_dpa_tv_size_map_table std_ntsc;
	struct via_chrome9_tv_dpa_tv_size_map_table std_pal;
	struct via_chrome9_tv_dpa_tv_size_map_table std_480p;
	struct via_chrome9_tv_dpa_tv_size_map_table std_576P;
	struct via_chrome9_tv_dpa_tv_size_map_table std_720p;
	struct via_chrome9_tv_dpa_tv_size_map_table std_1080i;
};

struct via_chrome9_tv_dpa_table {
	int mode_index;
	struct via_chrome9_gfx_dpa_tv_type_map_table *gfx_dpa_tv_type_map_table;
	struct via_chrome9_tv_dpa_tv_type_map_table *tv_dpa_tv_type_map_table;
};

struct via_chrome9_gfx_dpa_info_table {
	u32 clock_range_index;
	struct via_chrome9_gfx_dpa_value *gfx_dpa_value;
};

struct via_chrome9_gfx_dpa_index_table {
	u32  chipset;
	struct via_chrome9_tv_dpa_table *vt1625_dpa_tbl;
	struct via_chrome9_gfx_dpa_info_table *vt1632a_dpa_tbl;
	struct via_chrome9_gfx_dpa_info_table *vt1636_dpa_tbl;
	struct via_chrome9_gfx_dpa_info_table *hard_wired_dpa_tbl;
};

extern int via_chrome9_external_card_register_connector(
		struct via_chrome9_external_card *card,
		int connector_type);

extern void via_chrome9_external_card_unregister_connector(
		struct via_chrome9_connector *connector);

extern int via_chrome9_add_external_card_driver(struct drm_device *dev);
extern void via_chrome9_del_external_card_driver(void);
extern void via_chrome9_external_card_link_enc_conn(struct drm_device *dev);
extern void via_chrome9_external_card_write(struct i2c_client *client,
		u8 reg, u8 val);
extern u8 via_chrome9_external_card_read(struct i2c_client *client, u8 reg);
extern void via_chrome9_external_card_write_mask(struct i2c_client *client,
		u8 reg, u8 val, u8 mask);
extern int via_chrome9_get_clk_range_index(u32 clk);
extern void via_chrome9_set_dpa(u32 port,
		struct via_chrome9_gfx_dpa_value *gfx_dpa_value);
extern void via_chrome9_load_user_gfx_dpa_setting(u32 port,
		struct via_gfx_dpa *via_gfx_dpa);
extern void via_chrome9_load_user_lvds_dpa_setting(
		struct via_chrome9_external_card *card,
		struct via_lvds_dpa *via_lvds_dpa);
extern void  via_chrome9_vga_gfx_dpa_setting(
		struct via_chrome9_external_card *card,
		struct drm_display_mode *mode);
extern void via_chrome9_lvds_gfx_dpa_setting(
		struct via_chrome9_external_card *card,
		struct drm_display_mode *mode);
extern void via_chrome9_dvi_gfx_dpa_setting(
		struct via_chrome9_external_card *card,
		struct drm_display_mode *mode);
extern void via_chrome9_hdmi_gfx_dpa_setting(
		struct via_chrome9_external_card *card,
		struct drm_display_mode *mode);
#ifdef SUPPORT_SIL164
extern struct via_chrome9_external_card_ops sil164_ops;
#endif
#ifdef SUPPORT_VT3192
extern struct via_chrome9_external_card_ops vt3192_ops;
#endif
#ifdef SUPPORT_VT3345
extern struct via_chrome9_external_card_ops vt3345_ops;
#endif
#ifdef SUPPORT_AD9389
extern struct via_chrome9_external_card_ops ad9389_ops;
#endif
#ifdef SUPPORT_CH7301
extern struct via_chrome9_external_card_ops ch7301_ops;
#endif

extern struct via_chrome9_gfx_dpa_index_table gfx_dpa_index_table[];
/*  template code for some of the operations
*  init(client):
	do some hardware init about the external card
	card=i2c_get_clientdata(client)
	card->support_type = OUTPUT_ANALOG; //for crt
	via_chrome9_external_card_register_connector(card, connector1_type);
	via_chrome9_external_card_register_connector(card, connector2_type);
*
*  deinit(client):
	do some hardware deinit about the external card
	card = i2c_get_clientdata(client)
	list_for_each(connector, &card->connectors, list)
		via_chrome9_external_card_unregister_connector(connector)
*
*  try_detect(card, connector)
	try detect loop procedure
	card->base.encoder_type = XXX
	connector->current_type = XXX
*/
#endif
