 /*
 * Copyright 1998-2011 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2011 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "drmP.h"
#include "via_chrome9_drm.h"
#include "via_chrome9_drv.h"
#include "via_chrome9_dma.h"
#include "via_chrome9_mm.h"
#include "via_chrome9_3d_reg.h"
#include "via_chrome9_init.h"
#include "via_chrome9_object.h"
#include "via_chrome9_display_common.h"
#include "via_chrome9_fence.h"

#define HOTPLUG_ENABLE  1


/* fence IRQ handle function. */
static irqreturn_t via_chrome9_fence_irq_handler(
		struct drm_via_chrome9_private *dev_priv)
{
	int interrupt_status2 = 0;
	interrupt_status2 = getmmioregister(dev_priv->mmio_map,
		INTERRUPTCTLREG2);
	if (interrupt_status2 & CR_INT_STATUS) {
		setmmioregister(dev_priv->mmio_map, INTERRUPTCTLREG2,
			interrupt_status2 | CR_INT_STATUS);
		/* must read these registers,
		read fence id high 8 and low 24 bits */
		getmmioregister(dev_priv->mmio_map, 0x494);
		getmmioregister(dev_priv->mmio_map, 0x49c);
		queue_work(dev_priv->p_fence->fence_wq,
			&dev_priv->p_fence->fence_work);
		return IRQ_HANDLED;
	}
	return IRQ_NONE;
}

/* vblank IRQ handle function. */
static irqreturn_t via_chrome9_vblank_irq_handler(
		struct drm_via_chrome9_private *dev_priv)
{
	int interrupt_status = 0;
	int ret = 0;

	interrupt_status = getmmioregister(dev_priv->mmio_map,
		INTERRUPT_CONTROL_REG);
	if (interrupt_status & VIA_IRQ_IGA1_VSYNC_STS) {
		drm_handle_vblank(dev_priv->ddev, 0);
		ret = 1;
	}
	if (interrupt_status & VIA_IRQ_IGA2_VSYNC_STS) {
		drm_handle_vblank(dev_priv->ddev, 1);
		ret = 1;
	}
	if (ret) {
		setmmioregister(dev_priv->mmio_map, INTERRUPT_CONTROL_REG,
			interrupt_status);
		return IRQ_HANDLED;
	}
	return IRQ_NONE;
}

/**
 * when we set the irq mask enable bit, the irq status bit will be set by 1,
 * whether the device was connected or not, so then trigger interrupt right now.
 * so we should write 1 to clear the status bit when enable irq mask.
 */
static void via_chrome9_hpd_irq_set(struct drm_via_chrome9_private *dev_priv)
{
	u32 mm_200, mm_1280;
	int pci_device = dev_priv->ddev->pci_device;

	mm_200 = MMIO_RD(0x200);
	mm_1280 = MMIO_RD(0x1280);

	if (true == dev_priv->irq.irq_all)
		mm_200 |= VIA_IRQ_ALL_EN;

	if (true == dev_priv->irq.dvp1_sense_int) {
		mm_200 |= VIA_IRQ_DVP1_EN;
		mm_200 |= VIA_IRQ_DVP1_STS;
	}
	if (true == dev_priv->irq.dp_sense_int[0]) {
		mm_1280 |= VIA_IRQ_DP1_EN;
		mm_1280 |= VIA_IRQ_DP1_STS;
	}
	if (true == dev_priv->irq.dp_sense_int[1]) {
		mm_1280 |= VIA_IRQ_DP2_EN;
		mm_1280 |= VIA_IRQ_DP2_STS;
	}

	if (VX800_DEVICE == pci_device) {
		if (true == dev_priv->irq.internal_tmds_sense_int) {
			mm_1280 |= VIA_IRQ_IN_TMDS_EN;
			mm_1280 |= VIA_IRQ_IN_TMDS_STS;
		}
	}

	MMIO_WR(0x1280, mm_1280);
	MMIO_WR(0x200, mm_200);

}

void via_chrome9_hpd_init(struct drm_device *dev)
{

#if HOTPLUG_ENABLE
	struct drm_via_chrome9_private *dev_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;

	KMS_DEBUG("HOTPLUG ENABLED.\n");

	dev_priv->irq.irq_all = true;
	/* DVP1.  */
	dev_priv->irq.dvp1_sense_int = true;

	/* DP1.  */
	dev_priv->irq.dp_sense_int[0] = true;
	/* DP2.  */
	dev_priv->irq.dp_sense_int[1] = true;

	if (VX800_DEVICE == dev->pci_device)
		/* Internal DVI */
		dev_priv->irq.internal_tmds_sense_int = true;

	if (dev_priv->irq.installed)
		via_chrome9_hpd_irq_set(dev_priv);
#endif

	KMS_DEBUG("hpd initialized.\n");
}

void via_chrome9_hpd_fini(struct drm_device *dev)
{
	struct drm_via_chrome9_private *dev_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;

	dev_priv->irq.irq_all = false;
}

static irqreturn_t via_chrome9_hpd_irq_process(
		struct drm_via_chrome9_private *dev_priv)
{
	irqreturn_t ret = IRQ_NONE;
	bool queue_hotplug = false;
	int queue_work_status;
	u32 mm_200, mm_1280;
	u32 mm_c730, mm_c7b0;
	int pci_device = dev_priv->ddev->pci_device;

	/* bak 0x3c4 (SR_REG index) */
	u8 reg_3c4_save = STANDVGA_R8(REG_ZSR);
	/* bak 0x3d4 (CR_REG index) */
	u8 reg_3d4_save = STANDVGA_R8(REG_ZCR);

	mm_200 = MMIO_RD(0x200);
	mm_1280 = MMIO_RD(0x1280);

	KMS_DEBUG("************  mm_200 = 0x%x, mm_1280 = 0x%x  ************\n",
		mm_200, mm_1280);

	/**
	 * only when 0x200[31] = 1, these IRQs can be triggered.
	 * we handler devices hotplug IRQs only.
	 */

	/* DP1 or Internal HDMI sense */
	if (VIA_IRQ_DP1_EN & mm_1280) {
		if (VIA_IRQ_DP1_STS & mm_1280) {
			mm_c730 = MMIO_RD(0xc730);
			switch (mm_c730 & 0xC0000000) {
			case VIA_IRQ_DP_HOT_IRQ:
				KMS_DEBUG("VIA_IRQ_DP1_HOT_IRQ!\n");
				queue_hotplug = true;
				break;
			case VIA_IRQ_DP_HOT_UNPLUG:
				KMS_DEBUG("VIA_IRQ_DP1(HDMI)_HOT_UNPLUG!\n");
				queue_hotplug = true;
				break;
			case VIA_IRQ_DP_HOT_PLUG:
				KMS_DEBUG("VIA_IRQ_DP1(HDMI)_HOT_PLUG!\n");
				queue_hotplug = true;
				break;
			case VIA_IRQ_DP_NO_INT:
				KMS_DEBUG("VIA_IRQ_DP1_NO_INT!\n");
				break;
			}
			ret = IRQ_HANDLED;
		}
	}

	/* DP2 sense */
	if (VIA_IRQ_DP2_EN & mm_1280) {
		if (VIA_IRQ_DP2_STS & mm_1280) {
			mm_c7b0 = MMIO_RD(0xc7b0);
			switch (mm_c7b0 & 0xC0000000) {
			case VIA_IRQ_DP_HOT_IRQ:
				KMS_DEBUG("VIA_IRQ_DP2_HOT_IRQ!\n");
				queue_hotplug = true;
				break;
			case VIA_IRQ_DP_HOT_UNPLUG:
				KMS_DEBUG("VIA_IRQ_DP2_HOT_UNPLUG!\n");
				queue_hotplug = true;
				break;
			case VIA_IRQ_DP_HOT_PLUG:
				KMS_DEBUG("VIA_IRQ_DP2_HOT_PLUG!\n");
				queue_hotplug = true;
				break;
			case VIA_IRQ_DP_NO_INT:
				KMS_DEBUG("VIA_IRQ_DP2_NO_INT!\n");
				break;
			}
			ret = IRQ_HANDLED;
		}
	}

	/* CRT sense */
	if (VIA_IRQ_CRT_EN & mm_1280) {
		if (VIA_IRQ_CRT_STS & mm_1280) {
			KMS_DEBUG("VIA_IRQ_CRT_HOT_PLUG!\n");
			/*queue_hotplug = true; */
		}
	}

	/* internal TMDS  sense */
	if (VX800_DEVICE == pci_device) {
		if (VIA_IRQ_IN_TMDS_EN & mm_1280) {
			if (VIA_IRQ_IN_TMDS_STS & mm_1280) {
				if (BIT5 & via_chrome9_read_vga_io(REG_SR3E))
					KMS_DEBUG("VIA_IRQ_INTERNAL_DVI_IRQ: \
						connected!\n");
				else
					KMS_DEBUG("VIA_IRQ_INTERNAL_DVI_IRQ: \
						disconnected!\n");

				KMS_DEBUG("SR3E=%x\n",
					via_chrome9_read_vga_io(REG_SR3E));
				queue_hotplug = true;
				ret = IRQ_HANDLED;
			}
		}
	}

	/* clear interrupt status on 0x1280. */
	MMIO_WR(0x1280, mm_1280);

	/* DVP1 device sense */
	if (VIA_IRQ_DVP1_EN & mm_200) {
		if (VIA_IRQ_DVP1_STS & mm_200) {
			KMS_DEBUG("VIA_IRQ_DVP1_SENSE_IRQ!\n");
			dev_priv->is_ad9389_hpd_irq = true;
			queue_hotplug = true;
			ret = IRQ_HANDLED;
		}
	}

	/* clear interrupt status on 0x200. */
	MMIO_WR(0x200, mm_200);

	if (queue_hotplug) {
		if (!work_pending(&dev_priv->hotplug_work)) {
			queue_work_status = queue_work(dev_priv->wq,
				&dev_priv->hotplug_work);
			KMS_DEBUG("queue work request %s !\n",
				(queue_work_status == 0) ? "failed" : "passed");
		} else {
			KMS_DEBUG("queue work list pending !\n");
		}
	}

	KMS_DEBUG("############  mm_200 = 0x%x, \
		mm_1280 = 0x%x  ############\n",
		MMIO_RD(0x200), MMIO_RD(0x1280));

	/* restore the SR_REG index */
	STANDVGA_W8(REG_ZSR, reg_3c4_save);
	/* restore the CR_REG index */
	STANDVGA_W8(REG_ZCR, reg_3d4_save);

	return ret;
}


void via_chrome9_connector_hotplug(struct drm_connector *connector)
{
	struct via_chrome9_connector *via_chrome9_connector =
		to_via_chrome9_connector(connector);
	struct via_dp_connector_private_info *via_dp_connector_private_info =
		(struct via_dp_connector_private_info *)
		via_chrome9_connector->connector_private;

	struct drm_connector_helper_funcs *via_dp_helper_funcs =
		(struct drm_connector_helper_funcs *)connector->helper_private;
	struct drm_encoder  *encoder;
	struct via_chrome9_encoder *via_chrome9_encoder = NULL;
	struct via_dp_encoder_private_info *via_dp_encoder_private_info = NULL;

	KMS_DEBUG("via_chrome9_connector->connector_id =%x\n",
		via_chrome9_connector->connector_id);

	if (VIA_CHROME9_DEVICE_CONNECTOR_DP_1 ==
		via_chrome9_connector->connector_id) {
		via_chrome9_dp_1_read_dpcd_version(connector);
		if (via_dp_connector_private_info->dpcd_version) {
			KMS_DEBUG("DP1 read dpcd version = 0x%x !\n",
				via_dp_connector_private_info->dpcd_version);
	if (DP_LINKSTATUS_LOST ==
		via_chrome9_dp_1_read_dpcd_link_status(connector)) {
		encoder =
			via_dp_helper_funcs->best_encoder(connector);
		if (encoder) {
			via_chrome9_encoder =
				to_via_chrome9_encoder(encoder);
			via_dp_encoder_private_info =
				(struct via_dp_encoder_private_info *)
				via_chrome9_encoder->encoder_private;
		} else {
			KMS_DEBUG("DP1 connector does not have encoder.\n");
			return;
		}

		via_dp_encoder_private_info->link_training_done = false;
		via_dp_encoder_private_info->link_speed = SPEED_NONE;
		via_chrome9_dp_1_init_ephy(connector);
		KMS_DEBUG("DP1 init ephy done!\n");
	} else {
			KMS_DEBUG("DP1 link status is linked !\n");
	}

		} else {
			KMS_DEBUG("DP1 read dpcd version failed!\n");
		}
	}

	if (VIA_CHROME9_DEVICE_CONNECTOR_DP_2 ==
		via_chrome9_connector->connector_id) {
		via_chrome9_dp_2_read_dpcd_version(connector);
		if (via_dp_connector_private_info->dpcd_version) {
			KMS_DEBUG("DP2 read dpcd version = 0x%x !\n",
			via_dp_connector_private_info->dpcd_version);
			if (DP_LINKSTATUS_LOST ==
				via_chrome9_dp_2_read_dpcd_link_status(
					connector)) {
				encoder = via_dp_helper_funcs->best_encoder(
					connector);
				if (encoder) {
					via_chrome9_encoder =
						to_via_chrome9_encoder(encoder);
					via_dp_encoder_private_info = (
					struct via_dp_encoder_private_info *)
						via_chrome9_encoder
						->encoder_private;
				} else {
					KMS_DEBUG(
						"DP2 connector does not have \
						encoder.\n");
					return;
				}
			via_dp_encoder_private_info->link_training_done =
				false;
			via_dp_encoder_private_info->link_speed =
				SPEED_NONE;
			via_chrome9_dp_2_init_ephy(connector);
			KMS_DEBUG("DP2 init ephy done!\n");
			} else {
				KMS_DEBUG("DP2 read dpcd version failed!\n");
			}
		} else {
			KMS_DEBUG("DP2 read dpcd version failed!\n");
		}
	}
}


/*
 * Handle hotplug events outside the interrupt handler proper.
 */
static void via_chrome9_hotplug_work_func(struct work_struct *work)
{
	struct drm_via_chrome9_private *dev_priv = container_of(work,
		struct drm_via_chrome9_private, hotplug_work);
	struct drm_device *dev = dev_priv->ddev;
	struct drm_mode_config *mode_config = &dev->mode_config;
	struct drm_connector *connector;
	if (mode_config->num_connector) {
		list_for_each_entry(connector, &mode_config->connector_list,
			head)
		via_chrome9_connector_hotplug(connector);
	}
	KMS_DEBUG("\n");
	schedule_timeout_uninterruptible(msecs_to_jiffies(500));
	KMS_DEBUG("Done delay 500ms, prepare to send event!\n");

	/* handle it in fb and fire off a uevent and
	let userspace tell us what to do */
	drm_helper_hpd_irq_event(dev);
}


/*=== IRQ function define in KMS path ===*/
void via_chrome9_driver_irq_preinstall(struct drm_device *dev)
{
	struct drm_via_chrome9_private *dev_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;

	unsigned char i;

	KMS_DEBUG("\n");

	INIT_WORK(&dev_priv->hotplug_work, via_chrome9_hotplug_work_func);

	/* Disable *all* interrupts */
	for (i = 0; i < 2; i++)
		dev_priv->irq.dp_sense_int[i] = false;

	dev_priv->irq.dvp1_sense_int = false;
	dev_priv->irq.internal_tmds_sense_int = false;
	via_chrome9_hpd_irq_set(dev_priv);

#if HOTPLUG_ENABLE
	/* Clear bits */
	via_chrome9_hpd_irq_process(dev_priv);
#endif

}

int via_chrome9_driver_irq_postinstall(struct drm_device *dev)
{
	return 0;
}

void via_chrome9_driver_irq_uninstall(struct drm_device *dev)
{
	struct drm_via_chrome9_private *dev_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;
	unsigned i;

	if (dev_priv == NULL)
		return;

	/* Disable *all* interrupts */
	for (i = 0; i < 2; i++)
		dev_priv->irq.dp_sense_int[i] = false;

	dev_priv->irq.dvp1_sense_int = false;

	if (VX800_DEVICE == dev->pci_device)
		dev_priv->irq.internal_tmds_sense_int = false;

	via_chrome9_irq_uninstall_nonKMS(dev_priv);
	via_chrome9_hpd_irq_set(dev_priv);
}

irqreturn_t via_chrome9_driver_irq_handler(DRM_IRQ_ARGS)
{
	struct drm_device *dev = (struct drm_device *) arg;
	struct drm_via_chrome9_private *dev_priv =
		(struct drm_via_chrome9_private *)dev->dev_private;

	if (via_chrome9_fence_irq_handler(dev_priv) == IRQ_HANDLED)
		return IRQ_HANDLED;

	/* handle vblank irq */
	if (via_chrome9_vblank_irq_handler(dev_priv) == IRQ_HANDLED)
		return IRQ_HANDLED;

	return via_chrome9_hpd_irq_process(dev_priv);
}

int via_chrome9_enable_vblank(struct drm_device *dev, int crtc)
{
	u32 mm_200;

	if (crtc < 0 || crtc >= dev->num_crtcs) {
		DRM_ERROR("Invalid crtc %d\n", crtc);
		return -EINVAL;
	}

	mm_200 = MMIO_RD(INTERRUPT_CONTROL_REG);
	if (crtc == 0) {
		mm_200 |= VIA_IRQ_IGA1_VSYNC_EN | VIA_IRQ_IGA1_VSYNC_STS;
		via_chrome9_write_vga_io_bits(REG_CRF3, 0, BIT1);
		via_chrome9_write_vga_io_bits(REG_CR11, BIT4, BIT4);
	}
	if (crtc == 1) {
		mm_200 |= VIA_IRQ_IGA2_VSYNC_EN | VIA_IRQ_IGA2_VSYNC_STS;
		via_chrome9_write_vga_io_bits(REG_CRF3, 0, BIT1);
		via_chrome9_write_vga_io_bits(REG_CR11, BIT4, BIT4);
	}

	MMIO_WR(INTERRUPT_CONTROL_REG, mm_200);
	return 0;
}

void via_chrome9_disable_vblank(struct drm_device *dev, int crtc)
{
	u32 mm_200;

	if (crtc < 0 || crtc >= dev->num_crtcs) {
		DRM_ERROR("Invalid crtc %d\n", crtc);
		return;
	}

	mm_200 = MMIO_RD(INTERRUPT_CONTROL_REG);
	if (crtc == 0)
		mm_200 &= ~VIA_IRQ_IGA1_VSYNC_EN;
	if (crtc == 1)
		mm_200 &= ~VIA_IRQ_IGA2_VSYNC_EN;

	MMIO_WR(INTERRUPT_CONTROL_REG, mm_200);
}

/*=== IRQ function define in non-KMS path ===*/
void via_chrome9_irq_preinstall_nonKMS(
		struct drm_via_chrome9_private *dev_priv)
{

}

void via_chrome9_irq_uninstall_nonKMS(
		struct drm_via_chrome9_private *dev_priv)
{
	unsigned long interrupt_status;
	interrupt_status = getmmioregister(dev_priv->mmio_map,
		INTERRUPT_CONTROL_REG);
	/*if other interrupt enable bits are not set,
	then dsiable hw gfx's intertupt*/
	if (!(interrupt_status & INTERRUPT_ENABLE_MASK))
		setmmioregister(dev_priv->mmio_map,
			INTERRUPT_CONTROL_REG,
			interrupt_status & (~INTERRUPT_ENABLE));
}

void via_chrome9_irq_postinstall_nonKMS(
		struct drm_via_chrome9_private *dev_priv)
{
	unsigned long interrupt_status = 0;

	interrupt_status = getmmioregister(dev_priv->mmio_map,
		INTERRUPT_CONTROL_REG);
	if (!(interrupt_status & INTERRUPT_ENABLE))
		setmmioregister(dev_priv->mmio_map,
			INTERRUPT_CONTROL_REG,
			INTERRUPT_ENABLE);

}

irqreturn_t via_chrome9_driver_irq_handler_nonKMS(DRM_IRQ_ARGS)
{
	struct drm_via_chrome9_private *dev_priv =
		(struct drm_via_chrome9_private *)arg;
	u32 mm_1280;

	if (via_chrome9_fence_irq_handler(dev_priv) == IRQ_HANDLED)
		return IRQ_HANDLED;

	/* add dp(hdmi), dp-2 irq handle on non-KMS path. */
	mm_1280 = getmmioregister(dev_priv->mmio_map, 0x1280);
	if (0x2800 & mm_1280) {
		setmmioregister(dev_priv->mmio_map, 0x1280, mm_1280);
		return IRQ_HANDLED;
	}

	return IRQ_NONE;
}


