 /*
 * Copyright 1998-2011 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2011 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef VIA_CHROME9_MODE_H
#define VIA_CHROME9_MODE_H

#include "drmP.h"
#include "drm_mode.h"
#include "drm_crtc.h"
#include "drm_crtc_helper.h"
#include "via_chrome9_fb.h"
#include "via_chrome9_i2c.h"
#include "via_chrome9_display_common.h"

/* encoder type index */
#define VIA_CHROME9_ENCODER_DAC_INDEX		0x00000001
#define VIA_CHROME9_ENCODER_LVDS_INDEX		0x00000002
#define VIA_CHROME9_ENCODER_TMDS_INDEX		0x00000003
#define VIA_CHROME9_ENCODER_DP_1_INDEX		0x00000004
#define VIA_CHROME9_ENCODER_DP_2_INDEX		0x00000005
#define VIA_CHROME9_ENCODER_TV_INDEX		0x00000006
#define VIA_CHROME9_ENCODER_HDMI_INDEX		0x00000007
#define VIA_CHROME9_ENCODER_LVDS_2_INDEX	0x00000008

/* connector type index*/
#define VIA_CHROME9_CONNECTOR_CRT_INDEX		0x00000001
#define VIA_CHROME9_CONNECTOR_LCD_INDEX		0x00000002
#define VIA_CHROME9_CONNECTOR_DVI_D_INDEX	0x00000003
#define VIA_CHROME9_CONNECTOR_DP_1_INDEX	0x00000004
#define VIA_CHROME9_CONNECTOR_DP_2_INDEX	0x00000005
#define VIA_CHROME9_CONNECTOR_TV_INDEX		0x00000006
#define VIA_CHROME9_CONNECTOR_HDMI_INDEX		0x00000007
#define VIA_CHROME9_CONNECTOR_HDMI_DVI_D_INDEX	0x00000008
#define VIA_CHROME9_CONNECTOR_LCD_2_INDEX	0x00000009
/* encoder type */
#define VIA_CHROME9_DEVICE_ENCODER_DAC	(0x1L << VIA_CHROME9_ENCODER_DAC_INDEX)
#define VIA_CHROME9_DEVICE_ENCODER_LVDS	(0x1L << VIA_CHROME9_ENCODER_LVDS_INDEX)
#define VIA_CHROME9_DEVICE_ENCODER_TMDS	(0x1L << VIA_CHROME9_ENCODER_TMDS_INDEX)
#define VIA_CHROME9_DEVICE_ENCODER_DP_1	(0x1L << VIA_CHROME9_ENCODER_DP_1_INDEX)
#define VIA_CHROME9_DEVICE_ENCODER_DP_2	(0x1L << VIA_CHROME9_ENCODER_DP_2_INDEX)
#define VIA_CHROME9_DEVICE_ENCODER_TV	(0x1L << VIA_CHROME9_ENCODER_TV_INDEX)
#define VIA_CHROME9_DEVICE_ENCODER_HDMI	(0x1L << VIA_CHROME9_ENCODER_HDMI_INDEX)
#define VIA_CHROME9_DEVICE_ENCODER_LVDS_2	\
	(0x1L << VIA_CHROME9_ENCODER_LVDS_2_INDEX)

/* connector type */
#define VIA_CHROME9_DEVICE_CONNECTOR_CRT	\
	(0x1L << VIA_CHROME9_CONNECTOR_CRT_INDEX)
#define VIA_CHROME9_DEVICE_CONNECTOR_LCD	\
	(0x1L << VIA_CHROME9_CONNECTOR_LCD_INDEX)
#define VIA_CHROME9_DEVICE_CONNECTOR_DVI_D	\
	(0x1L << VIA_CHROME9_CONNECTOR_DVI_D_INDEX)
#define VIA_CHROME9_DEVICE_CONNECTOR_DP_1	\
	(0x1L << VIA_CHROME9_CONNECTOR_DP_1_INDEX)
#define VIA_CHROME9_DEVICE_CONNECTOR_DP_2	\
	(0x1L << VIA_CHROME9_CONNECTOR_DP_2_INDEX)
#define VIA_CHROME9_DEVICE_CONNECTOR_TV	\
	(0x1L << VIA_CHROME9_CONNECTOR_TV_INDEX)
#define VIA_CHROME9_DEVICE_CONNECTOR_HDMI	\
	(0x1L << VIA_CHROME9_CONNECTOR_HDMI_INDEX)
#define VIA_CHROME9_DEVICE_CONNECTOR_HDMI_DVI_D	\
	(0x1L << VIA_CHROME9_CONNECTOR_HDMI_DVI_D_INDEX)
#define VIA_CHROME9_DEVICE_CONNECTOR_LCD_2	\
	(0x1L << VIA_CHROME9_CONNECTOR_LCD_2_INDEX)
#define VIA_CHROME9_DEVICE_CONNECTOR_UNKNOW 0xFFFFFFFF

#define VIA_CHROME9_CONNECTOR_LIMIT 4

/* Video Mode Index (FB device driver's res. index)*/
#define M480x640        0x000
#define M640x480        0x001
#define M720x480        0x002
#define M720x576        0x003
#define M800x480        0x004
#define M800x600        0x005
#define M848x480        0x006
#define M852x480        0x007
#define M960x600        0x008
#define M1000x600       0x009
#define M1024x512       0x00A
#define M1024x576       0x00B
#define M1024x600       0x00C
#define M1024x768       0x00D
/* Interlace Mode */
#define M1024x768_I     0x20D
#define M1088x612       0x00E
#define M1152x720       0x00F
#define M1152x864       0x010
/* Interlace Mode */
#define M1152x864_I     0x210
#define M1200x720       0x011
#define M1280x600       0x012
#define M1280x720       0x013
#define M1280x768       0x014
#define M1280x800       0x015
#define M1280x960       0x016
#define M1280x1024      0x017
/* Interlace Mode */
#define M1280x1024_I    0x217
#define M1360x768       0x018
/* Reduce Banking */
#define M1360x768_RB    0x118
#define M1366x768       0x019
#define M1368x768       0x01A
#define M1400x1050      0x01B
/* Reduce Banking */
#define M1400x1050_RB   0x11B
#define M1440x900       0x01C
/* Reduce Banking */
#define M1440x900_RB    0x11C
#define M1440x1050      0x01D
#define M1600x900       0x01E
/* Reduce Banking */
#define M1600x900_RB    0x11E
#define M1600x1024      0x01F
#define M1600x1200      0x020
/* Interlace Mode */
#define M1600x1200_I    0x220
#define M1680x600       0x021
#define M1680x1050      0x022
/* Reduce Banking */
#define M1680x1050_RB   0x122
#define M1792x1344      0x023
#define M1856x1392      0x024
#define M1920x1080      0x025
/* Reduce Banking */
#define M1920x1080_RB   0x125
/* Interlace Mode */
#define M1920x1080_I    0x225
#define M1920x1200      0x026
/* Reduce Banking */
#define M1920x1200_RB   0x126
#define M1920x1440      0x027
#define M2048x1536      0x028
/* General timing, only for LCD*/
#define M800x480_GEN    0x029
#define M720x400        0x030
#define M832x624        0x031
#define M1280x854       0x032
#define M_INVALID       0xFFF

#define IGA1	0
#define IGA2	1
/* Define VIA Graphic Chip IGA Scaling support caps */
/* Support IGA1 DownScaling */
#define     CAPS_IGA1_DOWNSCALING  BIT0
/* Support IGA1 UpScaling */
#define     CAPS_IGA1_EXPAND             BIT1
/* Support IGA2 DownScaling */
#define     CAPS_IGA2_DOWNSCALING  BIT2

#define VIA_NONE_SCALING	0
#define VIA_PANNING		BIT0
/* IGA Scaling down */
#define VIA_SCALING_HW		BIT1
#define VIA_SCALING_3D		BIT2
#define VIA_HOR_EXPAND		BIT3
#define VIA_VER_EXPAND		BIT4
/* IGA Scaling up */
#define VIA_EXPAND		(BIT3 | BIT4)

/* Define IGA Scaling up/down status :  Horizontal or Vertical  */
/* Is IGA Hor scaling up/down status */
#define    HOR_SCALE   BIT0
/* Is IGA Ver scaling up/down status */
#define    VER_SCALE   BIT1
/* Is IGA Hor and Ver scaling up/down status */
#define    HOR_VER_SCALE   (BIT0 | BIT1)


/* Definition CRTC Timing Index */
#define VIA_H_TOTAL_INDEX						0
#define VIA_H_ADDR_INDEX						1
#define VIA_H_BLANK_START_INDEX				2
#define VIA_H_BLANK_END_INDEX					3
#define VIA_H_SYNC_START_INDEX				4
#define VIA_H_SYNC_END_INDEX					5
#define VIA_V_TOTAL_INDEX						6
#define VIA_V_ADDR_INDEX						7
#define VIA_V_BLANK_START_INDEX				8
#define VIA_V_BLANK_END_INDEX					9
#define VIA_V_SYNC_START_INDEX				10
#define VIA_V_SYNC_END_INDEX					11
#define VIA_H_TOTAL_SHADOW_INDEX				12
#define VIA_H_BLANK_END_SHADOW_INDEX		13
#define VIA_V_TOTAL_SHADOW_INDEX				14
#define VIA_V_ADDR_SHADOW_INDEX				15
#define VIA_V_BLANK_SATRT_SHADOW_INDEX		16
#define VIA_V_BLANK_END_SHADOW_INDEX		17
#define VIA_V_SYNC_SATRT_SHADOW_INDEX		18
#define VIA_V_SYNC_END_SHADOW_INDEX			19

/* Define Register Number for IGA1 CRTC Timing */
 /* location: {CR00,0,7},{CR36,3,3} */
#define VIA_IGA1_HOR_TOTAL_REG_NUM			2
/* location: {CR01,0,7},{CR45,1,1} */
#define VIA_IGA1_HOR_ADDR_REG_NUM			2
/* location: {CR02,0,7},{CR45,2,2} */
#define VIA_IGA1_HOR_BLANK_START_REG_NUM	2
/* location: {CR03,0,4},{CR05,7,7},{CR33,5,5} */
#define VIA_IGA1_HOR_BLANK_END_REG_NUM		3
/* location: {CR04,0,7},{CR33,4,4} */
#define VIA_IGA1_HOR_SYNC_START_REG_NUM		2
/* location: {CR05,0,4} */
#define VIA_IGA1_HOR_SYNC_END_REG_NUM		1
/* location: {CR06,0,7},{CR07,0,0},{CR07,5,5},{CR35,0,0} */
#define VIA_IGA1_VER_TOTAL_REG_NUM			4
/* location: {CR12,0,7},{CR07,1,1},{CR07,6,6},{CR35,2,2} */
#define VIA_IGA1_VER_ADDR_REG_NUM			4
/* location: {CR15,0,7},{CR07,3,3},{CR09,5,5},{CR35,3,3} */
#define VIA_IGA1_VER_BLANK_START_REG_NUM	4
/* location: {CR16,0,7} */
#define VIA_IGA1_VER_BLANK_END_REG_NUM		1
/* location: {CR10,0,7},{CR07,2,2},{CR07,7,7},{CR35,1,1} */
#define VIA_IGA1_VER_SYNC_START_REG_NUM		4
/* location: {CR11,0,3} */
#define VIA_IGA1_VER_SYNC_END_REG_NUM		1

/* Define Register Number for IGA2 CRTC Timing */
/* location: {CR50,0,7},{CR55,0,3} */
#define VIA_IGA2_HOR_TOTAL_REG_NUM			2
/* location: {CR51,0,7},{CR55,4,6},{CR55,7,7} */
#define VIA_IGA2_HOR_ADDR_REG_NUM			3
/* location: {CR52,0,7},{CR54,0,2},{CR6B,0,0} */
#define VIA_IGA2_HOR_BLANK_START_REG_NUM	3
/* location: {CR53,0,7},{CR54,3,5},{CR5D,6,6} */
#define VIA_IGA2_HOR_BLANK_END_REG_NUM		3
/* location: {CR56,0,7},{CR54,6,7},{CR5C,7,7} */
/* VT3314 and Later: {CR56,0,7},{CR54,6,7},{CR5C,7,7}, {CR5D,7,7} */
#define VIA_IGA2_HOR_SYNC_START_REG_NUM		4
/* location: {CR57,0,7},{CR5C,6,6} */
#define VIA_IGA2_HOR_SYNC_END_REG_NUM		2
/* location: {CR58,0,7},{CR5D,0,2} */
#define VIA_IGA2_VER_TOTAL_REG_NUM			2
/* location: {CR59,0,7},{CR5D,3,5} */
#define VIA_IGA2_VER_ADDR_REG_NUM			2
/* location: {CR5A,0,7},{CR5C,0,2} */
#define VIA_IGA2_VER_BLANK_START_REG_NUM	2
/* location: {CR5E,0,7},{CR5C,3,5} */
#define VIA_IGA2_VER_BLANK_END_REG_NUM		2
/* location: {CR5E,0,7},{CR5F,5,7} */
#define VIA_IGA2_VER_SYNC_START_REG_NUM		2
/* location: {CR5F,0,4} */
#define VIA_IGA2_VER_SYNC_END_REG_NUM		1

#define VIA_IGA1_FIFO_DEPTH_SELECT_REG_NUM			1
#define VIA_IGA1_FIFO_THRESHOLD_REG_NUM				2
#define VIA_IGA1_FIFO_HIGH_THRESHOLD_REG_NUM		2
#define VIA_IGA1_DISPLAY_QUEUE_EXPIRE_NUM_REG_NUM	2

#define VIA_IGA2_FIFO_DEPTH_SELECT_REG_NUM			3
#define VIA_IGA2_FIFO_THRESHOLD_REG_NUM				2
#define VIA_IGA2_FIFO_HIGH_THRESHOLD_REG_NUM		2
#define VIA_IGA2_DISPLAY_QUEUE_EXPIRE_NUM_REG_NUM	1

/* IGA1 pixel timing Registers */
/* [15:0] */
#define     VIA_IGA1_PIX_H_TOTAL_REG	0x8400
/* [31:16] */
#define     VIA_IGA1_PIX_H_ADDR_REG		0x8400
/* [15:0] */
#define     VIA_IGA1_PIX_H_BNK_ST_REG	0x8404
/* [31:16] */
#define     VIA_IGA1_PIX_H_BNK_END_REG	0x8404
/* [15:0]*/
#define     VIA_IGA1_PIX_H_SYNC_ST_REG	0x8408
/* [31:16]*/
#define     VIA_IGA1_PIX_H_SYNC_END_REG	0x8408
/* [10:0] */
#define     VIA_IGA1_PIX_V_TOTAL_REG	0x8424
/* [26:16] */
#define     VIA_IGA1_PIX_V_ADDR_REG		0x8424
/* [10:0] */
#define     VIA_IGA1_PIX_V_BNK_ST_REG	0x8428
/* [26:16] */
#define     VIA_IGA1_PIX_V_BNK_END_REG	0x8428
/* [10:0] */
#define     VIA_IGA1_PIX_V_SYNC_ST_REG	0x842C
/* [15:12] */
#define     VIA_IGA1_PIX_V_SYNC_END_REG 0x842C
/* [15:0] */
#define     VIA_IGA1_PIX_HALF_LINE_REG	0x8434
/* [15:0] */
#define     VIA_IGA1_PIX_H_TOTAL_MASK	0x0000FFFF
/* [31:16] */
#define     VIA_IGA1_PIX_H_ADDR_MASK	0xFFFF0000
/* [15:0] */
#define     VIA_IGA1_PIX_H_BNK_ST_MASK	0x0000FFFF
/* [31:16] */
#define     VIA_IGA1_PIX_H_BNK_END_MASK 0xFFFF0000
/* [15:0] */
#define     VIA_IGA1_PIX_H_SYNC_ST_MASK 0x0000FFFF
/* [31:16] */
#define     VIA_IGA1_PIX_H_SYNC_END_MASK 0xFFFF0000
/* [10:0] */
#define     VIA_IGA1_PIX_V_TOTAL_MASK	0x000007FF
/* [26:16] */
#define     VIA_IGA1_PIX_V_ADDR_MASK	0x07FF0000
/* [10:0] */
#define     VIA_IGA1_PIX_V_BNK_ST_MASK	0x000007FF
/* [26:16] */
#define     VIA_IGA1_PIX_V_BNK_END_MASK	0x07FF0000
/* [10:0] */
#define     VIA_IGA1_PIX_V_SYNC_ST_MASK 0x000007FF
/* [15:12] */
#define     VIA_IGA1_PIX_V_SYNC_END_MASK 0x0000F000
/* [15:0] */
#define     VIA_IGA1_PIX_HALF_LINE_MASK	0x0000FFFF

/***************************************************/
/* Definition IGA1 Design Method of CRTC Registers */
/***************************************************/
#define VIA_IGA1_HOR_TOTAL_FORMULA(x)			(((x) / 8) - 5)
#define VIA_IGA1_HOR_ADDR_FORMULA(x)				(((x) / 8) - 1)
#define VIA_IGA1_HOR_BLANK_START_FORMULA(x)		(((x) / 8) - 1)
#define VIA_IGA1_HOR_BLANK_END_FORMULA(x)		(((x) / 8) - 1)
#define VIA_IGA1_HOR_SYNC_START_FORMULA(x)		(((x) / 8) - 1)
#define VIA_IGA1_HOR_SYNC_END_FORMULA(x)			(((x) / 8) - 1)

#define VIA_IGA1_VER_TOTAL_FORMULA(x)				((x) - 2)
#define VIA_IGA1_VER_ADDR_FORMULA(x)				((x) - 1)
#define VIA_IGA1_VER_BLANK_START_FORMULA(x)		((x) - 1)
#define VIA_IGA1_VER_BLANK_END_FORMULA(x)		((x) - 1)
#define VIA_IGA1_VER_SYNC_START_FORMULA(x)		((x) - 1)
#define VIA_IGA1_VER_SYNC_END_FORMULA(x)			((x) - 1)

/***************************************************/
/* Definition IGA2 Design Method of CRTC Registers */
/***************************************************/
#define VIA_IGA2_HOR_TOTAL_FORMULA(x)			((x) - 1)
#define VIA_IGA2_HOR_ADDR_FORMULA(x)				((x) - 1)
#define VIA_IGA2_HOR_BLANK_START_FORMULA(x)		((x) - 1)
#define VIA_IGA2_HOR_BLANK_END_FORMULA(x)		((x) - 1)
#define VIA_IGA2_HOR_SYNC_START_FORMULA(x)		((x) - 1)
#define VIA_IGA2_HOR_SYNC_END_FORMULA(x)			((x) - 1)

#define VIA_IGA2_VER_TOTAL_FORMULA(x)				((x) - 1)
#define VIA_IGA2_VER_ADDR_FORMULA(x)				((x) - 1)
#define VIA_IGA2_VER_BLANK_START_FORMULA(x)		((x) - 1)
#define VIA_IGA2_VER_BLANK_END_FORMULA(x)		((x) - 1)
#define VIA_IGA2_VER_SYNC_START_FORMULA(x)		((x) - 1)
#define VIA_IGA2_VER_SYNC_END_FORMULA(x)			((x) - 1)

/*******************************************************/
/*Definition IGA1 Design Method of FIFO Registers*/
/*******************************************************/
#define VIA_IGA1_FIFO_DEPTH_SELECT_FORMULA(x)			((x / 2) - 1)
#define VIA_IGA1_FIFO_THRESHOLD_FORMULA(x)				(x / 4)
#define VIA_IGA1_DISPLAY_QUEUE_EXPIRE_NUM_FORMULA(x)	(x / 4)
#define VIA_IGA1_FIFO_HIGH_THRESHOLD_FORMULA(x)			(x / 4)

/*******************************************************/
/*Definition IGA2 Design Method of FIFO Registers*/
/*******************************************************/
#define VIA_IGA2_FIFO_DEPTH_SELECT_FORMULA(x)			((x / 2 / 4) - 1)
#define VIA_IGA2_FIFO_THRESHOLD_FORMULA(x)				(x / 4)
#define VIA_IGA2_DISPLAY_QUEUE_EXPIRE_NUM_FORMULA(x)	(x / 4)
#define VIA_IGA2_FIFO_HIGH_THRESHOLD_FORMULA(x)			(x / 4)

/*******************************************************/
/*Definition IGA1 Design Method of CRTC Pixel timing Registers*/
/*******************************************************/
#define VIA_IGA1_PIXELTIMING_HOR_TOTAL_FORMULA(x)		((x) - 1)
#define VIA_IGA1_PIXELTIMING_HOR_ADDR_FORMULA(x)		((x) - 1)
#define VIA_IGA1_PIXELTIMING_HOR_BLANK_START_FORMULA(x)	((x) - 1)
#define VIA_IGA1_PIXELTIMING_HOR_BLANK_END_FORMULA(x)	((x) - 1)
#define VIA_IGA1_PIXELTIMING_HOR_SYNC_START_FORMULA(x)	((x) - 1)
#define VIA_IGA1_PIXELTIMING_HOR_SYNC_END_FORMULA(x)	((x) - 1)

#define VIA_IGA1_PIXELTIMING_VER_TOTAL_FORMULA(x)		((x) - 2)
#define VIA_IGA1_PIXELTIMING_VER_ADDR_FORMULA(x)		((x) - 1)
#define VIA_IGA1_PIXELTIMING_VER_BLANK_START_FORMULA(x)	((x) - 1)
#define VIA_IGA1_PIXELTIMING_VER_BLANK_END_FORMULA(x)	((x) - 1)
#define VIA_IGA1_PIXELTIMING_VER_SYNC_START_FORMULA(x)	((x) - 1)
#define VIA_IGA1_PIXELTIMING_VER_SYNC_END_FORMULA(x)	((x) - 1)

#define VIA_IGA1_PIXELTIMING_HVSYNC_OFFSET_END_FORMULA(x, y)  \
	(((x) / 2) - 1 - (x - y))

/*******************************************************/
/*Definition IGA2 Scaling Factor Registers                       */
/*******************************************************/
#define LCD_HOR_SCALE_FACTOR_FORMULA(x, y)	(((x - 1) * 4096) / (y - 1))
#define LCD_VER_SCALE_FACTOR_FORMULA(x, y)	(((x - 1) * 2048) / (y - 1))


/*******************************************************/
/* Cacalate PLL */
/*******************************************************/
#define VIA_SORT_BY_PHASE_MARGIN	1
#define VIA_SORT_BY_F_REF_BW		2
#define VIA_SORT_BY_PIXEL_CLOCK	3

#define STD_SR_NUM	0x04
#define STD_GR_NUM	0x09
#define STD_AR_NUM	0x14

/* Bits per pixel */
#define BPP8		8
#define BPP16	16
#define BPP32	32


/* IGA1 */
/* IGA1 Horizontal Total Register */
struct  iga1_hor_total_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_HOR_TOTAL_REG_NUM];
};

/* VIA_IGA1 Horizontal Addressable Register */
struct  iga1_hor_addr_regs {
	u8  reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_HOR_ADDR_REG_NUM];
};

/* IGA1 Horizontal Blank Start Register */
struct  iga1_hor_blank_start_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_HOR_BLANK_START_REG_NUM];
};

/* IGA1 Horizontal Blank End Register */
struct iga1_hor_blank_end_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_HOR_BLANK_END_REG_NUM];
};

/* IGA1 Horizontal Sync Start Register */
struct iga1_hor_sync_start_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_HOR_SYNC_START_REG_NUM];
};

/* IGA1 Horizontal Sync End Register */
struct iga1_hor_sync_end_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_HOR_SYNC_END_REG_NUM];
};

/* IGA1 Vertical Total Register */
struct iga1_ver_total_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_VER_TOTAL_REG_NUM];
};

/* IGA1 Vertical Addressable Register */
struct iga1_ver_addr_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_VER_ADDR_REG_NUM];
};

/* IGA1 Vertical Blank Start Register */
struct iga1_ver_blank_start_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_VER_BLANK_START_REG_NUM];
};

/* IGA1 Vertical Blank End Register */
struct iga1_ver_blank_end_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_VER_BLANK_END_REG_NUM];
};

/* IGA1 Vertical Sync Start Register */
struct iga1_ver_sync_start_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_VER_SYNC_START_REG_NUM];
};

/* IGA1 Vertical Sync End Register */
struct iga1_ver_sync_end_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_VER_SYNC_END_REG_NUM];
};

/* IGA2 */
/* VIA_IGA2 Horizontal Total Register */
struct iga2_hor_total_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_HOR_TOTAL_REG_NUM];
};

/* IGA2 Horizontal Addressable Register */
struct iga2_hor_addr_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_HOR_ADDR_REG_NUM];
};

/* IGA2 Horizontal Blank Start Register */
struct iga2_hor_blank_start_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_HOR_BLANK_START_REG_NUM];
};

/* IGA2 Horizontal Blank End Register */
struct iga2_hor_blank_end_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_HOR_BLANK_END_REG_NUM];
};

/* IGA2 Horizontal Sync Start Register */
struct iga2_hor_sync_start_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_HOR_SYNC_START_REG_NUM];
};

/* IGA2 Horizontal Sync End Register */
struct iga2_hor_sync_end_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_HOR_SYNC_END_REG_NUM];
};

/* IGA2 Vertical Total Register */
struct iga2_ver_total_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_VER_TOTAL_REG_NUM];
};

/* IGA2 Vertical Addressable Register */
struct iga2_ver_addr_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_VER_ADDR_REG_NUM];
};

/* IGA2 Vertical Blank Start Register */
struct iga2_ver_blank_start_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_VER_BLANK_START_REG_NUM];
};

/* IGA2 Vertical Blank End Register */
struct iga2_ver_blank_end_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_VER_BLANK_END_REG_NUM];
};

/* IGA2 Vertical Sync Start Register */
struct iga2_ver_sync_start_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_VER_SYNC_START_REG_NUM];
};

/* IGA2 Vertical Sync End Register */
struct iga2_ver_sync_end_regs {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_VER_SYNC_END_REG_NUM];
};

/* IGA1 CRTC Timing Register */
struct iga1_crtc_timing_regs {
	struct iga1_hor_total_regs	htotal;
	struct iga1_hor_addr_regs	haddr;
	struct iga1_hor_blank_start_regs	hblank_st;
	struct iga1_hor_blank_end_regs	hblank_ed;
	struct iga1_hor_sync_start_regs	hsync_st;
	struct iga1_hor_sync_end_regs	hsync_ed;
	struct iga1_ver_total_regs	vtotal;
	struct iga1_ver_addr_regs	vaddr;
	struct iga1_ver_blank_start_regs	vblank_st;
	struct iga1_ver_blank_end_regs	vblank_ed;
	struct iga1_ver_sync_start_regs	vsync_st;
	struct iga1_ver_sync_end_regs	vsync_ed;
};

/* IGA2 CRTC Timing Register */
struct iga2_crtc_timing_regs {
	struct iga2_hor_total_regs	htotal;
	struct iga2_hor_addr_regs	haddr;
	struct iga2_hor_blank_start_regs	hblank_st;
	struct iga2_hor_blank_end_regs	hblank_ed;
	struct iga2_hor_sync_start_regs	hsync_st;
	struct iga2_hor_sync_end_regs	hsync_ed;
	struct iga2_ver_total_regs	vtotal;
	struct iga2_ver_addr_regs	vaddr;
	struct iga2_ver_blank_start_regs	vblank_st;
	struct iga2_ver_blank_end_regs	vblank_ed;
	struct iga2_ver_sync_start_regs	vsync_st;
	struct iga2_ver_sync_end_regs	vsync_ed;
};

struct vpit_reg_table {
	u8  misc;
	u8  sr[STD_SR_NUM];
	u8  gr[STD_GR_NUM];
	u8  ar[STD_AR_NUM];
};

/* location: {REG_CR13,0,7},{REG_CR35,5,7}*/
#define VIA_IGA1_OFFSET_REG_NUM				2
/*location: {REG_CR66,0,7},{REG_CR67,0,1}*/
#define VIA_IGA2_OFFSET_REG_NUM				2
/*location: {REG_CR66,0,7},{REG_CR67,0,1},{REG_CR71,7}*/
#define VIA_IGA2_OFFSET_13BIT_REG_NUM		3

/* IGA Offset Register */
struct via_iga1_offset_regs {
	u8	reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_OFFSET_REG_NUM];
};

/* IGA2 Offset Register */
struct via_iga2_offset_regs {
	u8	reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_OFFSET_REG_NUM];
};

/* IGA2 13-bit Offset register */
struct via_iga2_offset_13bit_regs {
	u8	reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_OFFSET_13BIT_REG_NUM];
};

/* IGA Offset Register */
struct via_offset_regs {
	struct via_iga1_offset_regs iga1;
	struct via_iga2_offset_regs iga2;
};

/*location: {REG_SR1C,0,7},{REG_SR1D,0,1}*/
#define VIA_IGA1_FETCH_COUNT_REG_NUM                2
/*location: {REG_CR65,0,7},{REG_CR67,2,3}*/
#define VIA_IGA2_FETCH_COUNT_REG_NUM                2

/* IGA1 Fetch Count Register */
struct via_iga1_fetch_count_regs {
	u8    reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_FETCH_COUNT_REG_NUM];
};

/* IGA2 Fetch Count Register */
struct via_iga2_fetch_count_regs {
	u8 reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_FETCH_COUNT_REG_NUM];
};

/* IGA Fetch Count Register */
struct via_iga_fetch_count_regs {
	struct via_iga1_fetch_count_regs iga1;
	struct via_iga2_fetch_count_regs iga2;
};

struct via_fifo_value {
	u32  max_depth;
	u32  threshold;
	u32  high_threshold;
	u32  disp_queue_expired_num;
};

struct via_disp_fifo {
	u32  device_id;
	struct via_fifo_value iga1_fifo;
	struct via_fifo_value iga2_fifo;
};



/* Display FIFO Relation */
struct via_iga1_fifo_depth_select {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_FIFO_DEPTH_SELECT_REG_NUM];
};

struct via_iga1_fifo_threshold_select {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_FIFO_THRESHOLD_REG_NUM];
};

struct via_iga1_fifo_high_threshold_select {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA1_FIFO_HIGH_THRESHOLD_REG_NUM];
};

struct via_Iga1_display_queue_expire_num {
	u8   reg_num;
	struct via_chrome9_io_bit
		reg[VIA_IGA1_DISPLAY_QUEUE_EXPIRE_NUM_REG_NUM];
};

struct via_iga2_fifo_depth_select {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_FIFO_DEPTH_SELECT_REG_NUM];
};

struct via_iga2_fifo_fhreshold_select {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_FIFO_THRESHOLD_REG_NUM];
};

struct via_iga2_fifo_high_threshold_select {
	u8   reg_num;
	struct via_chrome9_io_bit reg[VIA_IGA2_FIFO_HIGH_THRESHOLD_REG_NUM];
};

struct via_iga2_display_queue_expire_num {
	u8   reg_num;
	struct via_chrome9_io_bit
		reg[VIA_IGA2_DISPLAY_QUEUE_EXPIRE_NUM_REG_NUM];
};

struct via_fifo_depth_select {
	struct via_iga1_fifo_depth_select iga1_fifo_depth_select_reg;
	struct via_iga2_fifo_depth_select iga2_fifo_depth_select_reg;
};

struct via_fifo_threshold_select {
	struct via_iga1_fifo_threshold_select
		iga1_fifo_threshold_select_reg;
	struct via_iga2_fifo_high_threshold_select
		iga2_fifo_threshold_select_reg;
};

struct via_fifo_high_threshold_select {
	struct via_iga1_fifo_high_threshold_select
		iga1_fifo_high_thresold_select_reg;
	struct via_iga2_fifo_high_threshold_select
		iga2_fifo_high_thresold_select_reg;
};

struct via_display_queue_expire_num {
	struct via_Iga1_display_queue_expire_num
		iga1_display_queue_expire_num_reg;
	struct via_iga2_display_queue_expire_num
		iga2_display_queue_expire_num_reg;
};



/**********************************************/
/*Lcd scaling factor register*/
/**********************************************/
/* location: {CR9F,0,1},{CR77,0,7},{CR79,4,5}*/
#define LCD_HOR_SCALING_REG_NUM  3
/* location: {CR79,3,3},{CR78,0,7},{CR79,6,7}*/
#define LCD_VER_SCALING_REG_NUM  3

struct lcd_hor_scaling_regs {
	u8 reg_num;
	struct via_chrome9_io_bit reg[LCD_HOR_SCALING_REG_NUM];
};

struct lcd_ver_scaling_regs {
	u8 reg_num;
	struct via_chrome9_io_bit reg[LCD_VER_SCALING_REG_NUM];
};

struct lcd_scaling_regs {
	struct lcd_hor_scaling_regs hor_regs;
	struct lcd_ver_scaling_regs ver_regs;
};

struct via_chrome9_crtc;
struct via_chrome9_encoder;
struct via_chrome9_connector;
struct via_chrome9_framebuffer;
#define to_via_chrome9_connector(x) \
		container_of(x, struct via_chrome9_connector, base)
#define to_via_chrome9_encoder(x) \
		container_of(x, struct via_chrome9_encoder, base)
#define to_via_chrome9_crtc(x) \
		container_of(x, struct via_chrome9_crtc, base)
#define to_via_chrome9_framebuffer(x) \
	container_of(x, struct via_chrome9_framebuffer, base)

struct via_chrome9_mode_info {
	struct via_chrome9_crtc *crtcs[2];
	struct via_chrome9_fbdev *vfbdev;
	int chip_index;

	/* HDMI borders properties */
	struct drm_property *left_border_property;
	struct drm_property *right_border_property;
	struct drm_property *top_border_property;
	struct drm_property *bottom_border_property;

};

struct via_chrome9_crtc {
	struct drm_crtc base;
	/* identify the crtc  0=>crtc1  1=>crtc2 */
	int crtc_id;
	int crtc_scale_status;
	/* identify this crtc enabled or not */
	bool enabled;
	u16 lut_r[256], lut_g[256], lut_b[256];
	struct drm_gem_object *cursor_bo;
};

enum via_chrome9_encoder_type {
	VIA_CHROME9_ENCODER_UNKNOWN = 0,
	VIA_CHROME9_INTERNAL_ENCODER,
	VIA_CHROME9_EXTERNAL_ENCODER
};

enum via_chrome9_connector_type {
	VIA_CHROME9_CONNECTOR_UNKNOWN = 0,
	VIA_CHROME9_INTERNAL_CONNECTOR,
	VIA_CHROME9_EXTERNAL_CONNECTOR
};

struct via_chrome9_encoder {
	/* encoder base */
	struct drm_encoder base;
	/* the encoder id */
	uint32_t encoder_id;
	uint32_t di_port;
	/* internal or external card*/
	enum via_chrome9_encoder_type encoder_type;
	/* this encoder support types */
	uint32_t supported_connect_types;
	/* the current active type */
	uint32_t active_connect_type;
	void *encoder_private;
};

struct via_chrome9_connector {
	/* connector base */
	struct drm_connector base;
	/* connector id */
	unsigned int connector_id;
	/* internal or external connector*/
	enum via_chrome9_connector_type connector_type;
	struct via_chrome9_i2c_chan *ddc_bus;
	struct drm_encoder *detected_encoder;
	enum drm_connector_status
		(*detect)(struct drm_connector *conn, bool force);
	void (*set_xorg_options)(void *options, int size,
		struct drm_connector *connector);
	void (*set_edid)(void *options, int size,
		struct drm_connector *connector);
	struct edid *edid;
	struct list_head list;
	void *options;
	void *connector_info;
	void *connector_private;
};

struct via_chrome9_framebuffer {
	struct drm_framebuffer base;
	struct drm_gem_object *obj;
};

struct internal_encoder_list {
	int encoder_type;
	int support_chip_id;
	char *name;
};

/* Definition Sync Polarity*/
#define NEGATIVE       1
#define POSITIVE        0

struct via_chrome9_support_mode {
	u8 name[10];
	int modeindex;
	int hactive;
	int vactive;
};

struct via_chrome9_lcd_options {
	unsigned int panelsize_h;
	unsigned int panelsize_v;
	bool dual_channel;
	bool no_dithering;
	bool center;
	bool msb;
};
struct panel_matrix {
	u32 panel_size_index;
	u32 width;
	u32 height;
};

struct via_gfx_dpa {
	/*Clock Polarity*/
	u8	clk_polarity;
	/*Clock Adjust*/
	u8	clk_adjust;
	/*DVP0 or DVP1 Clock Driving Selection*/
	u8	clk_driving_sel;
	/*DVP0 or DVP1 Data Driving Selection*/
	u8	data_driving_sel;
	bool	is_clk_polarity_used;
	bool	is_clk_adjust_used;
	bool	is_clk_driving_sel_used;
	bool	is_data_driving_sel_used;
};

struct via_lvds_dpa {
	u8	vt1636_clk_sel_st1;
	u8	vt1636_clk_sel_st2;
	bool	is_vt_1636_clk_sel_st1_used;
	bool	is_vt_1636_clk_sel_st2_used;
};

struct via_vga_options {
	bool	no_ddc_value;
	/*Clock Polarity*/
	u8	clk_polarity;
	/*Clock Adjust*/
	u8	clk_adjust;
	/*DVP0 or DVP1 Clock Driving Selection*/
	u8	clk_driving_sel;
	/*DVP0 or DVP1 Data Driving Selection*/
	u8	data_driving_sel;
	bool	is_clk_polarity_used;
	bool	is_clk_adjust_used;
	bool	is_clk_driving_sel_used;
	bool	is_data_driving_sel_used;
};

struct via_lcd_options {
	u32	physical_width;
	u32	physical_height;
	u32	panel_index;
	bool	msb;
	bool	center;
	bool	fix_on_iga1;
	bool	dual_channel;
	bool	no_dithering;
	/*Clock Polarity*/
	u8	clk_polarity;
	/*Clock Adjust*/
	u8	clk_adjust;
	/*DVP0 or DVP1 Clock Driving Selection*/
	u8	clk_driving_sel;
	/*DVP0 or DVP1 Data Driving Selection*/
	u8	data_driving_sel;
	bool	is_clk_polarity_used;
	bool	is_clk_adjust_used;
	bool	is_clk_driving_sel_used;
	bool	is_data_driving_sel_used;
	u8	vt1636_clk_sel_st1;
	u8	vt1636_clk_sel_st2;
	bool	is_vt_1636_clk_sel_st1_used;
	bool	is_vt_1636_clk_sel_st2_used;
};

struct via_dvi_options {
	bool	no_ddc_value;
	/*Clock Polarity*/
	u8	clk_polarity;
	/*Clock Adjust*/
	u8	clk_adjust;
	/*DVP0 or DVP1 Clock Driving Selection*/
	u8	clk_driving_sel;
	/*DVP0 or DVP1 Data Driving Selection*/
	u8	data_driving_sel;
	u8	ad9389_circuit_state_adjust;
	bool	is_clk_polarity_used;
	bool	is_clk_adjust_used;
	bool	is_clk_driving_sel_used;
	bool	is_data_driving_sel_used;
	bool	is_ad9389_circuit_state_adjust_used;
};

struct via_hdmi_options {
	bool	attach_all_modes;
	/*Clock Polarity*/
	u8	clk_polarity;
	/*Clock Adjust*/
	u8	clk_adjust;
	/*DVP0 or DVP1 Clock Driving Selection*/
	u8	clk_driving_sel;
	/*DVP0 or DVP1 Data Driving Selection*/
	u8	data_driving_sel;
	u8	ad9389_circuit_state_adjust;
	bool	is_clk_polarity_used;
	bool	is_clk_adjust_used;
	bool	is_clk_driving_sel_used;
	bool	is_data_driving_sel_used;
	bool	is_ad9389_circuit_state_adjust_used;

	u32 left_border;
	u32 right_border;
	u32 top_border;
	u32 bottom_border;
};

struct via_dp_options {
	bool	no_ddc_value;
};
/*******************************************************/
/*Display data structure*/
/*******************************************************/
struct display_timing {
	u16   hor_total;
	u16   hor_addr;
	u16   hor_blank_start;
	u16   hor_blank_end;
	u16   hor_sync_start;
	u16   hor_sync_end;

	u16   ver_total;
	u16   ver_addr;
	u16   ver_blank_start;
	u16   ver_blank_end;
	u16   ver_sync_start;
	u16   ver_sync_end;
};

#define LCD_MODE_TABLE		0x1

struct display_timing_table {
	u32  mode_index;
	u32  refresh_rate;
	u32  clk;
	int   hsync_polarity;
	int   vsync_polarity;
	struct display_timing timing;
};

struct via_pll_map {
	u32     clk;
	u32     pll_3_new;
	u32     pll_3_adv;
};

/* define PLL index: */
#define CLK_14_887M     14887000
#define CLK_17_857M     17857000
#define CLK_17_897M     17897000
#define CLK_22_000M     22000000
#define CLK_25_175M     25175000
#define CLK_26_719M     26719000
#define CLK_27_000M     27000000
#define CLK_27_027M     27027000
#define CLK_28_274M     28274000
#define CLK_29_201M     29201000
#define CLK_29_581M     29581000
#define CLK_31_490M     31490000
#define CLK_31_500M     31500000
#define CLK_31_728M     31728000
#define CLK_32_668M     32688000
#define CLK_33_750M     33750000
#define CLK_36_000M     36000000
#define CLK_40_000M     40000000
#define CLK_41_291M     41291000
#define CLK_43_125M     43125000
#define CLK_43_163M     43163000
#define CLK_46_000M     46000000
#define CLK_46_996M     46996000
#define CLK_47_193M     47193000
#define CLK_48_000M     48000000
#define CLK_48_875M     48875000
#define CLK_49_500M     49500000
#define CLK_52_375M     52375000
#define CLK_52_406M     52406000
#define CLK_52_977M     52977000
#define CLK_56_250M     56250000
#define CLK_60_466M     60466000
#define CLK_61_500M     61500000
#define CLK_65_000M     65000000
#define CLK_65_178M     65178000
#define CLK_67_295M     67295000
#define CLK_68_125M     68125000
#define CLK_68_179M     68179000
#define CLK_68_900M     68900000
#define CLK_69_924M     69924000
#define CLK_70_159M     70159000
#define CLK_72_000M     72000000
#define CLK_72_000M     72000000
#define CLK_74_250M     74250000
#define CLK_74_481M     74481000
#define CLK_78_750M     78750000
#define CLK_78_800M     78800000
#define CLK_79_500M     79500000
#define CLK_80_136M     80136000
#define CLK_83_375M     83375000
#define CLK_83_500M     83500000
#define CLK_83_950M     83950000
#define CLK_84_000M     84000000
#define CLK_84_750M     84750000
#define CLK_85_500M     85500000
#define CLK_85_860M     85860000
/* special device request 1366x768 output mode */
#define CLK_85_875M     85875000
#define CLK_88_750M     88750000
#define CLK_94_500M     94500000
#define CLK_95_750M     95750000
#define CLK_101_000M    101000000
#define CLK_106_500M    106500000
#define CLK_107_214M    107214000
#define CLK_108_000M    108000000
#define CLK_113_250M    113250000
#define CLK_118_840M    118840000
#define CLK_121_750M    121750000
#define CLK_122_500M    122500000
#define CLK_126_266M    126266000
#define CLK_130_250M    130250000
#define CLK_132_750M    132750000
#define CLK_133_308M    133308000
#define CLK_135_000M    135000000
#define CLK_136_700M    136700000
#define CLK_138_400M    138400000
#define CLK_146_250M    146250000
#define CLK_148_500M    148500000
#define CLK_150_340M    150340000
#define CLK_153_920M    153920000
#define CLK_154_000M    154000000
#define CLK_154_600M    154600000
#define CLK_157_500M    157500000
#define CLK_162_000M    162000000
#define CLK_172_798M    172798000
#define CLK_187_000M    187000000
#define CLK_193_250M    193250000
#define CLK_202_500M    202500000
#define CLK_204_000M    204000000
#define CLK_204_750M    204000000
#define CLK_214_750M    214750000
#define CLK_218_250M    218250000
#define CLK_218_500M    218500000
#define CLK_220_750M    220750000
#define CLK_234_000M    234000000
#define CLK_234_586M    234586000
#define CLK_245_250M    245250000
#define CLK_246_500M    246500000
#define CLK_234_763M    234763000
#define CLK_267_250M    267250000
#define CLK_297_500M    297500000
#define CLK_297_000M    297000000

/* VX800 ,VX855, VX900 PLL value */
#define PLL3NEW_14_887M  0x00519083
#define PLL3NEW_17_857M  0x00629083
#define PLL3NEW_17_897M  0x00C61403
#define PLL3NEW_25_175M  0x008B1003
#define PLL3NEW_26_719M  0x00931003
#define PLL3NEW_27_000M  0x00951003
#define PLL3NEW_27_027M  0x00771002
#define PLL3NEW_28_274M  0x009C1003
#define PLL3NEW_29_201M  0x00A21003
#define PLL3NEW_29_581M  0x00A51003
#define PLL3NEW_31_490M  0x00AE1003
#define PLL3NEW_31_500M  0x00AE1003
#define PLL3NEW_31_728M  0x00AF1003
#define PLL3NEW_32_668M  0x00B51003
#define PLL3NEW_33_750M  0x00951002
#define PLL3NEW_36_000M  0x00C81003
#define PLL3NEW_40_000M  0x006E0C03
#define PLL3NEW_41_291M  0x00710C03
#define PLL3NEW_43_125M  0x00770C03
#define PLL3NEW_43_163M  0x00770C03
#define PLL3NEW_46_000M  0x007F0C03
#define PLL3NEW_46_996M  0x00818C83
#define PLL3NEW_47_193M  0x00810C03
#define PLL3NEW_48_000M  0x00840C03
#define PLL3NEW_48_875M  0x00508C81
#define PLL3NEW_49_500M  0x00880C03
#define PLL3NEW_52_375M  0x00920C03
#define PLL3NEW_52_406M  0x00920C03
#define PLL3NEW_52_977M  0x00920C03
#define PLL3NEW_56_250M  0x009B0C03
#define PLL3NEW_60_466M  0x00A70C03
#define PLL3NEW_61_500M  0x00AA0C03
#define PLL3NEW_65_000M  0x00B40C03
#define PLL3NEW_65_178M  0x006B0C01
#define PLL3NEW_67_295M  0x00BA0C03
#define PLL3NEW_68_125M  0x00BC0C03
#define PLL3NEW_68_179M  0x00BC0C03
#define PLL3NEW_68_900M  0x00980c02
#define PLL3NEW_69_924M  0x00C10C03
#define PLL3NEW_70_159M  0x00C20C03
#define PLL3NEW_72_000M  0x009F0C02
#define PLL3NEW_74_250M  0x00A40C02
#define PLL3NEW_74_481M  0x00CE0C03
#define PLL3NEW_78_750M  0x006C0803
/* 78.800 MHz */
#define PLL3NEW_78_800M  0x006C0803
#define PLL3NEW_79_500M  0x006D0803
#define PLL3NEW_80_136M  0x006E0803
#define PLL3NEW_83_375M  0x005B0882
#define PLL3NEW_83_500M  0x00440801
#define PLL3NEW_83_950M  0x00448881
#define PLL3NEW_84_000M  0x00740803
#define PLL3NEW_84_750M  0x00450801
/* 85.909080 MHz */
#define PLL3NEW_85_500M  0x00760803
/* 85.909080 MHz */
#define PLL3NEW_85_875M  0x00760803
#define PLL3NEW_94_500M  0x00820803
#define PLL3NEW_95_750M  0x00690802
/* 106.491464 Mhz */
#define PLL3NEW_106_500M 0x00750802
#define PLL3NEW_107_214M 0x00940803
/* 108.281236 MHz */
#define PLL3NEW_108_000M 0x00770802
#define PLL3NEW_113_250M 0x009D0803
#define PLL3NEW_118_840M 0x00A40803
#define PLL3NEW_121_750M 0x00A80803
/* 122.897711MHz */
#define PLL3NEW_122_500M 0x00AA0803
#define PLL3NEW_126_266M 0x008B0802
#define PLL3NEW_132_750M 0x00B70803
#define PLL3NEW_133_308M 0x00B80803
#define PLL3NEW_135_000M 0x00BB0803
#define PLL3NEW_136_700M 0x00BD0803
#define PLL3NEW_138_400M 0x00720801
#define PLL3NEW_146_250M 0x00CA0803
#define PLL3NEW_148_500M 0X00A40802
#define PLL3NEW_153_920M 0x00540402
#define PLL3NEW_154_000M 0x006A0403
#define PLL3NEW_154_600M 0x006A0403
#define PLL3NEW_157_500M 0x006C0403
#define PLL3NEW_172_798M 0x00770403
#define PLL3NEW_162_000M 0x006F0403
#define PLL3NEW_187_000M 0x00810403
#define PLL3NEW_193_250M 0x00850403
#define PLL3NEW_202_500M 0x008C0403
#define PLL3NEW_204_000M 0x008D0403
#define PLL3NEW_204_750M 0x008D0403
#define PLL3NEW_214_750M 0x00940403
#define PLL3NEW_218_250M 0x00780402
#define PLL3NEW_218_500M 0x00970403
#define PLL3NEW_220_750M 0x00980403
#define PLL3NEW_234_000M 0x00600401
#define PLL3NEW_234_586M 0x00A20403
#define PLL3NEW_234_763M 0x00A20403
#define PLL3NEW_245_250M 0x00870402
#define PLL3NEW_246_500M 0x00888482
#define PLL3NEW_267_250M 0x00B90403
#define PLL3NEW_297_000M 0x00A40402
#define PLL3NEW_297_500M 0x00CE0403

/* VT3409, VT3410 PLL value */
#define PLL3ADV_14_887M  0x00851404
#define PLL3ADV_17_857M  0x00C81405
#define PLL3ADV_17_897M  0x00C71405
#define PLL3ADV_25_175M  0x008D1005
#define PLL3ADV_26_719M  0x00961005
#define PLL3ADV_27_000M  0x00971005
#define PLL3ADV_27_027M  0x00971005
#define PLL3ADV_28_274M  0x009E1005
#define PLL3ADV_29_201M  0x00A31005
#define PLL3ADV_29_581M  0x00A61005
#define PLL3ADV_31_490M  0x00B01005
#define PLL3ADV_31_500M  0x00B01005
#define PLL3ADV_31_728M  0x008E1004
#define PLL3ADV_32_668M  0x00921004
#define PLL3ADV_33_750M  0x00971004
#define PLL3ADV_36_000M  0x00A11004
#define PLL3ADV_40_000M  0x00700C05
#define PLL3ADV_41_291M  0x00730C05
#define PLL3ADV_43_125M  0x00780C05
#define PLL3ADV_43_163M  0x00790C05
#define PLL3ADV_46_000M  0x00810C05
#define PLL3ADV_46_996M  0x00690C04
#define PLL3ADV_47_193M  0x004F0C03
#define PLL3ADV_48_000M  0x00860C05
#define PLL3ADV_48_875M  0x00520C03
#define PLL3ADV_49_500M  0x00530C03
#define PLL3ADV_52_375M  0x00750C04
#define PLL3ADV_52_406M  0x00580C03
#define PLL3ADV_52_977M  0x00940C05
#define PLL3ADV_56_250M  0x009D0C05
#define PLL3ADV_60_466M  0x00A90C05
#define PLL3ADV_61_500M  0x00AC0C05
#define PLL3ADV_65_000M  0x006D0C03
#define PLL3ADV_65_178M  0x00B60C05
#define PLL3ADV_67_295M  0x00BC0C05
#define PLL3ADV_68_125M  0x00BE0C05
#define PLL3ADV_68_179M  0x00BE0C05
#define PLL3ADV_68_900M  0x009A0C04
#define PLL3ADV_69_924M  0x00C30C05
#define PLL3ADV_70_159M  0x00C40C05
#define PLL3ADV_72_000M  0x00A10C04
#define PLL3ADV_74_250M  0x00A60C04
#define PLL3ADV_74_481M  0x00D00C05
#define PLL3ADV_78_750M  0x006E0805
/* 78.800 MHz */
#define PLL3ADV_78_800M  0x006E0805
#define PLL3ADV_79_500M  0x006F0805
#define PLL3ADV_80_136M  0x00700805
#define PLL3ADV_83_375M  0x005D0804
#define PLL3ADV_83_500M  0x00460803
#define PLL3ADV_83_950M  0x005E0804
#define PLL3ADV_84_000M  0x005E0804
#define PLL3ADV_84_750M  0x00470803
/* 85.909080 MHz */
#define PLL3ADV_85_500M  0x00780805
/* 85.909080 MHz */
#define PLL3ADV_85_875M  0x00780805
#define PLL3ADV_94_500M  0x00840805
#define PLL3ADV_95_750M  0x006B0804
/* 106.491464 Mhz */
#define PLL3ADV_106_500M 0x00770804
/* 107.386350 MHz */
#define PLL3ADV_107_214M 0x00960805
/* 108.281236 MHz */
#define PLL3ADV_108_000M 0x00790804
#define PLL3ADV_113_250M 0x009E0805
#define PLL3ADV_118_840M 0x00A60805
#define PLL3ADV_121_750M 0x00AA0805
/* 122.897711MHz */
#define PLL3ADV_122_500M 0x00670803
#define PLL3ADV_126_266M 0x008D0804
#define PLL3ADV_132_750M 0x00B90805
#define PLL3ADV_133_308M 0x00950804
#define PLL3ADV_135_000M 0x00970804
#define PLL3ADV_136_700M 0x00BF0805
#define PLL3ADV_138_400M 0x00740803
#define PLL3ADV_146_250M 0x00CC0805
#define PLL3ADV_148_500M 0x00A60804
#define PLL3ADV_153_920M 0x00560404
#define PLL3ADV_154_000M 0x006C0405
#define PLL3ADV_154_600M 0x006C0405
#define PLL3ADV_157_500M 0x006E0405
#define PLL3ADV_172_798M 0x00790405
#define PLL3ADV_162_000M 0x00710405
#define PLL3ADV_187_000M 0x00830405
#define PLL3ADV_193_250M 0x00870405
#define PLL3ADV_202_500M 0x00710404
#define PLL3ADV_204_000M 0x008F0405
#define PLL3ADV_204_750M 0x008F0405
#define PLL3ADV_214_750M 0x00960405
#define PLL3ADV_218_250M 0x007A0404
#define PLL3ADV_218_500M 0x00990405
#define PLL3ADV_220_750M 0x009A0405
#define PLL3ADV_234_000M 0x00620403
#define PLL3ADV_234_586M 0x00A40405
#define PLL3ADV_234_763M 0x00A40405
#define PLL3ADV_245_250M 0x00890404
#define PLL3ADV_246_500M 0x00AD0405
#define PLL3ADV_267_250M 0x00BB0405
#define PLL3ADV_297_000M 0x00A60404
#define PLL3ADV_297_500M 0x00D00405

/* Generate a index according to width and height */
#define     VIA_MAKE_ID(w, h)	(((h) << 16) | (w))

/* via index for tv and lcd use */
#define     VIA_480X640		VIA_MAKE_ID(480, 640)
#define     VIA_640X480		VIA_MAKE_ID(640, 480)
#define     VIA_800X600		VIA_MAKE_ID(800, 600)
#define     VIA_800X480_GEN	VIA_MAKE_ID(800, 480)
#define     VIA_800X480		VIA_MAKE_ID(800, 480)
#define     VIA_1024X600		VIA_MAKE_ID(1024, 600)
#define     VIA_1024X768		VIA_MAKE_ID(1024, 768)
#define     VIA_848X480		VIA_MAKE_ID(848, 480)
#define     VIA_720X480		VIA_MAKE_ID(720, 480)
#define     VIA_720X576		VIA_MAKE_ID(720, 576)
#define     VIA_1280X720		VIA_MAKE_ID(1280, 720)
#define     VIA_1280X768		VIA_MAKE_ID(1280, 768)
#define     VIA_1280X800		VIA_MAKE_ID(1280, 800)
#define     VIA_1280X1024		VIA_MAKE_ID(1280, 1024)
#define     VIA_1360X768		VIA_MAKE_ID(1360, 768)
#define     VIA_1366X768		VIA_MAKE_ID(1366, 768)
#define     VIA_1440X900		VIA_MAKE_ID(1440, 900)
#define     VIA_1400X1050		VIA_MAKE_ID(1400, 1050)
#define     VIA_1600X1200		VIA_MAKE_ID(1600, 1200)
#define     VIA_1920X1080		VIA_MAKE_ID(1920, 1080)
#define     VIA_INVALID                 0

/* Definition Refresh Rate */
#define REFRESH_43	43
#define REFRESH_47	47
#define REFRESH_50	50
#define REFRESH_60	60
#define REFRESH_70	70
#define REFRESH_75	75
#define REFRESH_85	85
#define REFRESH_90	90
#define REFRESH_100	100
#define REFRESH_120	120
#define REFRESH_160	160

#define DCLK_14_887M     14887
#define DCLK_17_857M     17857
#define DCLK_17_897M     17897
#define DCLK_22_000M     22000
#define DCLK_25_175M     25175
#define DCLK_25_200M     25200
#define DCLK_26_719M     26719
#define DCLK_27_000M     27000
#define DCLK_27_027M     27027
#define DCLK_28_274M     28274
#define DCLK_29_201M     29201
#define DCLK_29_581M     29581
#define DCLK_31_490M     31490
#define DCLK_31_500M     31500
#define DCLK_31_728M     31728
#define DCLK_32_668M     32688
#define DCLK_33_750M     33750
#define DCLK_36_000M     36000
#define DCLK_40_000M     40000
#define DCLK_41_291M     41291
#define DCLK_43_125M     43125
#define DCLK_43_163M     43163
#define DCLK_46_000M     46000
#define DCLK_46_996M     46996
#define DCLK_47_193M     47193
#define DCLK_48_000M     48000
#define DCLK_48_875M     48875
#define DCLK_49_500M     49500
#define DCLK_52_375M     52375
#define DCLK_52_406M     52406
#define DCLK_52_977M     52977
#define DCLK_54_000M     54000
#define DCLK_56_250M     56250
#define DCLK_60_466M     60466
#define DCLK_61_500M     61500
#define DCLK_65_000M     65000
#define DCLK_65_178M     65178
#define DCLK_67_295M     67295
#define DCLK_68_125M     68125
#define DCLK_68_179M     68179
#define DCLK_69_924M     69924
#define DCLK_70_159M     70159
#define DCLK_72_000M     72000
/* #define DCLK_72_300M      72300 */
#define DCLK_74_250M     74250
/* #define DCLK_74_270M      74270 */
#define DCLK_74_481M     74481
#define DCLK_78_750M     78750
#define DCLK_78_800M     78800
#define DCLK_79_500M     79500
#define DCLK_80_136M     80136
#define DCLK_83_375M     83375
#define DCLK_83_500M     83500
#define DCLK_83_950M     83950
#define DCLK_84_000M     84000
#define DCLK_84_750M     84750
#define DCLK_85_500M     85500
#define DCLK_85_860M     85860
/* SAMSUNG request 1366x768 output mode */
#define DCLK_85_875M     85875
#define DCLK_88_750M     88750
#define DCLK_94_500M     94500
#define DCLK_95_750M     95750
#define DCLK_101_000M    101000
#define DCLK_106_500M    106500
#define DCLK_107_214M    107214
#define DCLK_108_000M    108000
#define DCLK_113_250M    113250
#define DCLK_118_840M    118840
#define DCLK_121_750M    121750
#define DCLK_122_500M    122500
#define DCLK_126_266M    126266
#define DCLK_130_250M    130250
#define DCLK_132_750M    132750
#define DCLK_133_308M    133308
#define DCLK_135_000M    135000
#define DCLK_136_700M    136700
#define DCLK_138_400M    138400
#define DCLK_146_250M    146250
#define DCLK_148_500M    148500
#define DCLK_150_340M    150340
#define DCLK_153_920M    153920
#define DCLK_154_600M    154600
#define DCLK_157_500M    157500
#define DCLK_162_000M    162000
#define DCLK_172_798M    172798
#define DCLK_187_000M    187000
#define DCLK_193_250M    193250
#define DCLK_202_500M    202500
#define DCLK_204_000M    204000
#define DCLK_204_750M    204000
#define DCLK_214_750M    214750
#define DCLK_218_250M    218250
#define DCLK_218_500M    218500
#define DCLK_220_750M    220750
#define DCLK_234_000M    234000
#define DCLK_234_586M    234586
#define DCLK_245_250M    245250
#define DCLK_246_500M    246500
#define DCLK_234_763M    234763
#define DCLK_267_250M    267250
#define DCLK_297_500M    297500
#define DCLK_297_000M    297000

static inline void *via_chrome9_connector_get_private(
		const struct via_chrome9_connector *connector)
{
	return connector->connector_private;
}

static inline void via_chrome9_connector_set_private(
		struct via_chrome9_connector *connector, void *private)
{
	connector->connector_private = private;
}

static inline bool via_chrome9_is_internal_connector(
		struct via_chrome9_connector *connector)
{
	return connector->connector_type == VIA_CHROME9_INTERNAL_CONNECTOR;
}

/* global */
extern int via_chrome9_mode_set_init(struct drm_device *dev);
extern void via_chrome9_mode_set_fini(struct drm_device *dev);
extern int via_chrome9_suspend_mode_set(struct drm_device *dev);
extern int via_chrome9_resume_mode_set(struct drm_device *dev);
/* crtc */
extern void via_chrome9_crtc_init(struct drm_device *dev, int index);
extern int via_chrome9_crtc_set_base(struct drm_crtc *crtc, int x, int y,
		struct drm_framebuffer *fb);
extern int via_chrome9_crtc_do_set_base(struct drm_crtc *crtc,
		struct drm_framebuffer *fb, int x, int y, int atomic);

/* connector */
extern int via_chrome9_detect_enc_conn(struct drm_device *dev);
extern void via_chrome9_add_connector(struct drm_device *dev,
	int connector_id, enum via_chrome9_connector_type type);
extern int
via_chrome9_ddc_get_modes(struct via_chrome9_connector *connector);

/* encoder */
extern int via_chrome9_internal_encoder_init(struct drm_device *dev);

/* dpa setting */
extern void via_chrome9_digital_port_pad_on(u32 di_port, int iga_path,
		int chipset);
extern void via_chrome9_digital_port_pad_off(u32 di_port);
extern void via_chrome9_set_digital_port_sync_polarity(u32 port,
		struct drm_display_mode *mode);
extern bool via_chrome9_detect_device_by_read_edid(
		struct drm_connector *connector, u8 device_type);
extern int via_vga_get_modes(struct drm_connector *connector);
extern int via_vga_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode);
extern int via_lcd_get_modes(struct drm_connector *connector);
extern int via_lcd_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode);
extern int via_dvi_get_modes(struct drm_connector *connector);
extern int via_dvi_mode_valid(struct drm_connector *connector,
		struct drm_display_mode *mode);
extern void via_chrome9_prune_un_preferred_modes(struct drm_device *dev,
		struct list_head *mode_list);
extern void via_chrome9_prune_un_cea_modes(struct drm_device *dev,
		struct list_head *mode_list);
extern int via_chrome9_add_cea_mode_line(struct drm_connector *connector,
		u8 *ext_edid);
extern void via_chrome9_load_crtc_pixel_timing(struct drm_display_mode *mode);
extern void via_chrome9_set_vclk(struct drm_crtc *crtc, u32 pll_value);
extern bool via_chrome9_lvds_mode_fixup(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode);
extern bool via_hdmi_mode_fixup(struct drm_encoder *encoder,
		struct drm_display_mode *mode,
		struct drm_display_mode *adjusted_mode);
extern void via_chrome9_get_monitor_physical_size(
		struct drm_connector *connector,
		u32 *width, u32 *height);
extern struct drm_encoder *via_chrome9_best_single_encoder(
		struct drm_connector *connector);
#endif
