/*
 * Copyright 1998-2008 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2008 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, Free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef _VIAMPG_H
#define _VIAMPG_H

#ifdef	__cplusplus
extern "C" {
#endif

#include "vmi_types.h"

typedef struct _PicEntry_H264 
{
	union 
	{
		struct 
		{
			unsigned char   Index7Bits      : 7;  /*reference picture list index*/
			unsigned char  BotFieldFlag   : 1; /* 0:this picture is top field  1:this picture is bottom field*/
		};
		unsigned char  PicEntry;
	};
} PicEntry_H264, *LPPicEntry_H264;  /* 1 byte */
typedef struct _DXVA_PicEntry_H264 {
	union {
		struct {
			unsigned char  Index7Bits      : 7;
			unsigned char  AssociatedFlag  : 1;
		};
		unsigned char  bPicEntry;
	};
} DXVA_PicEntry_H264, *LPDXVA_PicEntry_H264;  /* 1 byte */

typedef struct _VIAMCSURFACE
{
	/*
	 *  First part isn't used currently, will be removed later.
	 */
	unsigned long dwSize;                   /* Size of this structure */
	unsigned long dwVersion;                /* Interface Version */
	unsigned long pDevice;                  /* Useless */
	void * BeginPicture;                        /* Pointer to VIA_BeginPicture */
	void * EndPicture;                          /* Pointer to VIA_Endicture */
	void * SetMacroBlock;                       /* Pointer to VIA_SetMacroBlock */
	void * SliceReceiveData;                    /* Pointer to VIA_SliceReceiveData */
	void * DriverProc;
	/* unsigned long dwCompose; */
	void * DisplayControl;
	void * SubPicture;
	void * SetDeInterlaceMode;                  /* Pointer to VIA_SetDeinterlaceMode */
	unsigned long dwState;                  /* Current state of MPG  */
	unsigned long dwBufferNumber;           /* Total no. of decoder buffer created. (Always 4 ) */
	Bool  bInitialized;                         /* Except dwState */
	unsigned long dwReserved[8];            /* For future expansion */

	/*
	 * Client fills out the following values before calling BeginPicture
	 */
	unsigned long dwTaskType;                /* Null, Decode or Display */

	/* Decode Picture */
	unsigned long dwPictureType;             /* I, P or B pictures */
	unsigned long dwDecodePictStruct;        /* Top Field, Bottom Field or Frame */
	unsigned long dwDecodeBuffIndex;         /* Decode buffer Index. 0 to ( dwBufferNumber -1) */
	unsigned long dwAlternateScan;           /* Zig-Zag Scan or Alternative Scan */
	unsigned int ForwardRefPictIsInterlace; /* Is forward reference picture interlace? 1: interlace, 0: non-interlace */
	unsigned int BackwardRefPictIsInterlace; /* Is backward reference picture interlace? 1: interlace, 0: non-interlace */

	/* Display picture */
	unsigned long dwDisplayPictStruct;       /* Top Field, Bottom Field or Frame */
	unsigned long dwDisplayBuffIndex;        /* Display buffer Index. 0 to ( dwBufferNumber -1) */
	unsigned long dwFrameRate;               /* Frame rate  multiplied by 1000 ( 29.97=29700 ) */
	unsigned long dwDeinterlaceMode;         /* BOB / Weave */
	unsigned long dwQMatrix[2][16];          /* Quantiser Matrix, 0:intra, 1:non_intra */
	unsigned long dwQMatrixChanged;          /* If The Quantiser Matrix is changed?        */
	unsigned long dwMBwidth;                 /* Macroblock Width                                           */
	/* unsigned long dwV_size_g2800; */      /* Decide if v_size is greater than 2800      */
	unsigned long dwMpeg2;                   /* Decide if it is a mpeg2 slice stream       */
	unsigned long dwTopFirst;
	unsigned long dwFramePredDct;
	unsigned long dwMBAmax;                  /* It is equal to mb_width * mb_height; */
	unsigned long dwIntravlc;
	unsigned long dwDcPrec;
	unsigned long dwQscaleType;
	unsigned long dwConcealMV;
	/* unsigned long dwSliceFirst; */
	unsigned long dwSecondField;
	unsigned long dwOldRefFrame;             /* Fordward reference */
	unsigned long dwRefFrame;                /* Backward reference */
	unsigned long BVMVRange;                 /* Backward Vertical Motion Vector Range   */
	unsigned long BHMVRange;                 /* Backward Horizontal Motion Vector Range */
	unsigned long FVMVRange;                 /* Fordward Vertical Motion Vector Range   */
	unsigned long FHMVRange;                 /* Fordward Horizontal Motion Vector Range */
	unsigned long dwMPEGProgressiveMode;     /* default value : VIA_PROGRESSIVE */
	int   framenum;
	unsigned long dwDecoderMode;             /* indicate H/W mpeg decoder mode: USE_MC, USE_IDCT, USE_SLICE */
	unsigned long dwMPEG4Decode;             /* Indicate to use the MPEG4 decoder */
	unsigned long dwWMV9Decode;              /* Indicate to use the WMV9 decoder */


	unsigned long dwWVC1Decode;              /* Indicate to use the WVC1 decoder */

	unsigned long dwWMV9Rounding;            /* Indicate to use the WMV9's rounding control */ 
	unsigned long dwLumaFilterMode;	         /* 0:bi-linear, 1:bi-cubic */
	unsigned long dwPQuant;                  /* WMV9 Picture Quantizer Scale */	
	unsigned long dwOverlapTSF;              /* Indicate to enable overlay transform filter */	
	unsigned long dwIntensityComp;           /* Indicate to enable intensity compensation */	
	unsigned long dwLumScale;                /* for intensity compensation */
	unsigned long dwLumShift;	             /* for intensity compensation */

	/* H264 */
	/* Picture parameter */
	/*unsigned long dwRefPicAddress[16]; */
	unsigned long dwH264Decode;              /* Indicate to use the H264 decoder */
	unsigned int wScalingLists4x4[6][16];
	unsigned int wScalingLists8x8[2][64];
	unsigned char bWeighted_pred_flag;
	unsigned char bWeighted_bipred_idc;
	unsigned char bDefaultScalingList;
	unsigned char bChromaFormat;
	/*added for H264 VLD decoder*/
	unsigned char bDirect8x8InferenceFlag;
	unsigned char bLog2_max_poc_lsb;  /*= H264 spec log2_max_pic_order_cnt_lsb_minus4*/
	unsigned char bPic_order_cnt_type;
	unsigned char bCabacFlag;
	unsigned char bDeblocking_filter_present;
	char bChroma_qp_index_offset[2];
	unsigned char bConstrained_intra_pred;
	unsigned char bTransform_8x8_mode;
	/* Slice Information */
	/* unsigned char ReservedBits0[4]; */
	/* unsigned int ReservedBits1[4][64]; */
	unsigned int wRefIdxNum[2];              /* [0]:Fordward, [1]:Backward */
	int dwCurFieldOrderCnt[2]; 
	//unsigned long dwRefFieldOrderCnt[16][2];  /*not used now*/
	int dwCurrPOC;
	unsigned char bSliceType;                /* 0:P, 1:B, 2:I */
	unsigned char bFrameMBsOnlyFlag; /*frame_mbs_only_flag*/
	unsigned char bMbaffFlag;                  /*mb_adaptive_frame_field_flag;*/
	unsigned char bFieldPicFlag;             /*field_pic_flag*/
	unsigned char bBottomFieldFlag;     /* top/bottom flag*/
	unsigned char bDisableDeblock;
	PicEntry_H264  bRefList[2][32];
	//unsigned char bRefFieldTerm[16][2];    /*not used now*/
	unsigned long dwRreserved[6];           /* for future expansion */
	/*added for H264 VLD decoder*/
	unsigned long bDirect_spatial_mv_pred;
	unsigned long  dwFirstMBInSlice;       /*for H 264 multi slice*/
	Bool NewPictureFlag;                      /*for H 264 multi slice*/
	int Cabac_init_idc;
	int Slice_alpha_c0_offset_div2;
	int Slice_beta_offset_div2;
	int SliceQPDelta;
	int QScale;               /*QPy*/ 
	unsigned char bRefLongTermList[2][32]; 
	unsigned char bRefShortTermList[2][32];  
	int  bWeightedLogDenom [2];    /* [] Range: 0~7 */
	int wWeightedFactor[2][3][32]; /* Range: -128 ~ 127 */
	int wWeightedOffset[2][3][32]; /* Range: -128 ~ 127 */    
	/* added for H264 PICT Decoding */
	int profile_idc;
	short log2_max_frame_num_minus4; 
	unsigned char delta_pic_order_always_zero_flag;
	unsigned char num_ref_frames;
	unsigned char pic_order_present_flag;
	short pic_init_qp_minus26;
	unsigned char redundant_pic_cnt_present_flag; 
	unsigned char  num_ref_idx_l0_active_minus1;
	unsigned char  num_ref_idx_l1_active_minus1;


	//add for VC1/WMV9 picture level
	//refer to vdpau, most flags are in seq/enrtypoint/metadata

	//picture information
	/** I=0, P=1, B=3, BI=4  from 7.1.1.4. */
	//unsigned char  picture_type; //in dwPictureType
	/** Progressive=0, Frame-interlace=2, Field-interlace=3; see VC-1 7.1.1.15*/
	unsigned char  frame_coding_mode;

	/* sequence/entrypoint field */
	unsigned char postprocflag;
	unsigned char pulldown;
	unsigned char interlace;
	unsigned char tfcntrflag;
	unsigned char finterpflag;
	unsigned char psf;
	unsigned char dquant;
	unsigned char panscan_flag;
	unsigned char refdist_flag;
	unsigned char quantizer;
	unsigned char extended_mv;
	unsigned char extended_dmv;
	unsigned char overlap;
	unsigned char vstransform;
	unsigned char loopfilter;
	unsigned char fastuvmc;
	unsigned char range_mapy_flag;
	unsigned char range_mapy;
	unsigned char range_mapuv_flag;
	unsigned char range_mapuv;
	unsigned char fourMV;
	unsigned char quarter_sample;

	/* only for simple/main profile metadata */
	unsigned char multires;
	unsigned char syncmarker;
	unsigned char rangered;
	unsigned char maxbframes;
	unsigned char deblockEnable;   
	unsigned char pquant;

	/* add for h264 on vt3410 */
	unsigned char residual_colour_transform_flag;
	unsigned char sp_for_switch_flag;
	unsigned char RefPicFlag; 
	unsigned char MbsConsecutiveFlag;
	unsigned char MinLumaBipredSize8x8Flag; 
	unsigned char IntraPicFlag;
	unsigned char bit_depth_luma_minus8;	
	unsigned char bit_depth_chroma_minus8;
	DXVA_PicEntry_H264 RefFrameList[16];
	int FieldOrderCntList[16][2];
	unsigned short FrameNumList[16];
	int UsedForReferenceFlags;
	unsigned short ExistingFrameFlags;
	unsigned char num_slice_groups_minus1;
	unsigned char slice_group_map_type;
	char pic_init_qs_minus26;
	unsigned short slice_group_change_rate_minus1;
	unsigned char SliceGroupMap[810];
} VIAMPGSURFACE, * LPVIAMPGSURFACE;

/* definition for dwDecoderMode */
#define USE_SLICE                 0
#define USE_MC                    1
#define USE_MC_IDCT               2
#define USE_PICTURE             3

/* dwTaskType of VIAMPGSURFACE */
#define VIA_TASK_NULL              0x00000000 /* Used only to check state and version */
#define VIA_TASK_DECODE            0x00000001 /* Decode new Picture also Display decoded picture */
#define VIA_TASK_DISPLAY           0x00000002 /* Display only */

/* dwPictureType of VIAMPGSURFACE */
#define VIA_PIC_TYPE_I              0x00000001
#define VIA_PIC_TYPE_P              0x00000002
#define VIA_PIC_TYPE_B              0x00000003
#define VIA_PIC_TYPE_BI             0x00000004

/* dwDecodePictStruct and dwDisplayPictStruct of VIAMPGSURFACE */
#define VIA_PICT_STRUCT_TOP                 0x00000001
#define VIA_PICT_STRUCT_BOTTOM              0x00000002
#define VIA_PICT_STRUCT_FRAME               0x00000003
#define VIA_PICT_STRUCT_MBAFF               0x00000004

/* dwAltScan */
#define VIA_ALTERNATE_SCAN                  0x1
#define VIA_ZIG_ZAG_SCAN                    0x0

/* dwDeinterlaceMode of VIAMPGSURFACE */
#define VIA_DEINTERLACE_WEAVE               0x00000000
#define VIA_DEINTERLACE_BOB                 0x00000001
#define VIA_DEINTERLACE_MEDIAN              0x00000002


/* dwMPEGProgressiveMode of VIAMPGSURFACE */
#define VIA_NON_PROGRESSIVE                 0x00000000
#define VIA_PROGRESSIVE                     0x00000010

typedef enum _MB_TYPE{
	MB_TYPE_INTER_16X16 = 0,
	MB_TYPE_INTER_16X8,
	MB_TYPE_INTER_8X16,
	MB_TYPE_INTER_8X8,
	MB_TYPE_MB_SUB,
	MB_TYPE_INTRA_16X16,
	MB_TYPE_INTRA_8X8,
	MB_TYPE_INTRA_4X4,
	MB_TYPE_MB_IPCM,
} MB_TYPE, *LPMB_TYPE;

typedef struct tag_VIAH264MBTYPE
{
	unsigned short MBType:           4;
	unsigned short Reserved0:        4;
	unsigned short FieldMB:          1;
	unsigned short BottomMB:         1;
	unsigned short HostResidDiff:    1;
	unsigned short TransformSize8x8: 1;
	unsigned short Reserved1:        4;
} VIAH264MBTYPE;

/*
   typedef struct tag_VIAH264_MBSNL
   {
   unsigned long MBdataLocation: 24;
   unsigned long MBskipFollowing: 8;
   } VIAH264MB_SNL;
   */

typedef struct tag_VIAH264MBAVAILABLE
{
	unsigned char MBAddrA_UpperBlock:  1;
	unsigned char MBAddrA_LowerBlock:  1;
	unsigned char MBAddrB:             1;
	unsigned char MBAddrC:             1;
	unsigned char MBAddrD:             1;
	unsigned char MBAddrB_FieldMB:     1;
	unsigned char MBAddrB_BottomField: 1;
	unsigned char Reserved:            1;
} VIAH264NEIGHBORMB;

typedef union tag_VIAH264CBP
{
	struct
	{
		unsigned short BlockY0:   1;
		unsigned short BlockY1:   1;
		unsigned short BlockY2:   1;
		unsigned short BlockY3:   1;
		unsigned short BlockCb:   1; /* In MC Path is BlockCb, In IDCT Path is ChromaDC */
		unsigned short BlockCr:   1; /* In MC Path is BlockCr, In IDCT Path is ChromaAC */
		unsigned short Reserved: 10;
	};
	unsigned short wCBPValue;
} VIAH264CBP;
/*
   typedef union tag_VIAQUANPARAM
   {
   struct
   {
   unsigned char        bReserveBits1;
   unsigned char        bQuanParam[3];
   };
   unsigned char            bReservedBits2[4];
   }VIAQUANPARAM;
   */
typedef struct tag_MVECTOR
{
	unsigned short           horz;
	unsigned short           vert;
}VIAMVECTOR;

typedef union tag_REFPICSELECT
{
	struct
	{
		unsigned char RefBufSelection:   5;
		/*unsigned char Reserved:          1; */
		unsigned char RefFieldMB:        1;
		unsigned char RefStructure:      1;
		unsigned char Reserved:          1;  /* jimmy */
	}RegUchar;

	unsigned char uchar;    
}VIAREFPICSELECT;


/**************************************************************************
  VIA Macro Block Structure
 ***************************************************************************/

/*  
 *      store the IDCT coefficient values.
 */
typedef struct _IDCTCoef4Group {
	unsigned char  CoefIndex[4];
	short CoefValue[4];
} IDCTCoef4Group, *LPIDCTCoef4Group;

/*add for H264 VLD*/
typedef struct  _VIASLICEDATA
{
	unsigned long   size_in_bit;
	unsigned long   bit_offset;
	unsigned char *pbuffer;
	unsigned char *pbuffer_end;
}VIASLICEDATA,*LPVIASLICEDATA;

typedef struct  _VIAMACROBLOCK {
	unsigned long dwMBLKInfo;       /* Codedblocks, DCT type, Predition direction,    */
	/* Prediction mode and Motion Vectors information */
	/* of current Macro Block                         */
	unsigned long dwMBLKHOffset;    /* Macro Block horizontal offset                  */
	unsigned long dwMBLKVOffset;    /* Macro Block vertical offset                    */
	unsigned long dwMV[32];         /* Motion Vectors in half pixel resolution 	*/
	/* and based on the current Bacro Block's 	*/
	/* upper left corner.                                   */
	/* 1st forward motion vector or Motion Vector 0         */
	/* dwMV[0] is PMV[0][0][0] (Horizontal)                 */
	/* dwMV[1] is PMV[0][0][1] (Vertical  )                 */
	/* 2nd forward motion vector or Motion Vector 1         */
	/* dwMV[2] is PMV[0][1][0] (Horizontal)                 */
	/* dwMV[3] is PMV[0][1][1] (Vertical  )                 */
	/* 1st backward motion vector or Motion Vector 2        */
	/* dwMV[4] is PMV[1][0][0] (Horizontal)                 */
	/* dwMV[5] is PMV[1][0][1] (Vertical  )                 */
	/* 2nd backward motion vector or Motion Vector 3        */
	/* dwMV[6] is PMV[1][1][0] (Horizontal)                 */
	/* dwMV[7] is PMV[1][1][1] (Vertical  )                 */
	/* 1st forward UV motion vector or Motion Vector 4    */
	/* dwMV[8] is PMV[0][0][0] (Horizontal)                 */
	/* dwMV[9] is PMV[0][0][1] (Vertical  )                 */
	/* 3rd forward motion vector or Motion Vector 5         */
	/* dwMV[10] is PMV[0][0][0] (Horizontal)                */
	/* dwMV[11] is PMV[0][0][1] (Vertical  )                */
	/* 4th forward motion vector or Motion Vector 6         */
	/* dwMV[12] is PMV[0][1][0] (Horizontal)                */
	/* dwMV[13] is PMV[0][1][1] (Vertical  )                */
	/* 3rd backward motion vector or Motion Vector 7        */
	/* dwMV[14] is PMV[1][0][0] (Horizontal)                */
	/* dwMV[15] is PMV[1][0][1] (Vertical  )                */
	/* 4th backward motion vector or Motion Vector 8        */
	/* dwMV[16] is PMV[1][1][0] (Horizontal)                */
	/* dwMV[17] is PMV[1][1][1] (Vertical  )                */
	/* 1st backward UV motion vector or Motion Vector 9   */
	/* dwMV[18] is PMV[0][0][0] (Horizontal)                */
	/* dwMV[19] is PMV[0][0][1] (Vertical  )                */
	/* 2nd forward UV motion vector or Motion Vector 10   */
	/* dwMV[20] is PMV[0][0][0] (Horizontal)                */
	/* dwMV[21] is PMV[0][0][1] (Vertical  )                */
	/* 2nd backward UV motion vector or Motion Vector 11  */
	/* dwMV[22] is PMV[0][0][0] (Horizontal)                */
	/* dwMV[23] is PMV[0][0][1] (Vertical  )                */
	/* 3rd forward UV motion vector or Motion Vector 12   */
	/* dwMV[24] is PMV[0][0][0] (Horizontal)                */
	/* dwMV[25] is PMV[0][0][1] (Vertical  )                */
	/* 3rd backward UV motion vector or Motion Vector 13  */
	/* dwMV[26] is PMV[0][0][0] (Horizontal)                */
	/* dwMV[27] is PMV[0][0][1] (Vertical  )                */
	/* 4th forward UV motion vector or Motion Vector 14   */
	/* dwMV[28] is PMV[0][0][0] (Horizontal)                */
	/* dwMV[29] is PMV[0][0][1] (Vertical  )                */
	/* 4th backward UV motion vector or Motion Vector 15  */
	/* dwMV[30] is PMV[0][0][0] (Horizontal)                */
	/* dwMV[31] is PMV[0][0][1] (Vertical  )                */ 
	/* Select which field to reference: bit 0 to bit 3 are corresponds to */
	/* MV 1 to MV4, and value 1 means BOTTOM, 0 means TOP */
	unsigned int referenceFieldSelect;
	/* 09-01-1999 */
	/*  I restore the original design as below. */
	void * lpIDCTBuff;           /* IDCT data for current macro block                      */
	/* Client needs to allocate and maintain 768 bytes        */
	/* For INTRA Macro Block                                  */
	/* ( 64 coeffiients per block * 1 bytes per coefficient * */
	/*   6 clocks ) buffer with the following layout.         */
	/*  Buf[  0..63 ] for Y0                                  */
	/*  Buf[ 64..127] for Y1                                  */
	/*  Buf[128..191] for Y2                                  */
	/*  Buf[192..255] for Y3                                  */
	/*  Buf[256..319] for Cb                                  */
	/*  Buf[320..383] for Cr                                  */
	/* For INTER Macro Block                                  */
	/* ( 64 coeffiients per block * 2 bytes per coefficient * */
	/*   6 clocks ) buffer with the following layout.         */
	/*  Buf[  0..127] for Y0                                  */
	/*  Buf[128..255] for Y1                                  */
	/*  Buf[256..383] for Y2                                  */
	/*  Buf[384..511] for Y3                                  */
	/*  Buf[512..639] for Cb                                  */
	/*  Buf[640..767] for Cr                                  */
	LPIDCTCoef4Group lpHWIDCTBuff;        /* We fill A.A.N. IDCT coefficient in this buffer which has 256                           */
	/* bytes per block,                                                                       */
	/* 128 bytes [0,2,4,.....126] are for Run of Y0  , where 0 <= Run <= 63                   */
	/* 128 bytes [1,3,5,.....127] are for Level of Y0, where -2048   <= Level  <= 2047        */
	/* 128 bytes [128,130,132,.....254] are for Run of Y1  , where 0 <= Run <= 63             */
	/* 128 bytes [129,131,133,.....255] are for Level of Y1, where -2048   <= Level  <= 2047  */
	/* 128 bytes [256,258,260,.....382] are for Run of Y2  , where 0 <= Run <= 63             */
	/* 128 bytes [257,259,261,.....383] are for Level of Y2, where -2048   <= Level  <= 2047  */
	/* 128 bytes [384,386,388,.....510] are for Run of Y3  , where 0 <= Run <= 63             */
	/* 128 bytes [385,387,389,.....511] are for Level of Y3, where -2048   <= Level  <= 2047  */
	/* 128 bytes [512,514,516,.....638] are for Run of Cb  , where 0 <= Run <= 63             */
	/* 128 bytes [513,515,517,.....639] are for Level of Cb, where -2048   <= Level  <= 2047  */
	/* 128 bytes [640,642,644,.....766] are for Run of Cr  , where 0 <= Run <= 63             */
	/* 128 bytes [641,643,645,.....767] are for Level of Cr, where -2048   <= Level  <= 2047  */
	unsigned char  byIDCTCount[6];        /* Number of (Run, Level) pair in each block                                              */
	/* for VT3259 */
	unsigned char  bReserved[2];          /* Alignment */
	unsigned long dwVMBNum;               /* The macroblock number per column */
	unsigned long dwHMBNum;               /* The macroblock number per row */
	unsigned long dwBiDirectionRounding;  /* Bi-direction prediction rounding control */
	unsigned long dwRounding;             /* Interpolation rounding control for non-integer motion vector */
	unsigned long dwQblockSize;           /* Block size of quarter-pel MC in case of progressive MB 0:16x16 ,1:8x8 */
	unsigned long dwUseQMC;               /* Quarter-pel enable */
	unsigned long dwUseGMC;               /* GMC enable */
	unsigned long dwUse4MV;               /* 4MV enable */
	unsigned long dwScanType;             /* 0:Zig-Zag 1:Alternate Vertical 2:Alternate Horizontal */
	/* [1:0] block u0,[3:2] block v0,[5:4] block y3,[7:6] block y2,[9:8] block y1,[11:10] block y0 */
	unsigned long dwDCScalValue[6];       /* DC scal value */
	unsigned long dwDCTMaxValue[6];       /* DCT coefficients clip value */
	unsigned long dwQP[6];                /* Inverse Quantization value */
	unsigned long dwIntra[6];             /* Intra block ,1:Intra block,0:Inter block */
	unsigned long dwInLoopDeblock; /* enbale WMV9's in-loop deblock filter */
	unsigned long dwBlkType;             /* WMV9 intra/inter block for progressive mode */
	unsigned long dwSubCBP[6]; 	 
	/* 
	 *     dwSubCBP[0]: Y0 subcbp
	 *     dwSubCBP[1]: Y1 subcbp   
	 *     dwSubCBP[2]: Y2 subcbp
	 *     dwSubCBP[3]: Y3 subcbp   
	 *     dwSubCBP[4]: Cb subcbp
	 *     dwSubCBP[5]: Cr subcbp   
	 *
	 * Used BIT0~BIT1 of this flag to present sub-block type.
	 * #define WMV9_BLK _8X8             0x00   
	 * #define WMV9_BLK_8X4              0x01   
	 * #define WMV9_BLK_4X8              0x02
	 * #define WMV9_BLK_4X4              0x03              
	 * Used BIT2~BIT5 of this flag to present subcbp(D, C, B, A)
	 *    x: don't care
	 *    8x8 one block:  D  (xxxD00b)
	 *    8x4 two blocks:  C(top), D(bottom) (xxCD01b)
	 *    4x8 two blocks:  C(left), D(right) (xxCD02b)
	 *    4x4 four blocks:  A(leftTop), B(rightTop), C(leftBottom), D(rightBottom)  (ABCD11b) 
	 */ 

	unsigned long dwFastUVMC;       /* Fast U,V motion compensation flag */
	void *lpWmvHWIDCTBuff;     /* add two vars used to change the way to store wmv IDCT coeff*/
	unsigned long dwWmvIDCTCoeffCnt;
	/* unsigned char cbDeblockEdge[6]; */

	/* Use for H264 */
	unsigned short      wMBaddress;
	VIAH264MBTYPE       wMBType;    
	VIAH264CBP          wPatternCode;
	VIAH264NEIGHBORMB   bNeighborAvailable;
	unsigned char       bQuanParam[3];
	VIAMVECTOR          MVector[32];
	VIAREFPICSELECT     bRefPicSelect[8];
	unsigned char       bPredictMode[17];
	unsigned short      wDeblockEdgeControl;
	unsigned long       dwDeblockBS[5];
	unsigned char       bReservedBits1[6];
	unsigned char       bDeblockIndexA[3][5];
	unsigned char       bDeblockIndexB[3][5];

	/* Weighted Prediction Information */
	unsigned char bWeightedLogDenom [2];    /* [] Range: 0~7 */
	/* unsigned int wReservedBits; */
	unsigned int wWeightedFactor[2][3][4]; /* Range: -128 ~ 127 */
	unsigned int wWeightedOffset[2][3][4]; /* Range: -128 ~ 127 */    
	/*VIAH264MB_SNL       dwMB_SNL; */
	/*unsigned char                bReserved[6];*/
} VIAMACROBLOCK, * LPVIAMACROBLOCK;


#define MBLK_LAST_MACROBLOCK   0x00800000   /* The last macroblock in this picture */

/* dwBLKInfo of VIAMACROBLOCK */
/* Prediction Type */
#define MBLK_PREDICY_BWD       0x00000100   /* Backward Prediction */
#define MBLK_PREDICY_FWD       0x00000080   /* Forward Prediction  */
#define MBLK_PREDICY_BD        (MBLK_PREDICY_BWD | MBLK_PREDICY_FWD) /* Bidirection Prediction */
#define MBLK_PREDICY_16X8      0x00000400   /* 16X8 Prediction       */
#define MBLK_PREDICY_DP        0x00000600   /* Dual Prime Prediction */
#define MBLK_PREDICY_FRAME     0x00000400   /* Frame Prediction      */
#define MBLK_PREDICY_FIELD     0x00000200   /* Field Prediction      */

/* DCT Type */
#define MBLK_DCT_FRAME         0x00000000   /* use Frame DCT for current macroblock */
#define MBLK_DCT_FIELD         0x00000040   /* use Field DCT for current macroblock */

/* Intra/Inter Flag */
#define MBLK_TYPE_INTRA        0x00000000   /* Intra BacroBlock */
#define MBLK_TYPE_INTER        0x80000000   /* Inter BacroBlock */

/* This 2 bits are not in "Macro Block Descrition register".
 * They are in "Motion Vector 0~4 register"
 * Please do "LPVIAMACROBLOCK->dwMV[?] = LPVIAMACROBLOCK->dwMV[?] | MBLK_MV?_REF_XXX | MBLK_REF_BUFF_INDEX_0"
 * before calling MPEGSetMacroBlock
 * Reference Field
 */
#define MBLK_MV_REF_BOTTOM     0x00008000   /* Reference Bottom field */
#define MBLK_REF_BUFF_INDEX_0  0x00000000
#define MBLK_REF_BUFF_INDEX_1  0x00002000
#define MBLK_REF_BUFF_INDEX_2  0x00004000
#define MBLK_REF_BUFF_INDEX_3  0x00006000

/* CBP */
#define MBLK_CBP_Y0            0x00000001   /* Indicate if Y0 block is coded */
#define MBLK_CBP_Y1            0x00000002   /* Indicate if Y1 block is coded */
#define MBLK_CBP_Y2            0x00000004   /* Indicate if Y2 block is coded */
#define MBLK_CBP_Y3            0x00000008   /* Indicate if Y3 block is coded */
#define MBLK_CBP_Cb            0x00000010   /* Indicate if Cb block is coded */
#define MBLK_CBP_Cr            0x00000020   /* Indicate if Cr block is coded */

/* WMV9 intra/inter block for progressive mode. */
/* BIT0 ~BIT4 (dwBlkType) */ 
#define WMV9_INTRA_Y0            0x00000010   /*Indicate if Y0 block is intra coded*/
#define WMV9_INTRA_Y1            0x00000008   /*Indicate if Y1 block is intra coded*/
#define WMV9_INTRA_Y2            0x00000004   /*Indicate if Y2 block is intra coded*/
#define WMV9_INTRA_Y3            0x00000002   /*Indicate if Y3 block is intra coded*/
#define WMV9_INTRA_CbCr         0x00000001   /*Indicate if CbCr block is intra coded*/

#define WMV9_BLK_8X8              0x00   
#define WMV9_BLK_8X4              0x01   
#define WMV9_BLK_4X8              0x02
#define WMV9_BLK_4X4              0x03   

/* 8x8 one block:  A
 * 8X4 two blocks:  A(top), B(bottom)
 * 4x8 two blocks:  A(left), B(right)
 * 4x4 four blocks:  A(leftTop), B(rightTop), C(leftBottom), D(rightBottom) */
#define WMV9_SUB_CBP_A        0x00000001   // subcbp
#define WMV9_SUB_CBP_B        0x00000002   // subcbp
#define WMV9_SUB_CBP_C        0x00000004   // subcbp
#define WMV9_SUB_CBP_D        0x00000008   // subcbp



/* 09-01-1999
 * Based on discuss between me and Jay.
 * The following structure and definition are for subpicture.
 */
/**************************************************************************
  VIA SubPicture Structure
 ***************************************************************************/
typedef struct  _VIASUBPICT {
	/*
	 *  First part isn't used currently, will be removed later.
	 */
	unsigned long dwSize;                   /* Size of this structure   */
	void * SubPicture;                      /* Pointer to VIA_SubPicture */
	unsigned char * lpSubpictureBuffer[2];  /* Subpicture frame buffer 0,1 */
	unsigned long dwPitch;                  /* Subpicture frame buffer pitch */
	unsigned long dwState;                  /* Current state of H/W SubPicture Engine */
	Bool  bInitialized;                     /* Except dwState */
	unsigned long dwReserved[8];            /* For future expansion */

	/*
	 * Client fills out the following values before calling SubPicure
	 */
	unsigned long dwTaskType;                /* Null, Write or Display */

	/* Write Buffer Index */
	unsigned long dwWriteBuffIndex;          /* Write buffer Index. 0 or 1 */

	/* Display Buffer Index */
	unsigned long dwDisplayBuffIndex;        /*Decode buffer Index. 0 or 1        */
	void * lpSPBuffer;                       /*Where SubPicture BMP located       */
	unsigned long  dwSPLeft;                 /*SubPicture position in Video frame */
	unsigned long  dwSPTop;
	unsigned long  dwSPWidth;                /* SubPicture width  */
	unsigned long  dwSPHeight;               /* SubPicture height */
	unsigned long  dwRamTab[16];
	unsigned long  dwRreserved[8];           /* For future expansion */
	/* dwRreserved[0] is used for SUBPICTURE surface Pitch */
} VIASUBPICT, * LPVIASUBPICT;


/* dwTaskType of VIASUBPICTURE */
#define VIA_TASK_SP_NULL    0x00000000  /* Used only to check state                           */
#define VIA_TASK_SP_DISPLAY 0x00000002  /* Display SubPciture which                           */
/* frame Idx = dwDisplayBuffIndex                     */
/* If callback function SubPicture receive            */
/* flag = ( VIA_TASK_SP_WRITE | VIA_TASK_SP_DISPLAY ) */
/* Write and Display can be done in one function call */
#define VIA_TASK_SP_RAMTAB  0x00000006  /* Update Ram_Table(16x24) */
#define VIA_TASK_SP_DISPLAY_DISABLE   0x00000008 /* Disable Display SubPciture */
#define VIA_TASK_SP_BLEND2  0x00000010  /* for XVMC, copies the source_surface(dwWriteBuffIndex)
										 * to the target_surface(dwDisplayBuffIndex) and associates 
										 * the subpicture with the target_surface. */
#define VIA_TASK_SP_CLEAR   0x00000020  /* for XVMC, Clear the area of the given subpicture 
										 * to "color". */  



/* VMI3.0 Support*/
typedef enum
{
	eVMI_PIC_DATA,
	eVMI_PIC_DATA_H264_No_Emulation           // no emulation three bytes(0x03)

} EVMIBufType;

typedef struct {
	void const  * BitStream; /** A pointer to the bitstream data bytes */    
	unsigned long   BitStreamBytes; /** The number of data bytes */

	EVMIBufType  BufType;  	
} VMIBitstreamBuffer;

typedef struct _VMIInputBuf{ 	
	VMIBitstreamBuffer *BitStreamBuffers;

	unsigned long           BitStreamBufferCount;
}SVMIInputBuf, *PSVMIInputBuf;

typedef SVMIInputBuf  InputBuf;
typedef PSVMIInputBuf InputBufPtr;

typedef void *  PictureInfoPtr;

#ifdef	__cplusplus
	extern "C" {
#endif


#endif   /*_VIAMPG_H*/


